package asit.address;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.actions.DispatchAction;
import asit.bescheid.Bescheid;
import asit.bescheid.EckData;
import asit.control.FlowControl;
import asit.control.FlowControlException;
import asit.moa.client.deliver.address.KompPostalAddress;
import asit.state.Error;


public class PostalAddressService extends DispatchAction {

  public static Log log_ = LogFactory.getLog(PostalAddressService.class.getName());
  
  // saves the record of natural person
  public ActionForward save(ActionMapping mapping, ActionForm form, HttpServletRequest request,
      HttpServletResponse response) throws ServletException, IOException {
   
    ActionMessages errors = form.validate(mapping, request);
    
    //  check for errors
    if (errors.size() > 0) {
      
      saveErrors(request, errors);
      return new ActionForward("/postaladdress.jsp");
    }
    
    log_.debug("Entering PERSON save method.");
    log_.debug("Location: " + request.getParameter("location"));
    log_.debug("MainLocation: " + request.getParameter("mainlocation"));
    
    String backLink = request.getParameter("backlink");
    log_.debug("Backlink: " + backLink);
    int i = 0;
    int mainLocation = 0;
    try {
      i = Integer.parseInt(request.getParameter("index"));
      mainLocation = Integer.parseInt(request.getParameter("mainlocation"));
    } catch(NumberFormatException e) {
      log_.debug("Error while parsing integer: " + e.getMessage());
    }
    try {
      KompPostalAddress address = new KompPostalAddress();
      BeanUtils.copyProperties(address, form);
      log_.debug("Saving person");
      FlowControl.setPostalAddress(request.getParameter("location"), mainLocation, i, address, request);
      log_.debug("Redirecting back to backlink");
      return new ActionForward("/" + backLink);
      
    } catch (FlowControlException e) {
      Error error = new Error(e.getShortMessage(), e.getMessage());
      error.updateRequest(request);
      return new ActionForward("/error.jsp");
    } catch (Exception e) {
      Error error = new Error("Fehler beim Bearbeiten der Personendaten", e.getClass().getName() + " - " + e.getMessage());
      error.updateRequest(request);
      return new ActionForward("/error.jsp");
    }

    
  }
  
  // sends the data of natural person record to the form to be edited
  public ActionForward edit(ActionMapping mapping, ActionForm form, HttpServletRequest request,
      HttpServletResponse response) throws ServletException, IOException {

    //ActionMessages errors = form.validate(mapping, request);
    
    //  check for errors
    /*if (errors.size() > 0) {
      
      saveErrors(request, errors);
      return new ActionForward("/postaladdress.jsp");
    }*/
    
    log_.debug("Entering edit method.");

    String backLink = request.getParameter("backlink");
    log_.debug("Backlink: " + backLink);
    
    int i = 0;
    int mainLocation = 0;
    try {
      i = Integer.parseInt(request.getParameter("index"));
      mainLocation = Integer.parseInt(request.getParameter("mainlocation"));
    } catch(NumberFormatException e) {
      log_.debug("Error while parsing integer: " + e.getMessage());
    }
    try {
      KompPostalAddress address = FlowControl.getPostalAddress(request.getParameter("location"), mainLocation, i, request);
      BeanUtils.copyProperties(form, address);
      
      return new ActionForward("/postaladdress.jsp");
    } catch (FlowControlException e) {
      Error error = new Error(e.getShortMessage(), e.getMessage());
      error.updateRequest(request);
      return new ActionForward("/error.jsp");
    } catch (Exception e) {
      Error error = new Error("Fehler beim Bearbeiten der Personendaten", e.getMessage());
      error.updateRequest(request);
      return new ActionForward("/error.jsp");
    }
 }
  
  // creates a new natural person record
  public ActionForward createnew(ActionMapping mapping, ActionForm form, HttpServletRequest request,
      HttpServletResponse response) throws ServletException, IOException {

    log_.debug("Entering createnew method.");
    String backLink = request.getParameter("backlink");
    log_.debug("Backlink: " + backLink);
    try {
      KompPostalAddress address = new KompPostalAddress();
      BeanUtils.copyProperties(form, address);
      return new ActionForward("/postaladdress.jsp");
    } catch (Exception e) {
      Error error = new Error("Fehler beim Bearbeiten der Personendaten", e.getMessage());
      error.updateRequest(request);
      return new ActionForward("/error.jsp");
    }
    
 }
  
  // this method saves the eckdaten of natural person
  public ActionForward saveeckdata(ActionMapping mapping, ActionForm form, HttpServletRequest request,
      HttpServletResponse response) throws ServletException, IOException {

    // there has just to be saved the feldname field
   
    String location = request.getParameter("location");
    String index = request.getParameter("index");
    
    int idx = 0;
    try {
      idx = Integer.parseInt(index);
    } catch(NumberFormatException e) {
      log_.debug("Error while parsing integer: " + e.getMessage());
    }
    Bescheid notification = (Bescheid)request.getSession().getAttribute("Bescheid");
    try {
      EckData data = null;
      if (idx < notification.getBWeitereEckDaten().size()) {
         data = (EckData)notification.getBWeitereEckDaten().get(idx);
      } else {
        data = new KompPostalAddress();
        notification.getBWeitereEckDaten().add(data);
      }
      data.setEFeldName(request.getParameter("EFeldName"));
      
      return new ActionForward("/NotificationService.do?action=display");
    } catch(Exception e) {
      Error error = new Error("Error", e.getMessage());
      error.updateRequest(request);
      return new ActionForward("/error.jsp");
    }
  }
  
  // prepares the editing of the eckdaten of a natural person
  public ActionForward editeckdata(ActionMapping mapping, ActionForm form, HttpServletRequest request,
      HttpServletResponse response) throws ServletException, IOException {
    
    String location = request.getParameter("location");
    String index = request.getParameter("index");
    
    int idx = 0;
    try {
      idx = Integer.parseInt(index);
    } catch(NumberFormatException e) {
      log_.debug("Error while parsing integer: " + e.getMessage());
    }
    
    Bescheid notification = (Bescheid)request.getSession().getAttribute("Bescheid");
    
    int size = notification.getBWeitereEckDaten().size();
    log_.debug("Eckdaten size: " + size);
    EckData data = (EckData)notification.getBWeitereEckDaten().get(idx);
    
    log_.debug("Feldname: " + data.getEFeldName());
    
    log_.debug("PostalAddress: " + (data == null));
    try {
      log_.debug("Kopiere daten in bean");
      BeanUtils.copyProperties(form, data);
      log_.debug("Forwarde");
      return new ActionForward("/eckdataperson.jsp");
    } catch(Exception e) {
      Error error = new Error("Error", e.getMessage());
      error.updateRequest(request);
      return new ActionForward("/error.jsp");
    }

  }
  
  // prepares the editing of the eckdaten of a natural person
  public ActionForward createeckdata(ActionMapping mapping, ActionForm form, HttpServletRequest request,
      HttpServletResponse response) throws ServletException, IOException {
    
    String location = request.getParameter("location");
    String index = request.getParameter("index");
    
    int idx = 0;
    try {
      idx = Integer.parseInt(index);
    } catch(NumberFormatException e) {
      log_.debug("Error while parsing integer: " + e.getMessage());
    }
    
    Bescheid notification = (Bescheid)request.getSession().getAttribute("Bescheid");
    
    int size = notification.getBWeitereEckDaten().size();
    
    EckData data = new KompPostalAddress();
    //notification.getBWeitereEckDaten().add(data);
    
    try {
      BeanUtils.copyProperties(form, data);
      return new ActionForward("/eckdataperson.jsp");
    } catch(Exception e) {
      Error error = new Error("Error", e.getMessage());
      error.updateRequest(request);
      return new ActionForward("/error.jsp");
    }

  }
  
}