package asit.bescheid;

import java.util.ArrayList;
import javax.xml.transform.TransformerException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import asit.common.NameValuePair;
import asit.common.Utils;
import asit.moa.client.deliver.address.Adressat;
import asit.moa.client.deliver.address.PersonConstants;


public class Spruchteil implements BescheidElement {

  protected ArrayList parteiantragtextblocks_ = new ArrayList();
  protected ArrayList parteiantragparteien_ = new ArrayList();
  
  protected ArrayList amtsanlass_ = new ArrayList();
  
  protected String rechtsgrundlage_ = "[ Kein Text vorhanden ]";
  protected ArrayList kosten_ = new ArrayList();
  
  protected static Log log_ = LogFactory.getLog(Spruchteil.class.getName()); 
  
  public void removeKosten(int index) {
    kosten_.remove(index);
  }
  
  public void addKosten(Kosten k) {
    kosten_.add(k);
  }
  
  public void setKosten(int index, Kosten kosten) {
    kosten_.set(index, kosten);
    
  }
  
  public Kosten getKosten(int index) {
    return (Kosten)kosten_.get(index);
  }
  
  public ArrayList getKosten() {
    return kosten_;
  }
  
  public Adressat getParteienAntragPartei(int index) {
    try {
      return (Adressat)((NameValuePair)parteiantragparteien_.get(index)).getValue();
    } catch(IndexOutOfBoundsException e) {
      return null;
    }
  }
  
  public void removeParteienAntragPartei(int index) {
    parteiantragparteien_.remove(index);
  }
  
  public void setParteienAntragPartei(int index, Adressat partei) {
    NameValuePair pair = (NameValuePair)parteiantragparteien_.get(index);
    pair.setValue(partei);
    //parteiantragparteien_.set(index, new NameValuePair(anliegen, partei));
  }
  
  public void setParteienAntragPartei(int index, String anliegen, Adressat partei) {
    parteiantragparteien_.set(index, new NameValuePair(anliegen, partei));
    
    amtsanlass_.clear();
  }
  
  public void addParteienAntragPartei(String anliegen, Adressat partei) {
    parteiantragparteien_.add(new NameValuePair(anliegen, partei));
    
    amtsanlass_.clear();
  }
  
  public void removeParteienAntragTextBlock(int index) {
    parteiantragtextblocks_.remove(index);
  }
  
  public ArrayList getParteienAntragParteien() {
    return parteiantragparteien_;
  }
  
  public ArrayList getParteienAntragTextBlocks() {
    return parteiantragtextblocks_;
  }
  
  public TextBlock getParteienAntragTextBlock(int index) {
    return (TextBlock)parteiantragtextblocks_.get(index);
  }
  
  public void setParteienAntragTextBlock(int index, TextBlock block) {
    parteiantragtextblocks_.set(index, block);
    
    amtsanlass_.clear();
  }
  
  public void addParteienAntragTextBlock(TextBlock block) {
    parteiantragtextblocks_.add(block);
    
    amtsanlass_.clear();
  }
  
  public ArrayList getAmtsanlassTextBlocks() {
    return amtsanlass_;
  }
  
  public TextBlock getAmtsanlassTextBlock(int index) {
    return (TextBlock)amtsanlass_.get(index);
  }
  
  public void removeAmtsanlassTextBlock(int index) {
    amtsanlass_.remove(index);
  }
  
  public void setAmtsanlassTextBlock(int index, TextBlock block) {
    amtsanlass_.set(index, block);
    
    parteiantragparteien_.clear();
    parteiantragtextblocks_.clear();

  }
  
  
  public void addAmtsanlassTextBlock(TextBlock block) {
    
    parteiantragparteien_.clear();
    parteiantragtextblocks_.clear();
    
    amtsanlass_.add(block);
  }
  
  /*
   * @see asit.bescheid.BescheidElement#toElement(org.w3c.dom.Document)
   */
  public Element toElement(Document doc) {
    
    Element spruchteil = doc.createElementNS(BescheidConstants.BESCHEID_NS, BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.SPRUCHTEIL);
    
    
    // kein amtsanlass -> checken ob parteiantraege passen
    if (amtsanlass_.size() == 0 && 
       (parteiantragparteien_.size() == 0 ||
        parteiantragtextblocks_.size() == 0)) {
      
      /*if (parteiantragparteien_.size() == 0) {
        addParteienAntragPartei("[ Leer ] ", new Adressat());
      }

      if (parteiantragtextblocks_.size() == 0) {
        addParteienAntragTextBlock(new TextBlock());
      }*/
      TextBlock block = new TextBlock();
      block.setTId("[Frist]");
      block.setTText("[Text]");
      addAmtsanlassTextBlock(block);
    }
    
    if (parteiantragparteien_.size() > 0 ||
        parteiantragtextblocks_.size() > 0) {
      Element parteienantrag = doc.createElementNS(BescheidConstants.BESCHEID_NS, BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.PARTEIENANTRAG);
      spruchteil.appendChild(parteienantrag);
      
      // fuege textbloecke hinzu
      for (int i=0; i<parteiantragtextblocks_.size(); i++) {
        parteienantrag.appendChild(getParteienAntragTextBlock(i).toElement(doc));
      }
      
      // fuege parteien hinzu
      for (int i=0; i<parteiantragparteien_.size(); i++) {
        Element parteiElement = doc.createElementNS(BescheidConstants.BESCHEID_NS, BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.PARTEI);
        
        Adressat partei = getParteienAntragPartei(i);
        partei.setElementName("p:" + BescheidConstants.KOMPAKTEPERSONENDATEN);
        parteiElement.appendChild(partei.toElement(doc));
        
        parteienantrag.appendChild(parteiElement);
      }
    }
    
    if (amtsanlass_.size() > 0) {
      Element amtsanlass = doc.createElementNS(BescheidConstants.BESCHEID_NS, BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.AMTSANLASS);
      spruchteil.appendChild(amtsanlass);

      // fuege amtsanlaesse hinzu
      for (int i=0; i<amtsanlass_.size(); i++) {
        amtsanlass.appendChild(getAmtsanlassTextBlock(i).toElement(doc));
      }
    }
    
    if (rechtsgrundlage_ == null) {
      rechtsgrundlage_ = "";
    }
    String formattedRechtsGrundlage = Utils.normalizeText(rechtsgrundlage_);
    Bescheid.addStringElement(spruchteil, doc, BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.RECHTSGRUNDLAGE, formattedRechtsGrundlage);

    if (kosten_.size() > 0) {
      //Element kosten = doc.createElementNS(BescheidConstants.BESCHEID_NS, BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.KOSTEN);
      //spruchteil.appendChild(kosten);

      // fuege amtsanlaesse hinzu
      for (int i=0; i<kosten_.size(); i++) {
        spruchteil.appendChild(getKosten(i).toElement(doc));
      }
    }

    
    return spruchteil;
  }

  /*
   * @see asit.bescheid.BescheidElement#update(org.w3c.dom.Element)
   */
  public void update(Element element) {
    
    log_.debug("Updating simple eckdata:");
    try {
      
      // retrieve rechtsgrundlage
      Node rechtsgrundlage = XPathAPI.selectSingleNode(element, "descendant-or-self::" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.RECHTSGRUNDLAGE + "/text()");
      if (rechtsgrundlage != null) {
        log_.debug("Found Rechtsgrundlage node: " + rechtsgrundlage.getNodeValue());
        rechtsgrundlage_ = rechtsgrundlage.getNodeValue();
      } else {
        rechtsgrundlage_ = "";
      }

      // retrieve parteienantragtextbloecke
      NodeList parteiantragtextbloecke = XPathAPI.selectNodeList(element, "descendant-or-self::" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.PARTEIENANTRAG + 
          "/" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.TEXTBLOCK);

      // partaeiantrage textbloecke
      if (parteiantragtextbloecke.getLength() > 0) {
        log_.debug("Found Parteiantrag textbloecke nodes: " + parteiantragtextbloecke.getLength());
        for (int i=0; i<parteiantragtextbloecke.getLength(); i++) {
          if (parteiantragtextbloecke.item(i) instanceof Element) {
            TextBlock block = new TextBlock();
            block.update((Element)parteiantragtextbloecke.item(i));
            parteiantragtextblocks_.add(block);
          }
        }
      }

      //  retrieve parteienantragtextbloecke
      NodeList parteiantragparteien = XPathAPI.selectNodeList(element, "descendant-or-self::" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.PARTEIENANTRAG + 
          "/" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.PARTEI +
          "/" + PersonConstants.PERSON_NS_PREFIX + ":" + BescheidConstants.KOMPAKTEPERSONENDATEN);
      
      // partaeiantrage parteien
      if (parteiantragparteien.getLength() > 0) {
        log_.debug("Found Parteiantrag parteien nodes: ");
        for (int i=0; i<parteiantragparteien.getLength(); i++) {
          if (parteiantragparteien.item(i) instanceof Element) {
            Adressat partei = new Adressat();
            partei.update((Element)parteiantragparteien.item(i));
            addParteienAntragPartei("FIXME",  partei);
          }
        }
      }


      // retrieve amtsanlaesse
      NodeList amtsanlaesse = XPathAPI.selectNodeList(element, "descendant-or-self::" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.AMTSANLASS + 
          "/" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.TEXTBLOCK);

      // partaeiantrage textbloecke
      if (amtsanlaesse.getLength() > 0) {
        log_.debug("Found Amtsanlass textbloecke nodes: ");
        for (int i=0; i<amtsanlaesse.getLength(); i++) {
          if (amtsanlaesse.item(i) instanceof Element) {
            TextBlock block = new TextBlock();
            block.update((Element)amtsanlaesse.item(i));
            amtsanlass_.add(block);
          }
        }
      }

      
      // retrieve kosten
      NodeList kosten = XPathAPI.selectNodeList(element, "descendant-or-self::" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.KOSTEN);

      // kosten bloecke
      if (kosten.getLength() > 0) {
        log_.debug("Found kosten nodes: ");
        for (int i=0; i<kosten.getLength(); i++) {
          if (kosten.item(i) instanceof Element) {
            Kosten k = new Kosten();
            k.update((Element)kosten.item(i));
            kosten_.add(k);
          }
        }
      }

      
    } catch(TransformerException e) {
      log_.debug("Error while evaluating XPath expression: " + e.getMessage());
    }
    
  }
  /**
   * @return Returns the rechtsgrundlage_.
   */
  public String getSRechtsgrundlage() {
    return rechtsgrundlage_;
  }
  /**
   * @param rechtsgrundlage_ The rechtsgrundlage_ to set.
   */
  public void setSRechtsgrundlage(String rechtsgrundlage) {
    rechtsgrundlage_ = rechtsgrundlage;
  }
}
