package asit.common;

import net.sf.hibernate.HibernateException;
import net.sf.hibernate.Session;
import net.sf.hibernate.SessionFactory;
import net.sf.hibernate.Transaction;
import net.sf.hibernate.cfg.Configuration;


public class HibernateUtil {

  private static final SessionFactory sessionFactory;

  static {
      try {
          // Create the SessionFactory
        
          // get system property of configuration file
        
          sessionFactory = new Configuration().configure().buildSessionFactory();
      } catch (Throwable ex) {
          // Make sure you log the exception, as it might be swallowed
          System.err.println("Initial SessionFactory creation failed." + ex);
          throw new ExceptionInInitializerError(ex);
      }
  }

  public static final ThreadLocal session = new ThreadLocal();

  public static void hsqlCleanup(Session s) {
    try {
        s.connection().createStatement().execute("SHUTDOWN");
    } catch (Exception e) {
    }
  }
  
  public static Session currentSession() throws HibernateException {
      Session s = (Session) session.get();
      // Open a new Session, if this Thread has none yet
      if (s == null) {
          s = sessionFactory.openSession();
          session.set(s);
      }
      return s;
      //return sessionFactory.openSession();
  }

  public static void closeSession() throws HibernateException {
      Session s = (Session) session.get();
      session.set(null);
      if (s != null)
          s.close();
  }
  
  public static void saveOrUpdate(Object object) throws HibernateException {
    Session session = HibernateUtil.currentSession();
    Transaction tx = session.beginTransaction();
    session.saveOrUpdate(object);
    tx.commit();
    HibernateUtil.hsqlCleanup(session);
    HibernateUtil.closeSession();
  }
  
  public static Object saveOrUpdateCopy(Object object) throws HibernateException {
    Session session = HibernateUtil.currentSession();
    Transaction tx = session.beginTransaction();
    Object ret = session.saveOrUpdateCopy(object);
    tx.commit();
    HibernateUtil.hsqlCleanup(session);
    HibernateUtil.closeSession();
    return ret;
  }
  
  public static void delete(Object object) throws HibernateException {
    Session session = HibernateUtil.currentSession();
    Transaction tx = session.beginTransaction();
    session.delete(object);
    tx.commit();
    HibernateUtil.hsqlCleanup(session);
    HibernateUtil.closeSession();
  }
}
