//Copyright (C) 2004 A-SIT
//http://www.a-sit.at
//
//All rights reserved.
//
//This source is provided for inspection purposes and recompilation only,
//unless specified differently in a contract with ASIT. This source has to
//be kept in strict confidence and must not be disclosed to any third party
//under any circumstances. Redistribution in source and binary forms, with
//or without modification, are <not> permitted in any case!
//
//THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
//ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
//ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
//OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
//LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
//OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//SUCH DAMAGE.

package asit.common;

import org.apache.log4j.Logger;

/**
 * This class is responsible for initializing the logging system. This
 * will be done the first time a logger is requested.
 * <p>
 * All other classes can use this class to get a logger with a specific
 * name.
 * <p>
 * Configuration possibilities:
 * <ul>
 *   <li>Specify the location of the log4j configuration file using the
 *       system property "log4j.configuration".</li>
 *   <li>Locate the file in the system resources using the following path:
 *       "asit/logging/logger.properties".</li>
 * </ul>
 * 
 * If both options are enabled and the resources are available the first one 
 * has a higher precedence.
 * 
 * @author <a href="mailto:arne.tauber@iaik.tugraz.at">Arne Tauber</a>
 */
public class LogFactory {
  
  /**
   * The system-wide unique instance.
   */
  private static LogFactory instance_ = null;
  
  protected final static String LOGGER_PROPS = "asit/logging/logger.properties";
  
  /**
   * Protected constructor to keep singleton.
   */
  protected LogFactory() {}
  
  
  /**
   * Returns a logger with a specific name. This name must not
   * be <pre>null</pre>.
   * 
   * @param name the name of the logger.
   * @return the logger with the given name.
   * @deprecated Please use Apache Commons LogFactory instead
   * 
   * @precondition (name <> null)
   */
  public synchronized static Logger getLogger(String name) {
    if (name == null) {
      throw new NullPointerException("Name of logger must not be null.");
    }
    if (instance_ == null) {
      instance_ = new LogFactory();
    }
    /*String prop = System.getProperty("log4j.configuration");
    if (prop != null &&
        (new File(prop)).exists()) {
      System.out.println("Initialisiere A-SIT Logger ueber Property File: " + prop);
      PropertyConfigurator.configure(prop);
    } else {
      System.out.println("Hole A-SIT Logger aus Resourcen...");
      // try to load log4j configuration via resources
      InputStream is = LogFactory.class.getClassLoader().getResourceAsStream(LOGGER_PROPS);
      if (is != null) {
        Properties properties = new Properties();
        try {
          properties.load(is);
          PropertyConfigurator.configure(properties);
        } catch(IOException e) {
          e.printStackTrace();
        }
      }
    }*/
    return Logger.getLogger("asit." + name);
  }
  
}
