package asit.common;

import iaik.asn1.ObjectID;
import iaik.asn1.structures.Name;
import iaik.pkcs.pkcs12.CertificateBag;
import iaik.pkcs.pkcs12.KeyBag;
import iaik.pkcs.pkcs12.PKCS12;
import iaik.security.provider.IAIK;
import iaik.utils.Base64Exception;
import iaik.utils.Base64InputStream;
import iaik.utils.Base64OutputStream;
import iaik.utils.Util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.servlet.http.HttpServletRequest;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import asit.framework.utils.FormatException;
import asit.moa.client.exception.DeliveryException;
import asit.moa.client.verify.VerifyResult;

public class Utils {
//    private static String defaulEncodingName_ = (String) AccessController.doPrivileged(new GetPropertyAction("file.encoding"));
    private static String defaulEncodingName_ = System.getProperty("file.encoding");

    static Log logger_ = LogFactory.getLog(Utils.class.getName());
    
    public static boolean isEmpty(String str) {
      return str == null ||
        "".equals(str);
    }
    
    public static void mapVerifyResultToJSP(VerifyResult result, HttpServletRequest request) {
      if (result.getSignatureCheckCode() == 0) {
        request.setAttribute("SC", "true");
        request.setAttribute("CHECK_SIGNATURE", "Die berprfung der Hash-Werte und des Werts der Signatur konnte erfolgreich durchgefhrt werden.");
      } else {
        request.setAttribute("SC", "false");
        request.setAttribute("CHECK_SIGNATURE", "Die berprfung der Hash-Werte und des Werts der Signatur konnte NICHT erfolgreich durchgefhrt werden.");
      }
      
      if (result.getCertificateCheckCode() == 0) {
        request.setAttribute("CC", "true");
        request.setAttribute("CHECK_CERTIFICATE", "Eine formal korrekte Zertifikatskette vom Signatorzertifikat zu einem vertrauenswrdigen Wurzelzertifikat konnte konstruiert werden. Jedes Zertifikat dieser Kette ist zum in der Anfrage angegebenen Prfzeitpunkt gltig.");
      } else {
        request.setAttribute("CC", "false");
        request.setAttribute("CHECK_CERTIFICATE", "Eine formal korrekte Zertifikatskette vom Signatorzertifikat zu einem vertrauenswrdigen Wurzelzertifikat konnte NICHT konstruiert werden.");
      }
      
      Name signer = result.getSubjectName();
      request.setAttribute("SIGNER_NAME", signer.getRDNValue(ObjectID.commonName));
      request.setAttribute("SIGNER_ORGANIZATIONAL_UNIT", signer.getRDNValue(ObjectID.organizationalUnit));
      request.setAttribute("SIGNER_ORGANIZATION", signer.getRDNValue(ObjectID.organization));
      request.setAttribute("SIGNER_COUNTRY", signer.getRDNValue(ObjectID.country));

      Name issuer = result.getIssuerName();
      request.setAttribute("ISSUER_NAME", issuer.getRDNValue(ObjectID.commonName));
      request.setAttribute("ISSUER_ORGANIZATIONAL_UNIT", issuer.getRDNValue(ObjectID.organizationalUnit));
      request.setAttribute("ISSUER_ORGANIZATION", issuer.getRDNValue(ObjectID.organization));
      request.setAttribute("ISSUER_COUNTRY", issuer.getRDNValue(ObjectID.country));

      request.setAttribute("CERT_SERIAL_NUMBER", result.getIssuerSerial().toString());
      
      if (result.isQualifiedCertificate()) {
        request.setAttribute("CERT_QUALITY", "Qualifiziertes Zertifikat");
      } else {
        request.setAttribute("CERT_QUALITY", "Gew&ouml;hnliches Zertifikat");
      } 
    }
    
    public static  String getSearchURL(boolean advanced, int startIndex, String mzsid, String appid, String datefrom, String datebetween, String errorcode, String errormessage) {
      String ret = "search.jsp?action=search";
      try {
        ret += "&mzsid=" + URLEncoder.encode(mzsid, "UTF-8");
        ret += "&appid=" + URLEncoder.encode(appid, "UTF-8");
        ret += "&datefrom=" + URLEncoder.encode(datefrom, "UTF-8");
        ret += "&datebetween=" + URLEncoder.encode(datebetween, "UTF-8");
        ret += "&errorcode=" + URLEncoder.encode(errorcode, "UTF-8");
        ret += "&errormessage=" + URLEncoder.encode(errormessage, "UTF-8");
        ret += "&start=" + startIndex;
        if (advanced) {
          ret += "&advanced=true";
        }
      } catch(Exception e) {
        
      }
      return ret;
    }

    
    public static Element packIntoSoap(Element element) throws ParserConfigurationException {
      Document doc = createEmptyDocumentNS();
      Element root = doc.createElement("soapenv:Envelope");
      root.setAttribute("xmlns:soapenv", "http://schemas.xmlsoap.org/soap/envelope/");
      root.setAttribute("xmlns:xsd", "http://www.w3.org/2001/XMLSchema");
      root.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
      
      Element body = doc.createElement("soapenv:Body");
      root.appendChild(body);
      body.appendChild(doc.importNode(element, true));
      
      return root;
    }
    
    public static Document createEmptyDocumentNS() throws ParserConfigurationException {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(true);
      return factory.newDocumentBuilder().newDocument();
    }

    public static Document createEmptyDocument(boolean ns) throws ParserConfigurationException {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(ns);
      return factory.newDocumentBuilder().newDocument();
    }

    
    // return c1\{c2}
    public static List diff(Collection c1, Collection c2) {
      ArrayList list = new ArrayList();
      for (Iterator it = c1.iterator(); it.hasNext();) {
        Object toCheck = it.next();
        if (!c2.contains(toCheck)) {
          list.add(toCheck);
        }
      }
      return list;
    }
    
    public static SSLSocketFactory configureHTTPSConnection(String clientAuthFile, String clientAuthPass, String clientAuthType, String keyStoreFile, String password, String trustStoreType) throws Exception {
      logger_.debug("CLIENT-AUTH: " + clientAuthFile + "::" + keyStoreFile);
      InputStream clientAuthStream = null;
      if (clientAuthFile != null) {
        clientAuthStream = new FileInputStream(clientAuthFile);
      }
      
      InputStream keyStoreStream = null;
      if (keyStoreFile != null) {
          keyStoreStream = new FileInputStream(keyStoreFile);
      }
      
      return configureHTTPSConnection(clientAuthStream, clientAuthPass, clientAuthType, keyStoreStream, password, trustStoreType); 
    }
    
    public static SSLSocketFactory configureHTTPSConnection(InputStream clientAuthFile, String clientAuthPass, String clientAuthType, InputStream trustStoreFile, String password, String trustStoreType) throws Exception {
      

      // handle keystore
      KeyStore clientStore = null;
      if (clientAuthFile != null) {
        if (clientAuthType == null) {
          clientAuthType = "JKS"; // default
        }
        clientStore = KeyStore.getInstance(clientAuthType);
        logger_.debug("Loading keystore: " + clientAuthFile);
        clientStore.load(clientAuthFile, clientAuthPass.toCharArray());
      }

      logger_.debug("Initializing KeyManager and TrustManager");

      KeyManager[] keyManagers = null;
      if (clientAuthFile != null) {
        KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
        kmf.init(clientStore, clientAuthPass.toCharArray());
        keyManagers = kmf.getKeyManagers();
      }

      // Handle truststore
      KeyStore trustStore = null;
      TrustManagerFactory tmf = null;
      if (trustStoreFile != null) {
        if (trustStoreType == null) {
          trustStoreType = "JKS"; // default
        }
        trustStore = KeyStore.getInstance(trustStoreType);
        logger_.debug("Loading truststore: " + trustStoreFile);
        trustStore.load(trustStoreFile, password.toCharArray());
        tmf = TrustManagerFactory.getInstance( "SunX509" );
        tmf.init( trustStore );
      }
      
      logger_.debug("Getting SSL Context");
      SSLContext context = SSLContext.getInstance("TLS");
      if (trustStoreFile != null) {
        logger_.debug("Using standard trustmanager.");
        context.init(keyManagers, tmf.getTrustManagers(), new SecureRandom());
      } else {
        logger_.debug("Using no trustmanager.");
        context.init(keyManagers, new TrustManager[] { new TrustAllManager() }, new SecureRandom());
      }
      //HttpsURLConnection.setDefaultSSLSocketFactory(context.getSocketFactory());
      logger_.debug("Returning Socket Factory");
      return context.getSocketFactory();
    }

    public static void serializeElement(Element element, OutputStream out) throws IOException {
      OutputFormat format = new OutputFormat();
      format.setOmitXMLDeclaration(true);
      format.setEncoding("UTF-8");
      format.setPreserveSpace(true);
      XMLSerializer serializer = new XMLSerializer(new OutputStreamWriter(out, "UTF-8"), format);
      serializer.serialize(element);
      //serialize(element, out, false, false);
    }
    
    public static void serializeDocument(Document doc, OutputStream out) throws IOException {
      OutputFormat format = new OutputFormat();
      format.setOmitXMLDeclaration(false);
      format.setEncoding("UTF-8");
      format.setPreserveSpace(true);
      XMLSerializer serializer = new XMLSerializer(new OutputStreamWriter(out, "UTF-8"), format);
      serializer.serialize(doc);
      //serialize(doc, out, false, false);
    }
    
    public static void serializeNode(Node node, OutputStream out) throws IOException {
      OutputFormat format = new OutputFormat();
      format.setOmitXMLDeclaration(true);
      format.setEncoding("UTF-8");
      format.setPreserveSpace(true);
      NodeSerializer serializer = new NodeSerializer(new OutputStreamWriter(out, "UTF-8"), format);
      serializer.doSerializeNode(node);
      
      //serialize(node, out, false, false);
    }
    
    public static Document readDocFromIsNoNS(InputStream is) throws Exception {
      DocumentBuilderFactory f = DocumentBuilderFactory.newInstance();
      f.setNamespaceAware(false);
      return f.newDocumentBuilder().parse(is);
    }
    
    public static Document readDocFromIsNS(InputStream is) throws Exception {
      DocumentBuilderFactory f = DocumentBuilderFactory.newInstance();
      f.setNamespaceAware(true);
      return f.newDocumentBuilder().parse(is);
    }
    
    public static Document readDocFromIsNS(Reader is) throws Exception {
      DocumentBuilderFactory f = DocumentBuilderFactory.newInstance();
      f.setNamespaceAware(true);
      InputSource source = new InputSource(is);
      return f.newDocumentBuilder().parse(source);
    }
    
    public static Document parseXMLString(String inputString) throws SAXException, IOException, ParserConfigurationException {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(true);
      InputSource source = new InputSource(new InputStreamReader(new ByteArrayInputStream(inputString.getBytes()), "UTF-8"));
      return factory.newDocumentBuilder().parse(source);
    }
    
    public static Document parseXMLStringNoNS(String inputString) throws SAXException, IOException, ParserConfigurationException {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(false);
      InputSource source = new InputSource(new InputStreamReader(new ByteArrayInputStream(inputString.getBytes("UTF-8")), "UTF-8"));
      return factory.newDocumentBuilder().parse(source);
    }
    
    public static String inputStreamToBase64(InputStream is) throws IOException {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      Base64OutputStream os = new Base64OutputStream(out);
      
      BufferedInputStream bis = new BufferedInputStream(is);
      byte[] buffer = new byte[1024];
      int len = -1;
      while ((len = bis.read(buffer)) != -1) {
        os.write(buffer, 0, len);
      }
      
      os.flush();
      out.flush();
      
//      System.out.println("Writte size: " + out.size());
      return out.toString(); 
    }
    
    public static void dumpInputOutputStream(InputStream is, OutputStream os) throws IOException {
      if (is == null) {
        return;
      }
      int ch;
      while ((ch = is.read()) != -1) {
        os.write(ch);
      }
    }
    
    public static void dumpInputOutputStream(Reader reader, OutputStream os) throws IOException {
      logger_.debug("Dumping Reader: ");
      if (reader == null) {
        return;
      }
      int ch;
      while ((ch = reader.read()) != -1) {
//        System.out.print((char)ch);
        os.write(ch);
      }
    }
    
    public static void createTrustStore(Hashtable certificates, String jks, String password) throws Exception {
      Enumeration e = certificates.keys();
      KeyStore ks = KeyStore.getInstance("JKS");
      ks.load(null, password.toCharArray());
      while (e.hasMoreElements()) {
        String alias = (String)e.nextElement();
        X509Certificate cert = (X509Certificate)certificates.get(alias);
        ks.setCertificateEntry(alias, cert);
      }
      ks.store(new FileOutputStream(jks), password.toCharArray()); 
    }
    
    public static void convertPKCS12ToJKS(String pkcs12, String jks, String password, String alias) throws Exception {
      Security.addProvider(new IAIK());
      
      String inFile = pkcs12;
      String outFile = jks;
      
      PKCS12 p = new PKCS12(new FileInputStream(inFile));
      p.decrypt(password.toCharArray());
      KeyBag keyBag = p.getKeyBag();
      
      
      PrivateKey priv = keyBag.getPrivateKey();
      X509Certificate[] certs = CertificateBag.getCertificates(p.getCertificateBags());

      
      KeyStore ks = KeyStore.getInstance("JKS");
      
      ks.load(null, password.toCharArray());
      
      ks.setKeyEntry(alias, priv, password.toCharArray(), certs);

      ks.store(new FileOutputStream(outFile), password.toCharArray()); 
    }
    
    
    /**
     * 
     * 
     * @author troessler
     * 
     * @param doc
     * @param outputStream
     * @param addXMLHeader
     * @param ident
     * @return
     */
    public static boolean serialize(Document doc, OutputStream outputStream,
            boolean addXMLHeader, boolean ident) {
        try {
            StringWriter strWriter = null;
            OutputFormat outFormat = null;
            XMLSerializer domSerializer;
            strWriter = new StringWriter();
            outFormat = new OutputFormat();
            // Setup format settings
            outFormat.setEncoding("UTF-8");
            outFormat.setVersion("1.0");
            outFormat.setIndenting(ident);
            outFormat.setPreserveSpace(true);
            //outFormat.setIndent(4);
            outFormat.setOmitXMLDeclaration(!addXMLHeader);
            // Serialize XML Document
//            domSerializer = new XMLSerializer(new OutputStreamWriter(
//                    outputStream, "UTF-8"), outFormat);
            domSerializer = new XMLSerializer(outputStream, outFormat);
            domSerializer.serialize(doc);
            return true;
        } catch (Exception ex) {
            return false;
        }
    }
    
    public static boolean serialize(Element doc, OutputStream outputStream,
        boolean addXMLHeader, boolean ident) {
    try {
        StringWriter strWriter = null;
        OutputFormat outFormat = null;
        XMLSerializer domSerializer;
        strWriter = new StringWriter();
        outFormat = new OutputFormat();
        // Setup format settings
        outFormat.setEncoding("UTF-8");
        outFormat.setVersion("1.0");
        outFormat.setIndenting(ident);
        outFormat.setPreserveSpace(true);
        //outFormat.setIndent(4);
        outFormat.setOmitXMLDeclaration(!addXMLHeader);
        // Serialize XML Document
//        domSerializer = new XMLSerializer(new OutputStreamWriter(
//                outputStream, "UTF-8"), outFormat);
        domSerializer = new XMLSerializer(outputStream, outFormat);
        domSerializer.serialize(doc);
        return true;
    } catch (Exception ex) {
        return false;
    }
}

    /**
     * 
     * @author troessler
     * 
     * @param xmltobequoted
     * @return
     */
    public static String simpleQuoter(String xmltobequoted) {
        String result = xmltobequoted.replaceAll("<", "&lt;");
        result = result.replaceAll(">", "&gt;");
        result = result.replaceAll("\"", "&quot;");
        return result;
    }

    /**
     * 
     * @author troessler
     * 
     * @param component
     * @param subjectdn
     * @return
     */
    public static String getSubjectDNDetail(String component, String subjectdn) {
        //	gain the value of the component
        String distname = component.toLowerCase();
        String subjectDN = subjectdn.toLowerCase();
        int fromIDX = subjectDN.indexOf(distname) + distname.length() + 1;
        int toIDX = subjectDN.indexOf(",", fromIDX);
        // if this is the last field in the certificate:
        if (toIDX == -1)
            toIDX = subjectDN.length();
        return subjectDN.substring(fromIDX, toIDX);
    }

    /**
     * 
     * @author troessler
     * 
     * @param description
     * @param paramname
     * @param paramvalue
     * @param readonly
     * @return
     */
    public static String generateHTMLInputFormEntry(String description,
            String paramname, String paramvalue, boolean readonly,
            boolean hidden) {
        if (hidden) {
            return ("<tr><td>" + "</td>" + "<td><input name=\"" + paramname
                    + "\" type=\"hidden\" id=\"" + paramname + "\" value=\""
                    + paramvalue + "\" size=\"50\" " + "></td></tr>");
        }
        else {
            String ronly = "";
            if (readonly)
                ronly = "readonly";
            return ("<tr><td>" + description + "</td>" + "<td><input name=\""
                    + paramname + "\" type=\"text\" id=\"" + paramname
                    + "\" value=\"" + paramvalue + "\" size=\"50\" " + ronly + "></td></tr>");
        }
    }

    /**
     * 
     * @author roessler
     * 
     * @param input
     * @return @throws
     *         IOException
     */
    public static byte[] base64Encoder(byte[] input) throws IOException {
        ByteArrayOutputStream bous = new ByteArrayOutputStream();
        Base64OutputStream b64ous = new Base64OutputStream(bous);
        b64ous.write(input);
        b64ous.flush();
        b64ous.close();
        bous.flush();
        bous.close();
        return bous.toByteArray();
    }

    /**
     * 
     * @author roessler
     * 
     * @param input
     * @return @throws
     *         IOException
     */
    public static byte[] base64Decoder(byte[] input) throws IOException {
        Base64InputStream b64is = new Base64InputStream(
                new ByteArrayInputStream(input));
        byte[] result = new byte[b64is.available()];
        b64is.read(result);
        b64is.close();
        return result;
    }

    /**
     * 
     * @author troessler
     * 
     * @param original
     * @param keyword
     * @param replacement
     * @return
     */
    public static String replaceWord(String original, String keyword,
            String replacement) {
        String result = "";
        int from = 0;
        int foundat = 0;
        int lengthkeyword = keyword.length();
        while ((foundat = original.indexOf(keyword, from)) >= 0) {
            result += original.substring(from, foundat);
            result += replacement;
            from = foundat + lengthkeyword;
        }
        return result + original.substring(from);
    }

    /**
     * 
     * @param httprequest
     * @return @throws
     *         UnsupportedEncodingException
     */
    public static Hashtable decodeURLEncodedHTTPRequest(String httprequest)
            throws UnsupportedEncodingException {
        Hashtable parameters = new Hashtable();
        
//        Enumeration keys= request.getParameterNames();
//        while (keys.hasMoreElements())
//        {
//           String key = (String)keys.nextElement();
//           String value = request.getParameter(key);
//           parameters.put(key, value);
//        } 
        
        StringTokenizer st;
        st = new StringTokenizer(httprequest, "&");
        int notfirst = 0;
        while (st.hasMoreTokens()) {
            String valname = st.nextToken("=").substring(notfirst);
            notfirst = 1;
            String value = URLDecoder.decode(st.nextToken("&").substring(1),
                                             defaulEncodingName_);
            parameters.put(valname, value);
        }
        return parameters;
    }

    /**
     * 
     * @author roessler
     * 
     *  
     */
    public static String getXMLString(Document xmldocument) {
        StringWriter strWriter = null;
        XMLSerializer probeMsgSerializer = null;
        OutputFormat outFormat = null;
        try {
            probeMsgSerializer = new XMLSerializer();
            strWriter = new StringWriter();
            outFormat = new OutputFormat();
            // Setup format settings
            outFormat.setEncoding("UTF-8");
            outFormat.setVersion("1.0");
            outFormat.setIndenting(true);
            outFormat.setIndent(4);
            outFormat.setOmitXMLDeclaration(true);
            // Define a Writer
            probeMsgSerializer.setOutputCharStream(strWriter);
            // Apply the format settings
            probeMsgSerializer.setOutputFormat(outFormat);
            // Serialize XML Document
            probeMsgSerializer.serialize(xmldocument);
            String result = strWriter.toString();
            strWriter.close();
            return result;
        } catch (IOException ioEx) {
            return ("Error " + ioEx);
        }
    }
    
    
    /**
     * 
     * @author roessler
     * 
     *  
     */
    public static String getXMLString(Element xmldocumentelement) {
        StringWriter strWriter = null;
        XMLSerializer probeMsgSerializer = null;
        OutputFormat outFormat = null;
        try {
            probeMsgSerializer = new XMLSerializer();
            strWriter = new StringWriter();
            outFormat = new OutputFormat();
            // Setup format settings
            outFormat.setEncoding("UTF-8");
            outFormat.setVersion("1.0");
            outFormat.setIndenting(true);
            outFormat.setIndent(4);
            outFormat.setOmitXMLDeclaration(true);
            // Define a Writer
            probeMsgSerializer.setOutputCharStream(strWriter);
            // Apply the format settings
            probeMsgSerializer.setOutputFormat(outFormat);
            // Serialize XML Document
            probeMsgSerializer.serialize(xmldocumentelement);
            String result = strWriter.toString();
            strWriter.close();
            return result;
        } catch (IOException ioEx) {
            return ("Error " + ioEx);
        }
    }
    
    
    /*
     * Replaces all occurences of variable with textToInsert in the content string
     */
    public static String replaceVariable(String content, String variable, String textToInsert) {
      //System.out.println("Variable: " + variable + "/" + textToInsert);
      if (textToInsert == null) {
//        System.out.println("Text to insert is null: " + variable);
        return content;
      }
      return content.replaceAll(variable, textToInsert);
    }
    
    /*
     * Replaces all occurences of a variable in a file
     */
    public static String replaceVariables(File file, Hashtable pairs) throws IOException {
      FileInputStream is = new FileInputStream(file);
      InputStreamReader reader = new InputStreamReader(is, "UTF-8");
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      int ch = 0;
      while((ch = reader.read()) != -1) {
        out.write(ch);
      }
      String ret = out.toString();
      if (pairs == null) {
        return ret;
      } else {
        Enumeration keys = pairs.keys();
        while (keys.hasMoreElements()) {
          String key = (String)keys.nextElement();
          ret = replaceVariable(ret, key, (String)pairs.get(key));
        }
        return ret;
      }
    }
    
    /*
     * Normalizes all arbitrary text (Normalisierung der Freitextfelder)
     */
    public static String normalizeText(String inputText) {
      String ret = inputText;
      
      // Eliminate all tabs
      ret = ret.replaceAll("\t", " ");
      
      // Normalize multiple spaces, 1 or more
      ret = ret.replaceAll("( ++)", " ");
      
      // Normalize line breaks
      /*ret = ret.replaceAll("\r\n", "\n"); // CR-LF -> LF  (Windows -> Unix)
      ret = ret.replaceAll("\r", "\n"); // CR only -> LF (Mac OS -> Unix)
      ret = ret.replaceAll("\n", "\r\n"); // All LF -> CR-LF (Unix -> Windows)*/
      
      ret = ret.replaceAll("\r\n|\n|\r", "\r\n"); // All in one

      //ret = ret.replaceAll("(\r\n){2,}", "\r\n\r\n"); // More than 2 line breaks -> 2 line breaks
      
      ret = ret.replaceAll("\r\n", "");
      
      // Normalize quotes ' and "
      ret = ret.replaceAll("\u0060|\u00B4|\u2018|\u2019", "\u0027"); // '
      ret = ret.replaceAll("\u201C|\u201D|\u2033", "\\u0022"); // "
      ret = ret.replaceAll("(\u0027++)", "\\u0022"); // '' -> "
            
      // Normalize punctuation -
      ret = ret.replaceAll("\u00AD|\u2013|\u2014", "\u002D");
      
      return ret.trim();
    }
    
    /*
     * Shows the unicode representation. Yet to fix..
     */
    public static String toUnicode(String text) {
      if (text == null) {
        return null;
      }
      byte[] chars = text.getBytes();
      String ret = "";
      for (int i=0; i<chars.length;i++) {
        ret += "\\u" + chars[i];
      }
      return ret;
    }
    
    /*
     * Performs a simple XSL transformation.
     */
    public static String transformInputStream(InputStream is, InputStream xsl) throws TransformerException, UnsupportedEncodingException {
      TransformerFactory factory = TransformerFactory.newInstance();
      Transformer transformer = factory.newTransformer(new StreamSource(new InputStreamReader(xsl, "UTF-8")));
      ByteArrayOutputStream os = new ByteArrayOutputStream();
      transformer.transform(new StreamSource(is), new StreamResult(new OutputStreamWriter(os, "UTF-8")));
      String ret = os.toString("UTF-8");
//      System.out.println(ret);
      return ret;
    }
    
 
    
    public static String formatNotificationDate(String date, String month, String year) throws FormatException {
      try {
        int d = Integer.parseInt(date);
        int m = Integer.parseInt(month);
        int y = Integer.parseInt(year);
        
        if ((d < 1) || (d > 31)) {
          throw new FormatException("Tag muss zwischen 1 und 31 liegen.");
        }
        
        if ((m < 1) || (m > 12)) {
          throw new FormatException("Monat muss zwischen 1 und 12 liegen.");
        }
        
        return y + "-" + addLeadingZero10(m) + "-" + addLeadingZero10(d);
        
      } catch(NumberFormatException e) {
        e.printStackTrace();
        throw new FormatException("Eingegebener Wert ist keine Zahl.");
      }
      
    }
    
    private static String addLeadingZero10(int number) {
      if (number < 10) {
        return "0" + number;
      } else {
        return String.valueOf(number);
      }
    }
    
    public static void checkBase64Signature(String base64) throws FormatException {
      
      try {
        Util.Base64Decode(base64.trim().getBytes());
      } catch (Base64Exception e) {
        e.printStackTrace();
        throw new FormatException("Signaturwert hat ungltiges Format.");
      }
      
    }
    
    public static String generateAppDeliveryID() throws DeliveryException {
      try {
        SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
        byte[] bytes = new byte[8];
        random.nextBytes(bytes);
        //System.out.println(bytes.length);
        return convertToHex(bytes);
      } catch(Exception e) {
        throw new DeliveryException(e.getMessage());
      }
    }
    
    private static String convertToHex(byte[] bytes) {
      String ret = "";
      for (int i=0; i<bytes.length; i++) {
        
        ret += Integer.toHexString(0x0100 + (bytes[i] & 0xFF)).substring(1);
      }
      return ret;
    }
    
    public static void main(String[] args) {
      ArrayList l1 = new ArrayList();
      l1.add("a");
      l1.add("b");
      l1.add("c");
      ArrayList l2 = new ArrayList();
      l2.add("a");
//      System.out.println(diff(l1, l2));
      
    }
    
}

class NodeSerializer extends XMLSerializer {
 
  
  
  /**
   * 
   */
  public NodeSerializer() {
    super();
    // TODO Auto-generated constructor stub
  }
  /**
   * @param arg0
   * @param arg1
   */
  public NodeSerializer(OutputStream arg0, OutputFormat arg1) {
    super(arg0, arg1);
    // TODO Auto-generated constructor stub
  }
  /**
   * @param arg0
   * @param arg1
   */
  public NodeSerializer(Writer arg0, OutputFormat arg1) {
    super(arg0, arg1);
    // TODO Auto-generated constructor stub
  }
  /**
   * @param arg0
   */
  public NodeSerializer(OutputFormat arg0) {
    super(arg0);
    // TODO Auto-generated constructor stub
  }
  public void doSerializeNode(Node node) throws IOException {
    serializeNode(node);
  }
  
  protected void serializeNode(Node arg0) throws IOException {
    // TODO Auto-generated method stub
    super.serializeNode(arg0);
  }
}