/*
 * Created on 20.07.2004
 * 
 * Abstract class is the basis for all other front-end servlets.
 *
 */
package asit.framework.servlet;


import java.io.IOException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;
import asit.framework.jsp.JSPConstants;
import asit.framework.servlet.content.ContentFactory;

/**
 * 
 * @author troessler
 * 
 *  
 */
public abstract class FrontEndServlet extends HttpServlet {

    protected String dispatchJSP_ = "dispatch.jsp";

    protected String serverAddress_;

    protected boolean debugMode_ = true;

    protected RequestDispatcher jspDispatcher_;

    protected ContentFactory contentFactoryImpl_;
    
    protected ServletConfig globalServletConfig_;

    /**
     * The Logger
     */
    static Logger logger_ = Logger.getLogger(FrontEndServlet.class.getName());
    
    /**
     * servlet initialization
     */
    public void init(ServletConfig config) throws ServletException {

        // it is very important to set the encoding to UTF-8 in order to process
        System.setProperty("file.encoding", "UTF-8");
        
        this.globalServletConfig_ = config;
        
        try {

            this.serverAddress_ = config.getServletContext().getInitParameter(
                    ServletConstants.WEBXML_SERVER_ADDRESS);

            this.debugMode_ = "true".equalsIgnoreCase(config.getServletContext().getInitParameter(
                    ServletConstants.WEBXML_DEBUG_MODE));                    
            
            this.contentFactoryImpl_ = this.getContentFactory();

        } catch (Exception e) {

            // TODO log4j
            e.printStackTrace();
        }

    }

    
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {
        doPost(request, response);

    }

    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        logger_.debug("Entering doPost.");
        logger_.debug("Session: " + request.getSession(true).getId());
      
        // very important - otherwise due to lacking encoding definition in the
        // request
        request.setCharacterEncoding("UTF-8");
        response.setContentType("text/html; charset=UTF-8");

        this.contentFactoryImpl_
                .setHttpServletRequest(request);
        
        this.contentFactoryImpl_.setHttpServletResponse(response);


        logger_.debug("Setting default attributes.");
        
        // response settings
        request.getSession().setAttribute(JSPConstants.DISPATCH_CONTENT,
            this.contentFactoryImpl_.getContent());
        request.getSession().setAttribute(JSPConstants.DISPATCH_BASE_URI,
                this.serverAddress_ + "/");
        request.getSession().setAttribute(JSPConstants.DISPATCH_PAGE_TITLE,
                this.contentFactoryImpl_.getPageTitle());
        request.getSession().setAttribute(JSPConstants.DISPATCH_TITLE,
                this.contentFactoryImpl_.getDispatchTitle());
        request.getSession().setAttribute(JSPConstants.CONTENT_TITLE,
                this.contentFactoryImpl_.getContentTitle());

        logger_.debug("Setting additional attributes.");
        
        // other attributes
        /*if (this.contentFactoryImpl_.getAttributes() != null) {
          for (int i=0; i<this.contentFactoryImpl_.getAttributes().size(); i++) {
            Attribute attr = (Attribute)this.contentFactoryImpl_.getAttributes().elementAt(i);
            request.getSession().setAttribute(attr.getName(), attr.getValue());
          }
        }*/

        // retrieve Dispatcher for displaying the response - may content factory wants to redirect in case of an error
        if (this.contentFactoryImpl_.getRedirector() == null) {
          this.jspDispatcher_ = request.getRequestDispatcher(this.dispatchJSP_);
        } else {
          this.jspDispatcher_ = request.getRequestDispatcher(this.contentFactoryImpl_.getRedirector());
        }
        this.jspDispatcher_.forward(request, response);

    }

    protected void setResponseJSP(String responseJSP) {
        this.dispatchJSP_ = responseJSP;
    }

    protected void setServerAddress(String serverAddress) {
        this.serverAddress_ = serverAddress;
    }

    /*
     * Returns the content factory for displaying title, content etc.
     */
    protected abstract ContentFactory getContentFactory() throws Exception;
    
    /**
     * 
     * @param filename
     * @return
     */
    public String getRealPath(String filename) {        
        return this.globalServletConfig_.getServletContext().getRealPath(filename);
    }
   
}

