package asit.framework.servlet.notification;

import iaik.ixsil.exceptions.InitException;
import iaik.ixsil.exceptions.URIException;
import iaik.ixsil.init.IXSILInit;
import iaik.ixsil.util.URI;
import java.io.File;
import java.io.IOException;
import org.w3c.dom.Element;
import asit.framework.servlet.content.ContentFactory;
import asit.moa.client.ClientFactory;
import asit.moa.client.exception.ClientException;
import asit.moa.client.exception.RequestException;
import asit.moa.client.sign.CreateSignatureClient;

/**
 * @author <a href="mailto:arne.tauber@iaik.at">Arne Tauber</a>
 */
public abstract class NotificationFactory extends ContentFactory {

  protected File xmlNotification_ = null;
  
  protected CreateSignatureClient client_ = null;
  
  public NotificationFactory(File messagePropertiesFile) throws IOException, ClientException {
  	super(messagePropertiesFile);
    
    // check, if there are set the necessary properties
    if (messageProperties_.getProperty("signature.connectionUrl") == null) {
      throw new NullPointerException("Property \"signature.connectionUrl\" is not set.");
    }
    if (messageProperties_.getProperty("signature.keyIdentifier") == null) {
      throw new NullPointerException("Property \"signature.keyIdentifier\" is not set.");
    }
     
    if (messageProperties_.getProperty("signature.xPathLocation") == null) {
      throw new NullPointerException("Property \"signature.xPathLocation\" is not set.");
    }

    if (messageProperties_.getProperty("signature.position") == null) {
      throw new NullPointerException("Property \"signature.position\" is not set.");
    }

    if (messageProperties_.getProperty("ixsil.initFile") == null) {
      throw new NullPointerException("Property \"ixsil.initFile\" is not set.");
    }
    
    try {
      // Initialize IXSIL
      IXSILInit.init(new URI(messageProperties_.getProperty("ixsil.initFile")));
      //IXSILInit.setPrintDebugLog(true);
    } catch (InitException e) {
      e.printStackTrace();
      throw new ClientException(e);
    } catch (URIException e) {
      e.printStackTrace();
      throw new ClientException(e);
    }
    client_ = ClientFactory.getSignatureClient();
    
  }


  /*
   * Creates the XML signature and sends the signature to the receiver.
   */
  protected Element createSignature() throws ClientException, RequestException {
    // set the key identifier
    client_.setKeyIdentifier(messageProperties_.getProperty("signature.keyIdentifier"));
    
    // Request
    Element doc = client_.getRequest();
    try {
//      DOMUtilsImpl.serializeElement(doc, System.out);
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    
    
    return client_.sendSignatureRequest(messageProperties_.getProperty("signature.connectionUrl"));
  }
 
}
