package asit.framework.servlet.notification;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import asit.moa.client.AbstractClient;
import asit.moa.client.ClientFactory;
import asit.moa.client.deliver.DeliveryClient;
import asit.moa.client.exception.ClientException;

/**
 * @author <a href="mailto:arne.tauber@iaik.at">Arne Tauber</a>
 */
public abstract class SignNotificationFactory extends NotificationFactory {

  protected File xmlNotification_ = null;
  protected File zsProperties_ = null;
  
  protected DeliveryClient zsClient_ = null;
  
  public SignNotificationFactory(File messagePropertiesFile,
                             File zsProperties) throws IOException, ClientException {
  	super(messagePropertiesFile);

//    if (messageProperties_.getProperty("signature.styleSheetURI") == null) {
//      throw new NullPointerException("Property \"signature.styleSheetURI\" is not set.");
//    }
    if (messageProperties_.getProperty("signature.localStyleSheet") == null) {
      throw new NullPointerException("Property \"signature.localStyleSheet\" is not set.");
    }

    if (zsProperties == null) {
      throw new NullPointerException("Argument \"zsProperties\" must not be null.");
    }
    zsProperties_ = zsProperties;
  }

  protected abstract void prepareSignature() throws ClientException, IOException;
  protected abstract void prepareZSClient(Document notifcation) throws ClientException;
  
  protected abstract String getZSType();
  
  /*
   * Creates the XML signature and sends the signature to the receiver.
   */
  protected String handleXMLNotification() {
    
    try {
      // Prepare references for the signature
      this.prepareSignature();
      
      Element response = this.createSignature();
      
      //DOMUtilsImpl.serializeElement(response, System.out);
      
      Document doc = AbstractClient.createNewDocument();
      
      // set stylesheet reference
      doc.appendChild(doc.createProcessingInstruction("xml-stylesheet", "type=\"text/xsl\" href=\"" + messageProperties_.getProperty("signature.styleSheetURI") + "\""));
      doc.appendChild(doc.importNode(response, true));
      
      // Ok, we have got a signature element, now deliver
      Properties zsProps = new Properties();
      zsProps.load(new FileInputStream(zsProperties_));
      zsClient_ = ClientFactory.getDeliveryClient(getZSType());
           
      prepareZSClient(doc);
      zsClient_.deliver();
      
    } catch (Exception e) {
      e.printStackTrace();
      return getErrorHTMLCode(e);
    }
      
    return this.successHTMLCode();
  }
  
  protected abstract String getErrorHTMLCode(Exception e);
  
  protected abstract String successHTMLCode();
  
	/* (non-Javadoc)
	 * @see asit.framework.servlet.content.ContentFactory#getContent()
	 */
	public String getContent() {
	  return this.handleXMLNotification();
	}


}
