package asit.framework.servlet.notification;

import iaik.asn1.ObjectID;
import iaik.asn1.structures.Name;
import iaik.ixsil.exceptions.InitException;
import iaik.ixsil.exceptions.URIException;
import iaik.ixsil.init.IXSILInit;
import iaik.ixsil.util.DOMUtilsImpl;
import iaik.ixsil.util.URI;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.commons.fileupload.DefaultFileItemFactory;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUpload;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import asit.framework.servlet.content.ContentFactory;
import asit.moa.MoaConstants;
import asit.moa.client.ClientFactory;
import asit.moa.client.exception.ClientException;
import asit.moa.client.verify.VerifyResult;
import asit.moa.client.verify.VerifySignatureClient;

/**
 * @author <a href="mailto:arne.tauber@iaik.at">Arne Tauber</a>
 */
public abstract class VerifyFileFactory extends ContentFactory {

  protected File xmlNotification_ = null;
  
  protected VerifySignatureClient verifyClient_ = null;
  
  static Logger logger_ = Logger.getLogger(VerifyFileFactory.class.getName());
  
  public VerifyFileFactory(File messagePropertiesFile) throws IOException, ClientException {
  	super(messagePropertiesFile);
    
    // check, if there are set the necessary properties
    if (messageProperties_.getProperty("verify.connectionUrl") == null) {
      throw new NullPointerException("Property \"verify.connectionUrl\" is not set.");
    }
    if (messageProperties_.getProperty("verify.trustprofile") == null) {
      throw new NullPointerException("Property \"verify.trustprofile\" is not set.");
    }
    if (messageProperties_.getProperty("verify.authority") == null) {
      throw new NullPointerException("Property \"verify.authority\" is not set.");
    }
    if (messageProperties_.getProperty("verify.location") == null) {
      throw new NullPointerException("Property \"verify.location\" is not set.");
    }

    if (messageProperties_.getProperty("ixsil.initFile") == null) {
      throw new NullPointerException("Property \"ixsil.initFile\" is not set.");
    }
    
    try {
      // Initialize IXSIL
      IXSILInit.init(new URI(messageProperties_.getProperty("ixsil.initFile")));

    } catch (InitException e) {
      e.printStackTrace();
      throw new ClientException(e);
    } catch (URIException e) {
      e.printStackTrace();
      throw new ClientException(e);
    }
    
  }
 
  protected String gP(String param) {
    return httpServletRequest_.getParameter(param);
  }
  
  protected String handleXMLNotification() {
    
    try {
      
      Enumeration e = httpServletRequest_.getParameterNames();
      
      logger_.debug("Parameters: ");
      while (e.hasMoreElements()) {
        logger_.debug("Element: " + (String)e.nextElement());
      }
      
      // get input file
      
      FileUpload upload = new FileUpload(new DefaultFileItemFactory());
      List fileItems = upload.parseRequest(httpServletRequest_);
      
      // we do only have one file
      FileItem file = (FileItem)fileItems.get(0);
      
      ///String file = gP("fileToVerify");
      logger_.debug(file.getName());
      //ByteArrayInputStream is = new ByteArrayInputStream(file.getBytes("UTF-8"));
      
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder;
      builder = factory.newDocumentBuilder();
      Document response = builder.parse(file.getInputStream());
      
      // set the signed document for later retrieving
      httpServletRequest_.getSession().setAttribute("xmldata", response);
      
      // Ok, now verify
      verifyClient_ = ClientFactory.getVerifySignatureClient();
      
      // verify against now
      verifyClient_.setContent(response.getDocumentElement());
      verifyClient_.addAdditionalNamespace("dsig", MoaConstants.DSIG_NS);
      
      verifyClient_.setSignatureLocation(messageProperties_.getProperty("verify.location"));
      verifyClient_.setTrustProfileID(messageProperties_.getProperty("verify.trustprofile"));
      
//      DOMUtilsImpl.serializeElement(verifyClient_.getRequest(), System.out);
      
      VerifyResult result = verifyClient_.sendVerifyRequest(messageProperties_.getProperty("verify.connectionUrl"));
      
      Name signer = result.getSubjectName();
      httpServletRequest_.setAttribute("SIGNER_NAME", signer.getRDNValue(ObjectID.commonName));
      httpServletRequest_.setAttribute("SIGNER_ORGANIZATIONAL_UNIT", signer.getRDNValue(ObjectID.organizationalUnit));
      httpServletRequest_.setAttribute("SIGNER_ORGANIZATION", signer.getRDNValue(ObjectID.organization));
      httpServletRequest_.setAttribute("SIGNER_COUNTRY", signer.getRDNValue(ObjectID.country));

      Name issuer = result.getIssuerName();
      httpServletRequest_.setAttribute("ISSUER_NAME", issuer.getRDNValue(ObjectID.commonName));
      httpServletRequest_.setAttribute("ISSUER_ORGANIZATIONAL_UNIT", issuer.getRDNValue(ObjectID.organizationalUnit));
      httpServletRequest_.setAttribute("ISSUER_ORGANIZATION", issuer.getRDNValue(ObjectID.organization));
      httpServletRequest_.setAttribute("ISSUER_COUNTRY", issuer.getRDNValue(ObjectID.country));
      
      httpServletRequest_.setAttribute("CERT_SERIAL_NUMBER", result.getIssuerSerial().toString());
      
      if (result.isQualifiedCertificate()) {
        httpServletRequest_.setAttribute("CERT_QUALITY", "Qualifiziertes Zertifikat");
      } else {
        httpServletRequest_.setAttribute("CERT_QUALITY", "Gew&ouml;hnliches Zertifikat");
      }
      
      if (result.getSignatureCheckCode() == 0) {
        httpServletRequest_.setAttribute("SC", "true");
        httpServletRequest_.setAttribute("CHECK_SIGNATURE", "Die berprfung der Hash-Werte und des Werts der Signatur konnte erfolgreich durchgefhrt werden.");
      } else {
        httpServletRequest_.setAttribute("SC", "false");
        httpServletRequest_.setAttribute("CHECK_SIGNATURE", "Die berprfung der Hash-Werte und des Werts der Signatur konnte NICHT erfolgreich durchgefhrt werden.");
      }
      
      if (result.getCertificateCheckCode() == 0) {
        httpServletRequest_.setAttribute("CC", "true");
        httpServletRequest_.setAttribute("CHECK_CERTIFICATE", "Eine formal korrekte Zertifikatskette vom Signatorzertifikat zu einem vertrauenswrdigen Wurzelzertifikat konnte konstruiert werden. Jedes Zertifikat dieser Kette ist zum in der Anfrage angegebenen Prfzeitpunkt gltig.");
      } else {
        httpServletRequest_.setAttribute("CC", "true");
        httpServletRequest_.setAttribute("CHECK_CERTIFICATE", "Eine formal korrekte Zertifikatskette vom Signatorzertifikat zu einem vertrauenswrdigen Wurzelzertifikat konnte NICHT konstruiert werden.");
      }
      
      httpServletRequest_.setAttribute("SHOW_NOTIFICATION", "false");
      
      // Add link to retrieve the signed data
      httpServletRequest_.setAttribute(VerifyResultConstants.SIGNED_DATA, "showData?sessionid=" + httpServletRequest_.getSession().getId());
      
    } catch (Exception e) {
      e.printStackTrace();
      return getErrorHTMLCode(e);
    }
      
    return this.successHTMLCode();
  }
  
  protected abstract String getErrorHTMLCode(Exception e);
  
  protected abstract String successHTMLCode();
  
  public String getContent() {
    return this.handleXMLNotification();
  }
  
}
