package asit.framework.servlet.notification;

import iaik.asn1.ObjectID;
import iaik.asn1.structures.Name;
import iaik.ixsil.exceptions.InitException;
import iaik.ixsil.exceptions.URIException;
import iaik.ixsil.init.IXSILInit;
import iaik.ixsil.util.URI;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.log4j.Logger;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import asit.common.Utils;
import asit.framework.servlet.content.ContentFactory;
import asit.moa.MoaConstants;
import asit.moa.client.ClientFactory;
import asit.moa.client.exception.ClientException;
import asit.moa.client.verify.VerifyResult;
import asit.moa.client.verify.VerifySignatureClient;

/**
 * @author <a href="mailto:arne.tauber@iaik.at">Arne Tauber</a>
 */
public abstract class VerifyNotificationFactory extends ContentFactory {

  /**
   * @param messagepropertiesfile
   * @throws IOException
   */
  public VerifyNotificationFactory(File messagepropertiesfile) throws ClientException, IOException {
    super(messagepropertiesfile);
    
//  check, if there are set the necessary properties
    if (messageProperties_.getProperty("verify.connectionUrl") == null) {
      throw new NullPointerException("Property \"verify.connectionUrl\" is not set.");
    }
    if (messageProperties_.getProperty("verify.location") == null) {
      throw new NullPointerException("Property \"verify.location\" is not set.");
    }
    if (messageProperties_.getProperty("verify.trustprofile") == null) {
      throw new NullPointerException("Property \"verify.trustprofile\" is not set.");
    }
    if (messageProperties_.getProperty("verify.authority") == null) {
      throw new NullPointerException("Property \"verify.authority\" is not set.");
    }
    if (messageProperties_.getProperty("verify.privateKey") == null) {
      throw new NullPointerException("Property \"verify.privateKey\" is not set.");
    }
    if (messageProperties_.getProperty("verify.certificate") == null) {
      throw new NullPointerException("Property \"verify.certificate\" is not set.");
    }
    if (messageProperties_.getProperty("verify.reference") == null) {
      throw new NullPointerException("Property \"verify.reference\" is not set.");
    }
    if (messageProperties_.getProperty("ixsil.initFile") == null) {
      throw new NullPointerException("Property \"ixsil.initFile\" is not set.");
    }
    try {
      // Initialize IXSIL
      IXSILInit.init(new URI(messageProperties_.getProperty("ixsil.initFile")));
    } catch (InitException e) {
      e.printStackTrace();
      throw new ClientException(e);
    } catch (URIException e) {
      e.printStackTrace();
      throw new ClientException(e);
    }
  }

  protected File xmlNotification_ = null;
  
  protected static Logger logger_ = Logger.getLogger(VerifyNotificationFactory.class.getName());
  
  //protected CreateSignatureClient client_ = null;
  
  protected VerifySignatureClient verifyClient_ = null;
  
  protected abstract String getNewSignatureValue();
  
  /*protected abstract String getNewSignatureTime();
  protected abstract String getNewEtsiHash();*/
  
  protected abstract Document createSignature() throws ClientException;
  
  /*
   * Creates the XML signature and sends the signature to the receiver.
   */
  protected String handleXMLNotification() {
    
    try {
      
      logger_.debug("Verify process: creating signature");
      
      Document response = this.createSignature();
      
      /*XMLSerializer s = new XMLSerializer(System.out, new OutputFormat());
      s.serialize(response);*/

      ByteArrayOutputStream out = new ByteArrayOutputStream();
      try {
        /*XMLSerializer serializer = new XMLSerializer(out, new OutputFormat("XML", "UTF-8", false));
        serializer.serialize(response);*/
        Utils.serializeDocument(response, out);
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        builder = factory.newDocumentBuilder();
        response = builder.parse(new ByteArrayInputStream(out.toByteArray()));
      } catch (Exception e1) {
        e1.printStackTrace();
        throw new ClientException(e1);
      }
      
      Utils.checkBase64Signature(this.getNewSignatureValue());
      
      Node signatureValue = XPathAPI.selectSingleNode(response, "/Bescheid/Signatur/Signature/SignatureValue/text()");
      logger_.debug("SIGNATURE VALUE: " + signatureValue.getNodeValue());
      logger_.debug("NEW VALUE: " + this.getNewSignatureValue());
      signatureValue.setNodeValue(this.getNewSignatureValue());

      // ETSI CONFORMANT SIGNATURE
      /*Node signingTime = XPathAPI.selectSingleNode(response, "/Bescheid/Signature/Object[attribute::Id='etsi-signed-1-1']/QualifyingProperties/SignedProperties/SignedSignatureProperties/SigningTime/text()");
      logger_.debug("TIME VALUE: " + signingTime.getNodeValue());
      logger_.debug("NEW VALUE: " + this.getNewSignatureTime());
      signingTime.setNodeValue(this.getNewSignatureTime());

      Node etsiHash = XPathAPI.selectSingleNode(response, "/Bescheid/Signature/SignedInfo/Reference[attribute::Type='http://uri.etsi.org/01903/v1.1.1#SignedProperties']/DigestValue/text()");
      logger_.debug("HASH VALUE: " + etsiHash.getNodeValue());
      logger_.debug("NEW VALUE: " + this.getNewEtsiHash());
      etsiHash.setNodeValue(this.getNewEtsiHash()); */

      
      logger_.debug("Exchanged Signature Value");
//      XMLSerializer s1 = new XMLSerializer(System.out, new OutputFormat());
//      s1.serialize(response);
      
      
      // set the signed document for later retrieving
      httpServletRequest_.getSession().setAttribute("xmldata", response);
      
      // Ok, now verify
      verifyClient_ = ClientFactory.getVerifySignatureClient();
      
      // verify against now
      verifyClient_.setContent(response.getDocumentElement());
      verifyClient_.addAdditionalNamespace("dsig", MoaConstants.DSIG_NS);
      
      verifyClient_.setSignatureLocation(messageProperties_.getProperty("verify.location"));
      verifyClient_.setTrustProfileID(messageProperties_.getProperty("verify.trustprofile"));
      
      VerifyResult result = verifyClient_.sendVerifyRequest(messageProperties_.getProperty("verify.connectionUrl"));
      Name signer = result.getSubjectName();
      httpServletRequest_.setAttribute("SIGNER_NAME", signer.getRDNValue(ObjectID.commonName));
      httpServletRequest_.setAttribute("SIGNER_ORGANIZATIONAL_UNIT", signer.getRDNValue(ObjectID.organizationalUnit));
      httpServletRequest_.setAttribute("SIGNER_ORGANIZATION", signer.getRDNValue(ObjectID.organization));
      httpServletRequest_.setAttribute("SIGNER_COUNTRY", signer.getRDNValue(ObjectID.country));

      Name issuer = result.getIssuerName();
      httpServletRequest_.setAttribute("ISSUER_NAME", issuer.getRDNValue(ObjectID.commonName));
      httpServletRequest_.setAttribute("ISSUER_ORGANIZATIONAL_UNIT", issuer.getRDNValue(ObjectID.organizationalUnit));
      httpServletRequest_.setAttribute("ISSUER_ORGANIZATION", issuer.getRDNValue(ObjectID.organization));
      httpServletRequest_.setAttribute("ISSUER_COUNTRY", issuer.getRDNValue(ObjectID.country));
      
      httpServletRequest_.setAttribute("CERT_SERIAL_NUMBER", result.getIssuerSerial().toString());
      
      if (result.isQualifiedCertificate()) {
        httpServletRequest_.setAttribute("CERT_QUALITY", "Qualifiziertes Zertifikat");
      } else {
        httpServletRequest_.setAttribute("CERT_QUALITY", "Gew&ouml;hnliches Zertifikat");
      }
      
      if (result.getSignatureCheckCode() == 0) {
        httpServletRequest_.setAttribute("SC", "true");
        httpServletRequest_.setAttribute("CHECK_SIGNATURE", "Die berprfung der Hash-Werte und des Werts der Signatur konnte erfolgreich durchgefhrt werden.");
      } else {
        httpServletRequest_.setAttribute("SC", "false");
        httpServletRequest_.setAttribute("CHECK_SIGNATURE", "Die berprfung der Hash-Werte und des Werts der Signatur konnte NICHT erfolgreich durchgefhrt werden.");
      }
      
      if (result.getCertificateCheckCode() == 0) {
        httpServletRequest_.setAttribute("CC", "true");
        httpServletRequest_.setAttribute("CHECK_CERTIFICATE", "Eine formal korrekte Zertifikatskette vom Signatorzertifikat zu einem vertrauenswrdigen Wurzelzertifikat konnte konstruiert werden. Jedes Zertifikat dieser Kette ist zum in der Anfrage angegebenen Prfzeitpunkt gltig.");
      } else {
        httpServletRequest_.setAttribute("CC", "true");
        httpServletRequest_.setAttribute("CHECK_CERTIFICATE", "Eine formal korrekte Zertifikatskette vom Signatorzertifikat zu einem vertrauenswrdigen Wurzelzertifikat konnte NICHT konstruiert werden.");
      }

      // Add link to retrieve the signed data
      httpServletRequest_.setAttribute(VerifyResultConstants.SIGNED_DATA, "showData?sessionid=" + httpServletRequest_.getSession().getId());
      
      httpServletRequest_.setAttribute("SHOW_NOTIFICATION", "true");

      
    } catch (Exception e) {
      e.printStackTrace();
      return getErrorHTMLCode(e);
    }
      
    return this.successHTMLCode();
    //return getErrorHTMLCode(new ClientException("Exceptino..."));
  }
  
  protected abstract String getErrorHTMLCode(Exception e);
  
  protected abstract String successHTMLCode();
  
	/* (non-Javadoc)
	 * @see asit.framework.servlet.content.ContentFactory#getContent()
	 */
	public String getContent() {
	  return this.handleXMLNotification();
	}


}
