/*
 * Version 1.0, 2004-02-02
 * 
 * (c) Stabsstelle IKT-Strategie des Bundes, Bundeskanzleramt Jede Art von
 * Verwendung und Modifikation erlaubt.
 * 
 * encodeBase64Encode() - (c) Apache Software Foundation, http://www.apache.org/
 * License see bottom of file
 *  
 */
package asit.framework.utils;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * @author troessler
 * 
 *  
 */
public class BPK {
    /**
     * 
     * @param stammzahl
     * @param context
     * @return @throws
     *         Exception
     * @throws UnsupportedEncodingException 
     * @throws NoSuchAlgorithmException 
     */
    public static String calcBPK(String stammzahl, String context,
            String encoding) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        // prefix according to specification
        String prefix = "urn:publicid:gv.at:cdid+";
        // concatenation according to algorithm
        String data = stammzahl + "+" + prefix + context;
        return calcHash(data, encoding);
    }

    /**
     * 
     * @param stammzahl
     * @param company
     * @return @throws
     *         Exception
     */
    public static String calcWBPK(String stammzahl, String company,
            String encoding) throws Exception {
        // prefix according to specification
        String prefix = "urn:publicid:gv.at:wbpk+FN+";
        // concatenation according to algorithm
        String data = stammzahl + "+" + prefix + company;
        return calcHash(data, encoding);
    }

    /**
     * 
     * @param data
     * @return @throws
     *         Exception
     * @throws NoSuchAlgorithmException 
     * @throws UnsupportedEncodingException 
     */
    public static String calcHash(String data, String encoding) throws NoSuchAlgorithmException, UnsupportedEncodingException
             {
        MessageDigest sha1 = MessageDigest.getInstance("SHA-1");
        byte[] digest = sha1.digest(data.getBytes(encoding));
        // quick and dirty binary to hex converter
        for (int i = 0; i < digest.length; i++) {
            String val = Integer.toHexString((int) digest[i] & 0xff);
            if (val.length() == 1) // q&d : add leading zeros
                val = "0" + val;
        }
        byte[] base64 = encodeBase64(digest);
        return (new String(base64));
    }

    public static void main(String[] args) throws Exception {
        String stammzahl, context, company;
        stammzahl = (args.length >= 1) ? args[0] : "MDEyMzQ1Njc4OWFiY2RlZg==";
        context = (args.length >= 2) ? args[1] : "BW";
        company = (args.length >= 3) ? args[2] : "468924i";
//        System.out.println("Stammzahl: " + stammzahl + "\n");
//        System.out.println("bPK: "
//                + calcBPK(stammzahl, context, System
//                        .getProperty("file.encoding", "ISO-8859-1")) + "\n");
//        System.out.println("wbPK: "
//                + calcWBPK(stammzahl, company, System
//                        .getProperty("file.encoding", "ISO-8859-1")) + "\n");
    }

    /*
     * The Apache Software License, Version 1.1
     * 
     * Copyright (c) 2001-2004 The Apache Software Foundation. All rights
     * reserved.
     * 
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions are
     * met:
     * 
     * 1. Redistributions of source code must retain the above copyright notice,
     * this list of conditions and the following disclaimer.
     * 
     * 2. Redistributions in binary form must reproduce the above copyright
     * notice, this list of conditions and the following disclaimer in the
     * documentation and/or other materials provided with the distribution.
     * 
     * 3. The end-user documentation included with the redistribution, if any,
     * must include the following acknowledgement: "This product includes
     * software developed by the Apache Software Foundation
     * (http://www.apache.org/)." Alternately, this acknowledgement may appear
     * in the software itself, if and wherever such third-party acknowledgements
     * normally appear.
     * 
     * 4. The names "Apache", "The Jakarta Project", "Commons", and "Apache
     * Software Foundation" must not be used to endorse or promote products
     * derived from this software without prior written permission. For written
     * permission, please contact apache@apache.org.
     * 
     * 5. Products derived from this software may not be called "Apache" nor may
     * "Apache" appear in their name without prior written permission of the
     * Apache Software Foundation.
     * 
     * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
     * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
     * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
     * NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR ITS CONTRIBUTORS BE
     * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
     * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
     * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
     * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
     * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
     * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
     * THE POSSIBILITY OF SUCH DAMAGE.
     * ====================================================================
     * 
     * This software consists of voluntary contributions made by many
     * individuals on behalf of the Apache Software Foundation. For more
     * information on the Apache Software Foundation, please see
     * <http://www.apache.org/>.
     *  
     */
    /**
     * Encodes hex octects into Base64.
     * 
     * Function talen from the file
     * /jakarta-commons/codec/src/java/org/apache/commons/codec/binary/Base64.java
     * License see above
     * 
     * Slightly modified (removed constants and isChunked logic) --Arno
     * 
     * @param binaryData
     *            Array containing binary data to encode.
     * @return Base64-encoded data.
     */
    public static byte[] encodeBase64(byte[] binaryData) {
        int lengthDataBits = binaryData.length * 8;
        int fewerThan24bits = lengthDataBits % 24;
        int numberTriplets = lengthDataBits / 24;
        byte encodedData[] = null;
        int encodedDataLength = 0;
        int nbrChunks = 0;
        byte[] lookUpBase64Alphabet = new byte[64];
        // init lookup alphabet
        for (int i = 0; i <= 25; i++)
            lookUpBase64Alphabet[i] = (byte) ('A' + i);
        for (int i = 26, j = 0; i <= 51; i++, j++)
            lookUpBase64Alphabet[i] = (byte) ('a' + j);
        for (int i = 52, j = 0; i <= 61; i++, j++)
            lookUpBase64Alphabet[i] = (byte) ('0' + j);
        lookUpBase64Alphabet[62] = (byte) '+';
        lookUpBase64Alphabet[63] = (byte) '/';
        if (fewerThan24bits != 0) {
            //data not divisible by 24 bit
            encodedDataLength = (numberTriplets + 1) * 4;
        }
        else {
            // 16 or 8 bit
            encodedDataLength = numberTriplets * 4;
        }
        encodedData = new byte[encodedDataLength];
        byte k = 0, l = 0, b1 = 0, b2 = 0, b3 = 0;
        int encodedIndex = 0;
        int dataIndex = 0;
        int i = 0;
        int chunksSoFar = 0;
        for (i = 0; i < numberTriplets; i++) {
            dataIndex = i * 3;
            b1 = binaryData[dataIndex];
            b2 = binaryData[dataIndex + 1];
            b3 = binaryData[dataIndex + 2];
            l = (byte) (b2 & 0x0f);
            k = (byte) (b1 & 0x03);
            byte val1 = ((b1 & 0x80) == 0) ? (byte) (b1 >> 2)
                    : (byte) ((b1) >> 2 ^ 0xc0);
            byte val2 = ((b2 & 0x80) == 0) ? (byte) (b2 >> 4)
                    : (byte) ((b2) >> 4 ^ 0xf0);
            byte val3 = ((b3 & 0x80) == 0) ? (byte) (b3 >> 6)
                    : (byte) ((b3) >> 6 ^ 0xfc);
            encodedData[encodedIndex] = lookUpBase64Alphabet[val1];
            encodedData[encodedIndex + 1] = lookUpBase64Alphabet[val2
                    | (k << 4)];
            encodedData[encodedIndex + 2] = lookUpBase64Alphabet[(l << 2)
                    | val3];
            encodedData[encodedIndex + 3] = lookUpBase64Alphabet[b3 & 0x3f];
            encodedIndex += 4;
        }
        // form integral number of 6-bit groups
        dataIndex = i * 3;
        if (fewerThan24bits == 8) {
            b1 = binaryData[dataIndex];
            k = (byte) (b1 & 0x03);
            byte val1 = ((b1 & 0x80) == 0) ? (byte) (b1 >> 2)
                    : (byte) ((b1) >> 2 ^ 0xc0);
            encodedData[encodedIndex] = lookUpBase64Alphabet[val1];
            encodedData[encodedIndex + 1] = lookUpBase64Alphabet[k << 4];
            encodedData[encodedIndex + 2] = '=';
            encodedData[encodedIndex + 3] = '=';
        }
        else if (fewerThan24bits == 16) {
            b1 = binaryData[dataIndex];
            b2 = binaryData[dataIndex + 1];
            l = (byte) (b2 & 0x0f);
            k = (byte) (b1 & 0x03);
            byte val1 = ((b1 & 0x80) == 0) ? (byte) (b1 >> 2)
                    : (byte) ((b1) >> 2 ^ 0xc0);
            byte val2 = ((b2 & 0x80) == 0) ? (byte) (b2 >> 4)
                    : (byte) ((b2) >> 4 ^ 0xf0);
            encodedData[encodedIndex] = lookUpBase64Alphabet[val1];
            encodedData[encodedIndex + 1] = lookUpBase64Alphabet[val2
                    | (k << 4)];
            encodedData[encodedIndex + 2] = lookUpBase64Alphabet[l << 2];
            encodedData[encodedIndex + 3] = '=';
        }
        return encodedData;
    }
}