// Created on 19.11.2004 by troessler for the SeLaTest project
//
// Copyright (C) 2004 A-SIT
//http://www.a-sit.at
//
//All rights reserved.
//
//This source is provided for inspection purposes and recompilation only,
//unless specified differently in a contract with ASIT. This source has to
//be kept in strict confidence and must not be disclosed to any third party
//under any circumstances. Redistribution in source and binary forms, with
//or without modification, are <not> permitted in any case!
//
//THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
//ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
//ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
//FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
//OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
//HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
//LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
//OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//SUCH DAMAGE.
package asit.framework.xml;

import java.io.ByteArrayInputStream;
import java.util.Hashtable;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author troessler
 */
public class ElementNameTranslationTable {

  private final int SIMPLENAME = 0;
  private final int QUALIFIEDNAME = 1;

  public static final String SIGNATURE_ELEMENT_NAME = "Signature";
  
  Hashtable nodeFullNameTable_;

  /**
   * @param xmldocument
   */
  public ElementNameTranslationTable(Document xmldocument) {
    this((Node) xmldocument.getDocumentElement());
  }

  /**
   * @param xmldocument
   */
  public ElementNameTranslationTable(Node rootnode) {

    Element rootElem = (Element) rootnode;
    NodeList nodelist = rootElem.getElementsByTagName("*");
    nodeFullNameTable_ = new Hashtable();
    putElementQNameInTable(rootElem);
    for (int nodeidx = nodelist.getLength() - 1; nodeidx >= 0; nodeidx--) {
      putElementQNameInTable(nodelist.item(nodeidx));
    }
  }
  
  /**
   * Returns the full qualified name of an element by the use of the Hashtable
   * full of element names
   * 
   * @param elementsimplename
   * @param index
   *          position in document
   * @return Qname of the requested element if exists (at the requested
   *         position), null in any other case
   */
  public String getElementsQualifiedName(String elementsimplename, int index) {
    Vector result = (Vector) this.nodeFullNameTable_.get(elementsimplename);
    if (result == null)
      return null;
    if (index >= result.size())
      return null;

    return (String) ((Vector) this.nodeFullNameTable_.get(elementsimplename))
        .get(index);
  }

  /**
   * Returns the full qualified name of an element by the use of the Hashtable
   * full of element names - first occurence
   * 
   * @param elementsimplename
   * @return
   */
  public String getElementsQualifiedName(String elementsimplename) {
    return getElementsQualifiedName(elementsimplename, 0);
  }

  /**
   * Returns the NS-prefix of an element by the use of the Hashtable full of
   * element names
   * 
   * @param elementsimplename
   * @return
   */
  public String getElementsNSPrefix(String elementsimplename, int index) {
    String qname = getElementsQualifiedName(elementsimplename, index);
    if (qname == null)
      return null;
    return qname.substring(0, qname.indexOf(':'));
  }

  /**
   * Returns the NS-prefix of an element by the use of the Hashtable full of
   * element names - first occurence
   * 
   * @param elementsimplename
   * @return
   */
  public String getElementsNSPrefix(String elementsimplename) {
    return getElementsNSPrefix(elementsimplename, 0);
  }

  /**
   * @param node
   * @return String[] [0]..node's simple name [1]..node's qualified name
   */
  private String[] retrievePlainElementName(Node node) {
    String nodefullname = node.getNodeName();
    int prefixindex = nodefullname.indexOf(':');
    String elementname = nodefullname;
    if (prefixindex > 0)
      elementname = nodefullname.substring(prefixindex + 1);
    String[] result = new String[2];
    result[SIMPLENAME] = elementname;
    result[QUALIFIEDNAME] = nodefullname;
    return result;
  }

  /**
   * Adds an element-name to the table (only if the given node is of type
   * ELEMENT_NODE)
   * 
   * @param node
   */
  private void putElementQNameInTable(Node node) {
    if (node.getNodeType() == Node.ELEMENT_NODE) {
      String[] nodenames = retrievePlainElementName(node);
      if (this.nodeFullNameTable_.contains(nodenames[SIMPLENAME])) {
        Vector existingentries = (Vector) this.nodeFullNameTable_
            .get(nodenames[SIMPLENAME]);
        existingentries.add(nodenames[QUALIFIEDNAME]);
        this.nodeFullNameTable_.put(nodenames[SIMPLENAME], existingentries);
      }
      else {
        Vector newentry = new Vector();
        newentry.add(nodenames[QUALIFIEDNAME]);
        this.nodeFullNameTable_.put(nodenames[SIMPLENAME], newentry);
      }
    }
  }

}