package asit.moa.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import asit.common.Utils;
import asit.moa.client.exception.ClientException;
import asit.moa.client.exception.RequestException;

public abstract class AbstractClient implements Client {

  protected Document doc_ = null;
  protected String keyIdentifier_ = null;
  protected String defaultAddress_ = null;
  
  static Log logger_ = LogFactory.getLog(AbstractClient.class.getName());
  
  protected String sslTrustStore_ = null;
  protected String sslTrustStoreType_ = null;
  protected String sslTrustStorePassword_ = null;
  
  protected String clientAuthStore_ = null;
  protected String clientAuthStoreType_ = null;
  protected String clientAuthStorePassword_ = null;
  
  protected SSLSocketFactory factory_ = null;
  
  protected Hashtable headers_ = new Hashtable();
  
  
  public QName qname_service_;
  private boolean hostNameVerifier_ = false;  
  
  public void addCustomHeader(String name, String value) {
    if (name != null) {
      headers_.put(name, value);
    }
  }
  
	public AbstractClient() throws ClientException {
	  doc_ = AbstractClient.createNewDocument();
  }
  
  public void setSSLSocketFactory(SSLSocketFactory factory) {
    factory_ = factory;
  }
  
  public void setSSLTrustStore(String fileLocation, String password, String type) {
    logger_.debug("Settings Trust SSL Keystore and Password for Client: " + fileLocation);
    sslTrustStore_ = fileLocation;
    sslTrustStorePassword_ = password;
    sslTrustStoreType_ = type;
  }
  
  public void setClientAuthStore(String fileLocation, String password) {
    logger_.debug("Settings Client SSL Keystore and Password for Client: " + fileLocation);
    clientAuthStore_ = fileLocation;
    clientAuthStorePassword_ = password;
  }
  
  
  public void setHostNameVerifierEnabled(boolean value) {
    hostNameVerifier_ = value;
  }
  public static Document createNewDocument() throws ClientException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setNamespaceAware(true);
    DocumentBuilder builder;
    try {
      builder = factory.newDocumentBuilder();
    } catch (ParserConfigurationException e) {
      e.printStackTrace();
      throw new ClientException(e);
    }
    return builder.newDocument();
  }
  
  protected Element getSOAPBody() throws ClientException {
    doc_ = AbstractClient.createNewDocument();
    Element root = doc_.createElementNS(SOAPConstants.SOAP_ENV_NS, SOAPConstants.SOAP_ENV_PREFIX + SOAPConstants.ENVELOPE);
    doc_.appendChild(root);
    
    root.setAttribute("xmlns" + SOAPConstants.SOAP_ENV_POSTFIX, SOAPConstants.SOAP_ENV_NS);
    //root.setAttribute(SOAPConstants.SOAP_ENV_PREFIX + SOAPConstants.ENCODING_STYLE, SOAPConstants.SOAP_ENV_ENCODING_STYLE);
    root.setAttribute("xmlns:xsd", "http://www.w3.org/2001/XMLSchema");
    root.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
        
    Element body = doc_.createElementNS(SOAPConstants.SOAP_ENV_NS, SOAPConstants.SOAP_ENV_PREFIX + SOAPConstants.BODY);
    root.appendChild(body);
    
    return body;
  }
  
  
  /**
   * 
   * @return
   * @throws ClientException
   * @throws RequestException
   */
  public Element sendRequest() throws ClientException,
      RequestException {
    return sendRequest(this.defaultAddress_);
  }
  
  
  public Element sendRequest(String address) throws ClientException {
    logger_.debug("Encoding: " + System.getProperty("file.encoding"));
    return sendRequest(address, this.getRequest()); 
  }
  

  public Element sendRequest(String address, Element request) throws ClientException {
    logger_.debug("Encoding: " + System.getProperty("file.encoding"));
    // sichern, dass nicht alte SUN JSSE Implementierung verwendet wird
    System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
    //System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
    
      if (address == null) {
        throw new NullPointerException("Argument \"address\" must not be null.");
      }
      //Service service = ServiceFactory.newInstance().createService(qname_service_);
      
      //Call call = service.createCall();
      
      
    
      Element body = getSOAPBody();
      Node importedNode = doc_.importNode(request, true);
      body.appendChild(importedNode);
      
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      logger_.debug("Request:");

      try {
        // Serialize document
        Utils.serializeDocument(doc_, out);
        // log it
        logger_.debug(out.toString());
      } catch (IOException e) {
        logger_.info("Error while serializing request document.");
        throw new ClientException(e);
      }
      
      // set protocol handler
      return sendRequest(address, new ByteArrayInputStream(out.toByteArray()));
    
  }
  
  public Element sendRequest(String address, InputStream is) throws ClientException {
    
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      
      try {
        Utils.dumpInputOutputStream(is, out);
      } catch(Exception e) {
        throw new ClientException(e);
      }
      
      
      logger_.debug("Trying to connect to: " + address);
      URL url;
      HttpURLConnection conn;
      int responseCode;
      String responseMessage = null;
      try {
        logger_.debug("ProtocolHandler: " + System.getProperty("java.protocol.handler.pkgs"));
        url = new URL(address);
        
        conn = (HttpURLConnection)url.openConnection();
        
        logger_.debug("Instance: " + conn.getClass().getName());
        logger_.debug("ProtocolHandler: " + System.getProperty("java.protocol.handler.pkgs"));
        logger_.debug("Got Instance: " + conn.getClass().getName());
        logger_.debug("Java version: " + System.getProperty("java.vm.version"));
        logger_.debug("Java boot: " + System.getProperty("sun.boot.library.path"));
        
        boolean keyStorePresent = System.getProperty("javax.net.ssl.keyStore") != null; 
        
        if (conn instanceof com.sun.net.ssl.HttpsURLConnection && !hostNameVerifier_) {
           logger_.debug("Connection is instance of HttpsURLConnection (SUN).");
           if (!keyStorePresent) {
         	  logger_.debug("Keystore has not been set by system property \"javax.net.ssl.keyStore\". Trying to manually set ssl parameters...");
              try {
                 if (factory_ != null) {
                   ((com.sun.net.ssl.HttpsURLConnection)conn).setSSLSocketFactory(factory_);
                 } else {
                   ((com.sun.net.ssl.HttpsURLConnection)conn).setSSLSocketFactory(Utils.configureHTTPSConnection(clientAuthStore_, clientAuthStorePassword_, clientAuthStoreType_, sslTrustStore_, sslTrustStorePassword_, sslTrustStoreType_));
                 }
               } catch(Exception e) {
                 e.printStackTrace();
               }
           } else {
         	  logger_.debug("Keystore has been set by system property \"javax.net.ssl.keyStore\". Skipping ssl setup...");
           }
               
         ((com.sun.net.ssl.HttpsURLConnection)conn).setHostnameVerifier(new com.sun.net.ssl.HostnameVerifier() {

           public boolean verify(String arg0, String arg1) {
             // TODO Auto-generated method stub
             return true;
           }
           
         });
         } else if (conn instanceof HttpsURLConnection && !hostNameVerifier_) {
            logger_.debug("Connection is instance of HttpsURLConnection (SUN).");
            if (!keyStorePresent) {
          	  logger_.debug("Keystore has not been set by system property \"javax.net.ssl.keyStore\". Trying to manually set ssl parameters...");
              try {
               if (factory_ != null) {
                 ((HttpsURLConnection)conn).setSSLSocketFactory(factory_);
               } else {
                 ((HttpsURLConnection)conn).setSSLSocketFactory(Utils.configureHTTPSConnection(clientAuthStore_, clientAuthStorePassword_, clientAuthStoreType_, sslTrustStore_, sslTrustStorePassword_, sslTrustStoreType_));
               }
             } catch(Exception e) {
               e.printStackTrace();
             }
             
            } else {
          	  logger_.debug("Keystore has been set by system property \"javax.net.ssl.keyStore\". Skipping ssl setup...");
            }
            ((HttpsURLConnection)conn).setHostnameVerifier(new HostnameVerifier() {

               public boolean verify(String arg0, SSLSession arg1) {
                 return true;
               }
             });
         }
        
        conn.setRequestMethod("POST");
        conn.setUseCaches(false);
        conn.setDoOutput(true);
        
        conn.setRequestProperty("Content-Length", String.valueOf(out.size()));
//        conn.setRequestProperty("Content-Type", "text/xml;charset=utf-8");
        conn.setRequestProperty("Content-Type", "text/xml");
        
        // setting an empty SOAPAction header
        conn.setRequestProperty("SOAPAction", "");
        
        // set custom headers
        Enumeration customHeaders = headers_.keys();
        String key;
        while (customHeaders.hasMoreElements()) {
          key = (String)customHeaders.nextElement();
          conn.setRequestProperty(key, (String)headers_.get(key));
        }
        
        out.writeTo(conn.getOutputStream());
        conn.getOutputStream().flush();
        conn.connect();
        responseCode = conn.getResponseCode();
        
        logger_.debug("Response Code: " + responseCode);
        responseMessage = conn.getResponseMessage();
        logger_.debug("Response message: " + responseMessage);
      } catch (MalformedURLException e1) {
        throw new ClientException(e1);
      } catch (IOException e) {
        throw new ClientException(e);
      }
      
      
      

      
      
      // Something has gone wrong
      if (responseCode != 200) {
        
        logger_.debug("Did not receive a 200 HTTP OK code.");
        
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
          Utils.dumpInputOutputStream(conn.getErrorStream(), bos);
          logger_.debug("Response:");
          logger_.debug(bos.toString());
        } catch (IOException e3) {
          e3.printStackTrace();
        }
        
        // Try to fetch out the error message
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        DocumentBuilder builder;
        Document doc = null;
        try {
          builder = factory.newDocumentBuilder();
          doc = builder.parse(new ByteArrayInputStream(bos.toByteArray()));
        } catch (Exception e) {
          logger_.debug("Error while parsing error stream.");
        }
        
        if (doc != null) {
        
          // get soap prefix
          String soapPrefix = doc.getDocumentElement().getPrefix();
          if (soapPrefix == null) {
            soapPrefix = "";
          }
          
          String selectString = "/" + soapPrefix + ":Envelope/" + soapPrefix + ":Body/" + soapPrefix + ":Fault/faultstring/text()";
          
          Node faultString = null;
          try {
            faultString = XPathAPI.selectSingleNode(doc, selectString);
          } catch (TransformerException e2) {
            logger_.debug("Error while parsing out faultstring.");
          }
          if (faultString != null &&
              faultString.getNodeValue() != null) {
            logger_.debug("Received fault code: " + faultString.getNodeValue());
            throw new ClientException(faultString.getNodeValue());
          }
        }
        
        throw new ClientException("Server error: " + responseCode + " / " + responseMessage);
      }
      
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(true);
      
      
      
      DocumentBuilder builder;
      try {
        builder = factory.newDocumentBuilder();
        logger_.debug("Reading in parser mode.");
        
        //InputStreamReader utf8Reader = new InputStreamReader(conn.getInputStream(), "UTF-8");
        //ByteArrayOutputStream bos = new ByteArrayOutputStream();
        //Utils.dumpInputOutputStream(utf8Reader, bos);
        
        //InputSource source = new InputSource(new InputStreamReader(conn.getInputStream(), "UTF-8"));
        
        Element response = builder.parse(conn.getInputStream()).getDocumentElement();
        
        //Element response = builder.parse(new ByteArrayInputStream(bos.toString("UTF-8").getBytes())).getDocumentElement();
        // Write response to logger
        logger_.debug("");
        logger_.debug("Received Response document:");
        logger_.debug("");
        ByteArrayOutputStream responseOutputStream = new ByteArrayOutputStream();
        Utils.serializeElement(response, responseOutputStream);
        logger_.debug(responseOutputStream.toString());
        
        String soapPrefix = response.getPrefix();
        
        NodeList resp = XPathAPI.selectNodeList(response, "/" + soapPrefix + ":Envelope/" + soapPrefix + ":Body/child::node()[1]");
        
        logger_.debug("ResponseElements: " + resp.getLength());
        if (resp.getLength() > 0) {
          logger_.debug("Found envelope content: " + ((Element)resp.item(0)).getTagName());
          return (Element)resp.item(0);
        } else {
          throw new ClientException("Error while retrieving content from envelope.");
        }
        

      } catch (Exception e) {
        throw new ClientException(e);
      }   
      
  }
 
  
  /**
   * 
   * @param address
   */
  public void setDefaultAddress(String address) {
    if (address == null) {
      throw new NullPointerException("Argument \"address\" must not be null");
    }
    this.defaultAddress_ = address;
  }
  
}
