package asit.moa.client;

import java.util.Hashtable;
import asit.moa.client.deliver.DeliveryClient;
import asit.moa.client.deliver.DeliveryHandler;
import asit.moa.client.deliver.mail.MailDeliveryHandler;
import asit.moa.client.deliver.moazs.MOAZSDeliveryHandler;
import asit.moa.client.deliver.robot.RobotDeliveryHandler;
import asit.moa.client.exception.ClientException;
import asit.moa.client.id.MOAIDClient;
import asit.moa.client.id.MOAIDClientImpl;
import asit.moa.client.sign.CreateSignatureClient;
import asit.moa.client.sign.CreateSignatureClientImpl;
import asit.moa.client.sign.MetaInfo;
import asit.moa.client.sign.MetaInfoImpl;
import asit.moa.client.sign.SignReference;
import asit.moa.client.sign.SignReferenceImpl;
import asit.moa.client.sign.SingleSignatureInfo;
import asit.moa.client.sign.SingleSignatureInfoImpl;
import asit.moa.client.status.StatusClient;
import asit.moa.client.status.StatusClientImpl;
import asit.moa.client.verify.VerifySignatureClient;
import asit.moa.client.verify.VerifySignatureClientImpl;


/**
 * @author atauber
 */
public class ClientFactory {

  private static ClientFactory instance_ = null;
  
  protected static Hashtable deliveryHandlers_ = new Hashtable();
  
  protected ClientFactory() {
    registerDeliveryHandler(new MailDeliveryHandler());
    registerDeliveryHandler(new RobotDeliveryHandler());
    registerDeliveryHandler(new MOAZSDeliveryHandler());
  }
  
  public synchronized void registerDeliveryHandler(DeliveryHandler handler) {
    if (handler == null) {
      throw new NullPointerException("Handler must not be null.");
    }
    deliveryHandlers_.put(handler.getType(), handler);
  }
  
  public synchronized static CreateSignatureClient getSignatureClient() throws ClientException {
    // create thread safe singleton instance
    if (instance_ == null) {
      instance_ = new ClientFactory();
    }
    return new CreateSignatureClientImpl();
  }
  
  public synchronized static StatusClient getStatusClient() throws ClientException {
    // create thread safe singleton instance
    if (instance_ == null) {
      instance_ = new ClientFactory();
    }
    return new StatusClientImpl();
  }
  
  public synchronized static MOAIDClient getMOAIDClient() throws ClientException {
    // create thread safe singleton instance
    if (instance_ == null) {
      instance_ = new ClientFactory();
    }
    return new MOAIDClientImpl();
  }
  
  public synchronized static VerifySignatureClient getVerifySignatureClient() throws ClientException {
    // create thread safe singleton instance
    if (instance_ == null) {
      instance_ = new ClientFactory();
    }
    return new VerifySignatureClientImpl();
  }
  
  // use one of DeliveryTypes as type parameter
  public synchronized static DeliveryClient getDeliveryClient(String type) throws ClientException {
    if (type == null) {
      throw new NullPointerException("DeliveryType must not be null.");
    }
    //  create thread safe singleton instance
    if (instance_ == null) {
      instance_ = new ClientFactory();
    }  
    DeliveryClient client = ((DeliveryHandler)deliveryHandlers_.get(type)).getDeliveryClient();
    
    if (client == null) {
      throw new ClientException("No delivery client of type " + type + " registered.");
    }
    return client;
    
  }
  
  public synchronized static SignReference getSignReference() {
    // create thread safe singleton instance
    if (instance_ == null) {
      instance_ = new ClientFactory();
    }
    return new SignReferenceImpl();
  }
  
  public synchronized static SingleSignatureInfo getSingleSignatureInfo() {
    // create thread safe singleton instance
    if (instance_ == null) {
      instance_ = new ClientFactory();
    }
    return new SingleSignatureInfoImpl();
  }
  
  public synchronized static MetaInfo getMetaInfo(String mimeType, String description) {
    // create thread safe singleton instance
    if (instance_ == null) {
      instance_ = new ClientFactory();
    }
    return new MetaInfoImpl(mimeType, description);
  }
  
}
