package asit.moa.client.deliver.address;

import iaik.ixsil.util.DOMUtilsImpl;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import asit.bescheid.BescheidConstants;
import asit.bescheid.BescheidElement;


public class Behoerde implements BescheidElement, Serializable {

  // behoerde
  protected KompNichtNatuerlichePerson behoerde_ = new KompNichtNatuerlichePerson();
  
  // liste von sachbearbeiter
  protected ArrayList sachbearbeiter_ = new ArrayList();
  
  protected String file_ = null;
  protected String behoerdenFile_ = null;
  protected String sachbearbeiterFile_ = null;
  
  protected static Log log_ = LogFactory.getLog(Behoerde.class.getName()); 
  
  
  public void loadDefaults(ServletContext context) throws ParserConfigurationException, SAXException, IOException {
    
    file_ = context.getRealPath("/WEB-INF/res/behoerde/");
    
    File f = new File(file_);
    if (!f.exists()) {
      f.mkdirs();
    }
    
    behoerdenFile_ = file_ + "/behoerde.xml";
    
    File behoerdenFile = new File(behoerdenFile_);
    
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setNamespaceAware(true);
    DocumentBuilder builder = factory.newDocumentBuilder();
    
    
    // update behoerde
    if (behoerdenFile.exists()) {
      update(builder.parse(behoerdenFile_).getDocumentElement());
    }
  }
  
  public void saveAll() {
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(false);
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document doc = builder.newDocument();
      Element toSave = toElement(doc);
      toSave.setAttribute("xmlns:" + PersonConstants.PERSON_NS_PREFIX, PersonConstants.PERSON_NS);
      toSave.setAttribute("xmlns:" + BescheidConstants.BESCHEID_NS_PREFIX, BescheidConstants.BESCHEID_NS);
      DOMUtilsImpl.serializeElement(toSave, new FileOutputStream(behoerdenFile_));
      
    } catch(Exception e) {
      e.printStackTrace();
    }
  }
  
  public String getLeadingZero(int i) {
    if (i < 10) {
      return "0" + i;
    } else {
      return "" + i;
    }
  }
  
  public KompNichtNatuerlichePerson getBehoerde() {
    return behoerde_;
  }
  
  public ArrayList getSachbearbeiter() {
    return sachbearbeiter_;
  }

  /* (non-Javadoc)
   * @see asit.bescheid.BescheidElement#toElement(org.w3c.dom.Document)
   */
  public Element toElement(Document doc) {
    
    Element aussteller = doc.createElementNS(BescheidConstants.BESCHEID_NS, BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.AUSSTELLER);
    
    // add behoerde
    behoerde_.setElementName("b:Behoerde");
    aussteller.appendChild(behoerde_.toElement(doc));
    
    // add sachbearbeiter
    for (int i=0; i<sachbearbeiter_.size(); i++) {
      KompNatuerlichePerson sachbearbeiter = (KompNatuerlichePerson)sachbearbeiter_.get(i);
      
      Sachbearbeiter bearbeiter = new Sachbearbeiter();
      bearbeiter.familienname_ = sachbearbeiter.getPFamilienname();
      bearbeiter.vorname_ = sachbearbeiter.getPVorname();
      bearbeiter.affix_ = sachbearbeiter.getPAffix();
      
      
      aussteller.appendChild(bearbeiter.toElement(doc));
      
    }
    
    return aussteller;
  }

  /* (non-Javadoc)
   * @see asit.bescheid.BescheidElement#update(org.w3c.dom.Element)
   */
  public void update(Element element) {
    
    try {
      // read out behoerde
      Node behoerde = XPathAPI.selectSingleNode(element, "descendant-or-self::" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.BEHOERDE);
      if (behoerde != null) {
        log_.debug("Found Behoerde node: " + behoerde.getNodeValue());
        behoerde_ = new KompNichtNatuerlichePerson();
        behoerde_.update((Element)behoerde);
      }

      // read out sachbearbeiter
      NodeList sachbearbeiter = XPathAPI.selectNodeList(element, "descendant-or-self::" + BescheidConstants.BESCHEID_NS_PREFIX + ":" + BescheidConstants.SACHBEARBEITER);
      if (sachbearbeiter.getLength() > 0) {
        log_.debug("Found Sachbearbeiter nodes: " + behoerde.getNodeValue());
        
        for (int i=0; i<sachbearbeiter.getLength(); i++) {
          if (sachbearbeiter.item(i) instanceof Element) {
            Sachbearbeiter sb = new Sachbearbeiter();
            sb.update((Element)sachbearbeiter.item(i));
            sachbearbeiter_.add(sb);
          }          
        }
      }

      
    } catch(TransformerException e) {
      e.printStackTrace();
    }
  }

}
