package asit.moa.client.deliver.mail;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.BodyPart;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.Message.RecipientType;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import org.w3c.dom.Element;
import asit.bescheid.NotificationUtils;
import asit.common.BufferDataSource;
import asit.common.Utils;
import asit.moa.client.AbstractClient;
import asit.moa.client.deliver.DeliveryClient;
import asit.moa.client.deliver.address.Adressat;
import asit.moa.client.deliver.address.KompNatuerlichePerson;
import asit.moa.client.deliver.moazs.BinaryDocument;
import asit.moa.client.exception.ClientException;
import asit.moa.client.exception.DeliveryException;


/**
 * @author atauber
 */
public class MailDeliveryClient extends AbstractClient implements DeliveryClient {


  protected String mailSubject_ = "Zustellung";
  protected String mailSenderName_ = "no-reply@zustellung.gv.at";
  protected String mailSenderAddress_;
  
  protected String mailText_ = "Folgende Anlagen sind enthalten:\n\n";
  
  protected Properties properties_ = null;
  
  protected Adressat personData_ = null;
  
  protected ArrayList binaryDocuments_ = new ArrayList();
  
  protected int defaultPort_ = 25;
  
  protected String htmlBody_ = null;
  protected String htmlBodyCharset_ = null;
  
  protected List bcc_ = new ArrayList();
  /**
   * @throws ClientException
   */
  public MailDeliveryClient() throws ClientException {
    super();
    defaultAddress_ = "mail1.iaik.at";
  }
  
  public void addBCCRecipient(String recipient) {
    bcc_.add(recipient);
  }
  
  public void setHTMLBody(String content) {
    htmlBody_ = content;
  }
  
  public void setHTMLBody(String content, String charset) {
    htmlBody_ = content;
    htmlBodyCharset_ = charset;
  }
  
  public void setDefaultPort(int port) {
    defaultPort_ = port;
  }
  
  public void deliver() throws DeliveryException {
    
    if (personData_ == null) {
      throw new DeliveryException("No receiver for e-mail delivery specified.");
    }
    
    if (NotificationUtils.isEmpty(personData_.getAInternetAdress())) {
      throw new DeliveryException("No e-mail for delivery specified.");
    }
    
    BodyPart messageBodyPart = new MimeBodyPart();
    
    try {
      

      Properties props = new Properties();
      
//      System.out.println("SMTP DEBUG: " + defaultAddress_ + "/" + defaultPort_);
      
      props.put("mail.smtp.host", defaultAddress_);
      props.put("mail.smtp.port", String.valueOf(defaultPort_));
      Session session = Session.getInstance(props, null);
      
      Message message = new MimeMessage(session);
      message.setFrom(new javax.mail.internet.InternetAddress(mailSenderAddress_, mailSenderName_, "UTF-8"));
      
      String recString = personData_.getAInternetAdress();
      String add = "";
      if (personData_.getPerson() != null) {
        
        if (personData_.getPerson() instanceof KompNatuerlichePerson) {
          KompNatuerlichePerson p = (KompNatuerlichePerson)personData_.getPerson();
          if (p.getPVorname() != null) {
            add += p.getPVorname();
            if (p.getPFamilienname() != null) {
              add += " ";
            }
          }
          if (p.getPFamilienname() != null) {
            add += p.getPFamilienname();
          }
        }
        
      }
      if ("".equals(add)) {
        message.addRecipient(RecipientType.TO, new javax.mail.internet.InternetAddress(recString));
      } else {
        message.addRecipient(RecipientType.TO, new javax.mail.internet.InternetAddress(recString, add, "UTF-8"));
      }
      
      // blind copies
      for (int i=0; i<bcc_.size(); i++) {
        String bcc = (String)bcc_.get(i);
        message.addRecipient(RecipientType.BCC, new javax.mail.internet.InternetAddress(bcc));
      }
      
      message.setSubject(mailSubject_);
      

      Multipart multipart = new MimeMultipart();    
      
      messageBodyPart = new MimeBodyPart();
      
      // set mail content
      if (htmlBody_ == null) {
        messageBodyPart.setText(mailText_);
      } else {
        if (htmlBodyCharset_ == null) {
          messageBodyPart.setContent(htmlBody_, "text/html; charset=UTF-8");
        } else {
          messageBodyPart.setContent(htmlBody_, "text/html; charset=" + htmlBodyCharset_);
        }
      }
      multipart.addBodyPart(messageBodyPart);
      
      for (int i=0; i<binaryDocuments_.size(); i++) {
        BinaryDocument bdoc = (BinaryDocument)binaryDocuments_.get(i);

        mailText_ += " - " + bdoc.getFileName() + "\n";
        
        //File file = new File("c:/temp/" + bdoc.getFileName());
        //FileOutputStream out = new FileOutputStream(file);
        //OutputStreamWriter writer = new OutputStreamWriter(out, "UTF-8");
        
        //ByteArrayOutputStream bos = new ByteArrayOutputStream();
        
        //Utils.dumpInputOutputStream(bdoc.getInputStream(), out);
        
        // remove xmlns:xml attribute
        if (bdoc.getMimeType().equals("text/xml")) {
          ByteArrayOutputStream bos = new ByteArrayOutputStream();
          Utils.dumpInputOutputStream(bdoc.getInputStream(), bos);
          String deliverContent = Utils.replaceVariable(bos.toString(), "xmlns:xml=\"http://www.w3.org/XML/1998/namespace\"", "");
          bdoc.setInputStream(new ByteArrayInputStream(deliverContent.getBytes()));
        }
        //writer.write(deliverContent, 0, deliverContent.length());
        //out.flush();
        //out.close();
        
        messageBodyPart = new MimeBodyPart();
        DataSource source = new BufferDataSource(bdoc);//new FileDataSource(file);
        messageBodyPart.setDataHandler(new DataHandler(source));
        messageBodyPart.setFileName(bdoc.getFileName());
        multipart.addBodyPart(messageBodyPart);
      }

      message.setContent(multipart);
      Transport transport = session.getTransport("smtp");
      Transport.send(message);
    } catch (Exception e) {
      e.printStackTrace();
      throw new DeliveryException(e.getMessage());
    }
  }
  /* (non-Javadoc)
   * @see asit.moa.client.deliver.DeliveryClient#setReceiver(asit.moa.client.deliver.moazs.pdata.PersonData)
   */
  public void setReceiver(Adressat data) {
    personData_ = data;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.DeliveryClient#addBinaryDocument(java.io.InputStream, java.lang.String, java.lang.String)
   */
  public void addPayLoadBinaryDocument(InputStream is, String fileName, String mimeType) {
    
    binaryDocuments_.add(new BinaryDocument(is, fileName, mimeType));
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.DeliveryClient#addBinaryDocument(asit.moa.client.deliver.moazs.BinaryDocument)
   */
  public void addPayLoadBinaryDocument(BinaryDocument document) {
    binaryDocuments_.add(document);
  }
  

  /* (non-Javadoc)
   * @see asit.moa.client.Client#getRequest()
   */
  public Element getRequest() throws ClientException {
    // TODO Auto-generated method stub
    return null;
  }
  /* (non-Javadoc)
   * @see asit.moa.client.Client#sendRequest(java.lang.String)
   */
  public Element sendRequest(String address) throws ClientException {
    // TODO Auto-generated method stub
    return null;
  }
  /* (non-Javadoc)
   * @see asit.moa.client.Client#setDefaultAddress(java.lang.String)
   */
  public void setDefaultAddress(String address) {
    // TODO Auto-generated method stub
    this.defaultAddress_ = address;
  }
  /**
   * @param mailSubject_ The mailSubject_ to set.
   */
  public void setMailSubject(String mailSubject_) {
    this.mailSubject_ = mailSubject_;
  }
  /**
   * @param mailText_ The mailText_ to set.
   */
  public void setMailText(String mailText_) {
    this.mailText_ = mailText_;
  }
  /**
   * @param personData_ The personData_ to set.
   */
  public void setPersonData(Adressat personData_) {
    this.personData_ = personData_;
  }

  /**
   * @param mailSender_ The mailSender_ to set.
   */
  public void setMailSender(String mailSender_) {
    this.mailSenderName_ = mailSender_;
  }
  
  public String getMailSenderAddress() {
    return mailSenderAddress_;
  }
  public void setMailSenderAddress(String mailSenderAddress_) {
    this.mailSenderAddress_ = mailSenderAddress_;
  }
}

