package asit.moa.client.deliver.moazs;

import iaik.x509.X509Certificate;

import java.io.InputStream;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.w3c.dom.Element;

import asit.moa.client.deliver.DeliveryClient;
import asit.moa.client.deliver.address.Adressat;
import asit.moa.client.exception.ClientException;
import asit.moa.client.exception.DeliveryException;

/**
 * app2mzs 
 */
public interface MOAZSClient extends DeliveryClient {
  
  String DQ_RSA                                       = "RSa";
  String DQ_NON_RSA                                   = "nonRSa";
  String DQ_EINSCHREIBEN                              = "R";
  String DQ_EINSCHREIBEN_EIGENHAENDIG                 = "R+";
  String DQ_EINSCHREIBEN_MIT_RUECKSCHEIN              = "RS";
  String DQ_EINSCHREIBEN_MIT_RUECKSCHEIN_EIGENHAENDIG = "RS+";
  
  Set DQS_GV      = new HashSet(Arrays.asList(new String[] { DQ_RSA, DQ_NON_RSA }));
  Set DQS_PRIVATE = new HashSet(Arrays.asList(new String[] { DQ_EINSCHREIBEN_EIGENHAENDIG, DQ_EINSCHREIBEN_MIT_RUECKSCHEIN, DQ_EINSCHREIBEN_MIT_RUECKSCHEIN_EIGENHAENDIG }));
  Set DQS = new HashSet(Arrays.asList(new String[] { DQ_RSA, DQ_NON_RSA, DQ_EINSCHREIBEN, DQ_EINSCHREIBEN_EIGENHAENDIG, DQ_EINSCHREIBEN_MIT_RUECKSCHEIN, DQ_EINSCHREIBEN_MIT_RUECKSCHEIN_EIGENHAENDIG }));
  
  void setServer(String url, X509Certificate certificate)throws ClientException;
  
  void setSenderData(String profileID, String keyID);
  void setSenderProfileID(String profileID);
  void setKeyIdentifier(String keyID);
  void setReceiver(Adressat person);
  void setMetaData(String appDeliveryID, String deliveryQuality, boolean requiresEncryption);
  void setMetaData(String appDeliveryID, String deliveryQuality, boolean requiresEncryption, boolean isPrivateDelivery);
  
  String generateAppDeliveryID() throws DeliveryException;
  
  void setXMLDocument(Element xmlDoc, String xmlProfileID);
  void setXMLDocument(Element xmlDoc,
                      String fileName,
                      String mimeType,
                      String resultingMimeType,
                      String signatureXPath,
                      Element signatureStyleSheet,
                      String previewStyleSheetFileName,
                      Element previewStyleSheet);
  
  void addPayLoadDocumentReference(String url, String fileName, String mimeType, byte[] md5Checksum) throws ClientException;
  
  DeliveryResponse sendDeliveryRequest(String address) throws ClientException;
  DeliveryResponse sendDeliveryRequest(String address, InputStream is) throws ClientException;
  DeliveryResponse sendDeliveryRequest(String address, Element request) throws ClientException;
  DeliveryResponse sendDeliveryRequest() throws ClientException;
  
}
