/*
 * Created on 24.11.2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package asit.moa.client.deliver.moazs;

import iaik.utils.Util;
import iaik.x509.X509Certificate;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.SecureRandom;
import java.util.Vector;

import javax.xml.transform.TransformerException;

import org.apache.log4j.Logger;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import asit.bescheid.BescheidConstants;
import asit.common.Utils;
import asit.moa.MoaConstants;
import asit.moa.client.AbstractClient;
import asit.moa.client.deliver.address.Adressat;
import asit.moa.client.exception.ClientException;
import asit.moa.client.exception.DeliveryException;


public class MOAZSClientImpl extends AbstractClient implements MOAZSClient {
	
	private Logger log = Logger.getLogger(getClass());

  protected String senderProfileID_ = null;
  protected String senderKeyID_ = null;
  
  protected Adressat receiver_ = null;

  protected Element metaData_ = null;
  
  protected Element xmlDocument_ = null; 
  
  protected Vector payLoadElements_ = new Vector();
  
  protected Document doc_ = null;
  
  protected String server_ = null;
  protected X509Certificate x509_ = null;
  
  public MOAZSClientImpl() throws ClientException {
    doc_ = AbstractClient.createNewDocument();
  }
  
  /*
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setSenderData(java.lang.String, java.lang.String)
   */
  public void setSenderData(String profileID, String keyID) {
    if (profileID == null) {
      throw new NullPointerException("SenderProfileID must not be null.");
    }
    if (keyID == null) {
      throw new NullPointerException("SenderKeyID must not be null.");
    }
    senderProfileID_ = profileID;
    senderKeyID_ = keyID;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setReceiver(asit.moa.client.deliver.moazs.pdata.PersonData)
   */
  public void setReceiver(Adressat person) {
    if (person == null) {
      throw new NullPointerException("Person Receiver must not be null.");
    }
    receiver_ = person;
    receiver_.setElementName(MoaConstants.MOA_ZS_NS_PREFIX + ":" + BescheidConstants.RECEIVER);
  }

  /*
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setMetaData(java.lang.String, java.lang.String, boolean)
   */
  public void setMetaData(String appDeliveryID, String deliveryQuality, boolean requiresEncryption) {
     this.setMetaData(appDeliveryID, deliveryQuality, requiresEncryption, false);
  }

  /*
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setMetaData(java.lang.String, java.lang.String, boolean, boolean)
   */
  public void setMetaData(String appDeliveryID, String deliveryQuality, boolean requiresEncryption, boolean isPrivateDelivery) {

    if (appDeliveryID == null) {
      throw new NullPointerException("AppDeliveryID must not be null.");
    }
    if (deliveryQuality == null) {
      throw new NullPointerException("DeliveryQuality must not be null.");
    }

    metaData_ = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.METADATA);
    
    Element appid = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.APPDELIVERYID);
    metaData_.appendChild(appid);
    appid.appendChild(doc_.createTextNode(appDeliveryID));

    Element delqual = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.DELIVERYQUALITY);
    metaData_.appendChild(delqual);
    delqual.appendChild(doc_.createTextNode(deliveryQuality));

    Element reqenc = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.REQUIRESENCRYPTION);
    metaData_.appendChild(reqenc);
    reqenc.appendChild(doc_.createTextNode(String.valueOf(requiresEncryption)));

    // privatedelivery
    Element privateDeliveryElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.PRIVATEDELIVERY);
    metaData_.appendChild(privateDeliveryElement);
    privateDeliveryElement.appendChild(doc_.createTextNode(String.valueOf(isPrivateDelivery)));

    
  }

  /*
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setPayLoadXMLDocument(org.w3c.dom.Element)
   */
  public void setXMLDocument(Element xmlDoc, String xmlProfileID) {
    if (xmlDoc == null) {
      throw new NullPointerException("XML Document must not be null.");
    }
    if (xmlProfileID == null) {
      throw new NullPointerException("XML Profile ID must not be null.");
    }
    xmlDocument_ = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.XMLDOCUMENT);
    Element xmlContent = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.XMLCONTENT);
    
    Node xmlContentToImport = doc_.importNode((Node) xmlDoc, true);
    xmlContent.appendChild(xmlContentToImport);
    Element profile = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.XMLPROFILEID);
    profile.appendChild(doc_.createTextNode(xmlProfileID));
    xmlDocument_.appendChild(profile);    
    xmlDocument_.appendChild(xmlContent);
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setPayLoadXMLDocument(java.lang.String, java.lang.String, java.lang.String, java.lang.String, org.w3c.dom.Element, java.lang.String, org.w3c.dom.Element)
   */
  public void setXMLDocument(Element xmlDoc, String fileName, String mimeType, String resultingMimeType,
      String signatureXPath, Element signatureStyleSheet, String previewStyleSheetFileName,
      Element previewStyleSheet) {
    
    if (xmlDoc == null) {
      throw new NullPointerException("XML Document must not be null.");
    }
    if (fileName == null) {
      throw new NullPointerException("FileName must not be null.");
    }
    if (mimeType == null) {
      throw new NullPointerException("MimeType must not be null.");
    }
    if (resultingMimeType == null) {
      throw new NullPointerException("Resulting MimeType must not be null.");
    }
    if (signatureXPath == null) {
      throw new NullPointerException("Signature XPath must not be null.");
    }
    if (signatureStyleSheet == null) {
      throw new NullPointerException("Signature StyleSheet must not be null.");
    }

    xmlDocument_ = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.XMLDOCUMENT);
    Element xmlContent = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.XMLCONTENT);   
    Node xmlContentToImport = doc_.importNode((Node) xmlDoc, true);
    xmlContent.appendChild(xmlContentToImport);
    xmlDocument_.appendChild(xmlContent);
    
    Element fileNameElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.FILENAME);
    fileNameElement.appendChild(doc_.createTextNode(fileName));
    xmlDocument_.appendChild(fileNameElement);
    
    Element mimeTypeElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.MIMETYPE);
    mimeTypeElement.appendChild(doc_.createTextNode(mimeType));
    xmlDocument_.appendChild(mimeTypeElement);
    
    Element resultingMimeTypeElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.RESULTING_MIMETYPE);
    resultingMimeTypeElement.appendChild(doc_.createTextNode(resultingMimeType));
    xmlDocument_.appendChild(resultingMimeTypeElement);
    
    Element signatureXPathElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.SIGNATURE_XPATH);
    signatureXPathElement.appendChild(doc_.createTextNode(signatureXPath));
    xmlDocument_.appendChild(signatureXPathElement);
   
    Element signatureStyleSheetElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.SIGNATURE_STYLESHEET);
    Element xmlContentElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.XMLCONTENT);
    signatureStyleSheetElement.appendChild(xmlContentElement);
    
    xmlContentElement.appendChild(doc_.importNode(signatureStyleSheet, true));
    xmlDocument_.appendChild(signatureStyleSheetElement);
    
    if (previewStyleSheetFileName != null &&
        previewStyleSheet != null) {
      Element previewStyleSheetElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.PREVIEW_STYLESHEET);
      xmlDocument_.appendChild(previewStyleSheetElement);
      
      Element previewFileName = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.FILENAME);
      previewStyleSheetElement.appendChild(previewFileName);
      previewFileName.appendChild(doc_.createTextNode(previewStyleSheetFileName));
      
      Element previewSS = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.XMLCONTENT);
      previewStyleSheetElement.appendChild(previewSS);
      previewSS.appendChild(doc_.importNode(previewStyleSheet, true));
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setPayLoadBinaryDocument(java.lang.String, java.lang.String, java.lang.String)
   */
  public void addPayLoadBinaryDocument(InputStream binDoc, String fileName, String mimeType) throws IOException {
    if (binDoc == null) {
      throw new NullPointerException("InputStream must not be null.");
    }
    if (fileName == null) {
      throw new NullPointerException("FileName must not be null.");
    }

    if (mimeType == null) {
      throw new NullPointerException("MimeType must not be null.");
    }

    // Copy InputStream to Base64
    Element payLoad = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.PAYLOAD);

    Element binaryDocument = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.BINARY_DOCUMENT);
    payLoad.appendChild(binaryDocument);
    
    Element base64 = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.BASE64CONTENT);
    binaryDocument.appendChild(base64);
    base64.appendChild(doc_.createTextNode(Utils.inputStreamToBase64(binDoc)));
    
    Element file = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.FILENAME);
    binaryDocument.appendChild(file);
    file.appendChild(doc_.createTextNode(fileName));

    Element mime = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.MIMETYPE);
    binaryDocument.appendChild(mime);
    mime.appendChild(doc_.createTextNode(mimeType));

    payLoadElements_.addElement(payLoad);
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setPayLoadDocumentReference(java.lang.String, java.lang.String, java.lang.String, byte[])
   */
  public void addPayLoadDocumentReference(String url, String fileName, String mimeType,
      byte[] md5Checksum) throws ClientException {
    if (url == null) {
      throw new NullPointerException("URL must not be null.");
    }
    if (fileName == null) {
      throw new NullPointerException("FileName must not be null.");
    }

    if (mimeType == null) {
      throw new NullPointerException("MimeType must not be null.");
    }

    Element payLoad = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.PAYLOAD);

    Element documentReference = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.DOCUMENT_REFERENCE);
    payLoad.appendChild(documentReference);
    
    Element urlElement = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.URL);
    documentReference.appendChild(urlElement);
    urlElement.appendChild(doc_.createTextNode(url));
    
    Element file = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.FILENAME);
    documentReference.appendChild(file);
    file.appendChild(doc_.createTextNode(fileName));

    Element mime = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.MIMETYPE);
    documentReference.appendChild(mime);
    mime.appendChild(doc_.createTextNode(mimeType));

    if (md5Checksum != null) {
      Element md5 = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.MD5CHECKSUM);
      documentReference.appendChild(md5);
      try {
        md5.appendChild(doc_.createTextNode(new String(Utils.base64Encoder(md5Checksum))));
      } catch (Exception e) {
        throw new ClientException("Error while encoding MD5 checksum: " + e.getMessage());
      }
    }
    
    payLoadElements_.addElement(payLoad);
    
  }

  /**
   * 
   * @return
   * @throws ClientException
   */
  public DeliveryResponse sendDeliveryRequest() throws ClientException {
	  if (log.isDebugEnabled()) {
		  Element request = this.getRequest(); 
		  ByteArrayOutputStream baout = new ByteArrayOutputStream();
		  try {
			Utils.serializeElement(request, baout);
			String serializedElement = new String(baout.toByteArray(), "UTF-8");
			log.debug("DeliveryRequest = " + serializedElement);
		} catch (IOException e) {
			log.debug("Unable to show request. (" + e.getMessage() + ")");
		}
	  }
    return sendDeliveryRequest(defaultAddress_);
  }
  
  
  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#sendDeliveryRequest(java.lang.String)
   */
  public DeliveryResponse sendDeliveryRequest(String address) throws ClientException {
    Element response = super.sendRequest(address);
    return parseResponse(response);
  }
  
  public DeliveryResponse sendDeliveryRequest(String address, Element request) throws ClientException {
    Element response = super.sendRequest(address, request);
    return parseResponse(response);
  }
    
  public static DeliveryResponse parseResponse(Element response) throws ClientException {
    
    String prefix = response.getPrefix();
    if (prefix == null) {
      prefix = "";
    }
    try {
//      try { Utils.serializeElement(response, System.out); } catch(Exception err) {}
      String name = response.getLocalName();
      Node responseNode = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/child::*");
      if (responseNode instanceof Element) {
        Element resp = (Element)responseNode;
        
//      ok, we have an error
        if (ResponseTypes.ERROR.equals(resp.getLocalName())) {
          Node n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":Error/" + prefix + ":AppDeliveryID/text()");
          String app = n == null ? "" : n.getNodeValue();
          
          n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":Error/" + prefix + ":MZSDeliveryID/text()");
          String mzs = n == null ? "" : n.getNodeValue();
          
          n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":Error/" + prefix + ":Code/text()");
          String code = n == null ? "" : n.getNodeValue();
          
          n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":Error/" + prefix + ":Text/text()");
          String text = n == null ? "" : n.getNodeValue();

          n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":Error/" + prefix + ":DocumentReference/text()");
          String docRef = n == null ? null : n.getNodeValue();

          ErrorDeliveryResponse ret = new ErrorDeliveryResponse(app, mzs, code, text, docRef);
          ret.setRawData(response);
          return ret;
          
        } else if (ResponseTypes.PARTIAL_SUCCESS.equals(resp.getLocalName())) {
          Node n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":PartialSuccess/" + prefix + ":AppDeliveryID/text()");
          String app = n == null ? "" : n.getNodeValue();
          
          n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":PartialSuccess/" + prefix + ":MZSDeliveryID/text()");
          String mzs = n == null ? "" : n.getNodeValue();
          
          PartialSuccessDeliveryResponse ret = new PartialSuccessDeliveryResponse(app, mzs);
          ret.setRawData(response);
          return ret;
        } else if (ResponseTypes.SUCCESS.equals(resp.getLocalName())) {
          Node n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":Success/" + prefix + ":AppDeliveryID/text()");
          String app = n == null ? "" : n.getNodeValue();
          
          n = XPathAPI.selectSingleNode(response, "self::" + prefix + ":" + name + "/" + prefix + ":Success/" + prefix + ":MZSDeliveryID/text()");
          String mzs = n == null ? "" : n.getNodeValue();
//          System.out.println("DID: " + mzs);
          SuccessDeliveryResponse ret = new SuccessDeliveryResponse(app, mzs);
          ret.setRawData(response);
          return ret;
        }
      }
    } catch (TransformerException e) {
      e.printStackTrace();
      throw new ClientException("Error while reading delivery response: " + e.getMessage());
    }
    
    return null;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#getRequest()
   */
  public Element getRequest() throws ClientException {
    Element root = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.DELIVERYREQUEST);
    root.setAttribute("xmlns:" + MoaConstants.MOA_ZS_NS_PREFIX, MoaConstants.MOA_ZS_NS);
    root.setAttribute("xmlns:" + MoaConstants.MOA_ZS_PDATA_NS_PREFIX, MoaConstants.MOA_ZS_PDATA_NS);
    
    if (senderProfileID_ == null &&
        senderKeyID_ == null) {
      throw new NullPointerException("Please define first both sender profile id and sender key id.");
    }
    
    if (receiver_ == null) {
      throw new NullPointerException("Receiver must not be null.");
    }
    
    if (metaData_ == null) {
      throw new NullPointerException("MetaData must not be null.");
    }
    
    if (server_ != null) {
      Element server = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.SERVER);
      root.appendChild(server);
      Element zuseurlid = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.ZUSEURLID);
      zuseurlid.appendChild(doc_.createTextNode(server_));
      server.appendChild(zuseurlid);
      if (x509_ != null) {
        Element x509 = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.X509);
        try {
          x509.appendChild(doc_.createTextNode(new String(Util.Base64Encode(x509_.getEncoded()))));
        } catch (Exception e) {
          throw new ClientException(e);
        }
        server.appendChild(x509);
      }
    }
    
    Element sender = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.SENDER);
    root.appendChild(sender);
    
    Element profileid = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.PROFILEID);
    profileid.appendChild(doc_.createTextNode(senderProfileID_));
    sender.appendChild(profileid);

    Element keyid = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.SIGKEYID);
    keyid.appendChild(doc_.createTextNode(senderKeyID_));
    sender.appendChild(keyid);

//    Element receiver = doc_.createElementNS(MoaConstants.MOA_ZS_NS, MoaConstants.MOA_ZS_NS_PREFIX + ":" + MoaConstants.RECEIVER);
//    root.appendChild(receiver);
    
    receiver_.setUseGerman(false);    
    root.appendChild(receiver_.toElement(doc_));
    
    root.appendChild(metaData_);
    
    if (xmlDocument_ != null) {
      root.appendChild(xmlDocument_);
    }
    
    for (int i=0; i<payLoadElements_.size(); i++) {
      Element payLoad = (Element)payLoadElements_.elementAt(i);
      root.appendChild(payLoad);
    }
        
    return root;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.DeliveryClient#addBinaryDocument(asit.moa.client.deliver.moazs.BinaryDocument)
   */
  public void addBinaryDocument(BinaryDocument document) {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.DeliveryClient#deliver()
   */
  public void deliver() throws DeliveryException {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#generateAppDeliveryID()
   */
  public String generateAppDeliveryID() throws DeliveryException {
    try {
      SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
      byte[] bytes = new byte[8];
      random.nextBytes(bytes);     
      return convertToHex(bytes);
    } catch(Exception e) {
      throw new DeliveryException(e.getMessage());
    }
  }
  
  private String convertToHex(byte[] bytes) {
    String ret = "";
    for (int i=0; i<bytes.length; i++) {
      
      ret += Integer.toHexString(0x0100 + (bytes[i] & 0x00FF)).substring(1);
    }
    return ret;
  }
  
  public static void main(String[] args) throws Exception {
    MOAZSClientImpl impl = new MOAZSClientImpl();
//    System.out.println(impl.generateAppDeliveryID());
    //System.out.println(impl.convertToHex(new byte[] { (byte)0xE1, (byte)0xFF }));
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#sendDeliveryRequest(java.lang.String, java.io.InputStream)
   */
  public DeliveryResponse sendDeliveryRequest(String address, InputStream is) throws ClientException {
    Element response = super.sendRequest(address, is);
    return parseResponse(response);
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setSenderProfileID(java.lang.String)
   */
  public void setSenderProfileID(String profileID) {
    senderProfileID_ = profileID;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.moazs.MOAZSClient#setKeyIdentifier(java.lang.String)
   */
  public void setKeyIdentifier(String keyID) {
    senderKeyID_ = keyID;
  }

  public void setServer(String url, X509Certificate certificate)
      throws ClientException {
    
    if (url == null) {
      throw new NullPointerException("URL must not be null.");
    }
    this.server_ = url;
    if (certificate != null) {
      this.x509_ = certificate;
    }
    
  }

}



