package asit.moa.client.deliver.robot;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import javax.net.ssl.SSLSocketFactory;

import org.w3c.dom.Element;

import asit.common.Utils;
import asit.moa.client.deliver.DeliveryClient;
import asit.moa.client.deliver.DeliveryRobotWrapper;
import asit.moa.client.deliver.address.Adressat;
import asit.moa.client.exception.ClientException;
import asit.moa.client.exception.DeliveryException;


/**
 * @author atauber
 */
public class RobotDeliveryClient implements DeliveryClient {

  protected Properties properties_ = null;
  protected DeliveryRobotWrapper deliveryRobot_  = null;
  
  public RobotDeliveryClient() {
    properties_ = new Properties();
    
    try {
      properties_.load(new FileInputStream(System.getProperty("delivery.robot.properties")));
    } catch(IOException e) {
      e.printStackTrace();
    }
    
    // initialize delivery-robot wrapper
    this.deliveryRobot_ = new DeliveryRobotWrapper(properties_.getProperty("robot.senddir"),
                                                   properties_.getProperty("robot.senderxml"),
                                                   properties_.getProperty("robot.webserviceurl"),
                                                   properties_.getProperty("robot.notificationmail"),
                                                   properties_.getProperty("robot.ldapurl"),
                                                   new File(properties_.getProperty("robot.executable")));
                                                   
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.ZSClient#addDocument(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
   */
  public void addDocument(String content, String receiverSurName,
    String receiverGivenName, String receiverEmail, String deliveryNumber,
      String fileExtension) throws DeliveryException {

    // fix Axis bug
    String deliverContent = Utils.replaceVariable(content, "xmlns:xml=\"http://www.w3.org/XML/1998/namespace\"", "");
    
    //System.out.println("Writing out...");
    deliveryRobot_.addDocToSend(deliverContent, receiverSurName, receiverGivenName, receiverEmail, deliveryRobot_.generateDeliverNumber(), fileExtension);
  }
  
  /*
   * (non-Javadoc)
   * @see asit.framework.moaclient.ZSClient#addDocument(java.io.InputStream, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
   */
  public void addDocument(InputStream is, String receiverSurName,
      String receiverGivenName, String receiverEmail, String deliveryNumber,
        String fileExtension) throws DeliveryException {

      ByteArrayOutputStream out = new ByteArrayOutputStream();
      int ch;
      try {
        while ((ch = is.read()) != -1) {
          out.write(ch);
        }
        //System.out.println("Writing out...");
        
        // fix Axis bug
        String deliverContent = Utils.replaceVariable(out.toString("UTF-8"), "xmlns:xml=\"http://www.w3.org/XML/1998/namespace\"", "");
        
        deliveryRobot_.addDocToSend(deliverContent, receiverSurName, receiverGivenName, receiverEmail, deliveryRobot_.generateDeliverNumber(), fileExtension);
      } catch (IOException e) {
        e.printStackTrace();
        throw new DeliveryException(e.getMessage());
      }
    }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.ZSClient#deliver()
   */
  public void deliver() throws DeliveryException {
    //deliveryRobot_.deliverDocuments();
  }
  
  /* (non-Javadoc)
   * @see asit.framework.moaclient.ZSClient#generateDeliveryNumber()
   */
  public String generateDeliveryNumber() {
    return this.deliveryRobot_.generateDeliverNumber();
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#getRequest()
   */
  public Element getRequest() throws ClientException {
    // TODO Auto-generated method stub
    return null;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#sendRequest(java.lang.String)
   */
  public Element sendRequest(String address) throws ClientException {
    // TODO Auto-generated method stub
    return null;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#setDefaultAddress(java.lang.String)
   */
  public void setDefaultAddress(String address) {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.DeliveryClient#addPayLoadBinaryDocument(java.io.InputStream, java.lang.String, java.lang.String)
   */
  public void addPayLoadBinaryDocument(InputStream binDoc, String fileName, String mimeType) throws IOException {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#setSSLTrustStore(java.lang.String, java.lang.String)
   */
  public void setSSLTrustStore(String fileLocation, String password) {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#setHostNameVerifierEnabled(boolean)
   */
  public void setHostNameVerifierEnabled(boolean value) {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#sendRequest(java.lang.String, java.io.InputStream)
   */
  public Element sendRequest(String address, InputStream is) throws ClientException {
    // TODO Auto-generated method stub
    return null;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.deliver.DeliveryClient#setReceiver(asit.bescheid.Adressat)
   */
  public void setReceiver(Adressat data) {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#setClientAuthStore(java.lang.String, java.lang.String)
   */
  public void setClientAuthStore(String fileLocation, String password) {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#setSSLTrustStore(java.lang.String, java.lang.String, java.lang.String)
   */
  public void setSSLTrustStore(String fileLocation, String password, String type) {
    // TODO Auto-generated method stub
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.Client#setSSLSocketFactory(javax.net.ssl.SSLSocketFactory)
   */
  public void setSSLSocketFactory(SSLSocketFactory factory) {
    // TODO Auto-generated method stub
    
  }

}
