// Copyright (C) 2002 IAIK
// http://jce.iaik.at
//
// Copyright (C) 2003 Stiftung Secure Information and 
//                    Communication Technologies SIC
// http://www.sic.st
//
// All rights reserved.
//
// This source is provided for inspection purposes and recompilation only,
// unless specified differently in a contract with IAIK. This source has to
// be kept in strict confidence and must not be disclosed to any third party
// under any circumstances. Redistribution in source and binary forms, with
// or without modification, are <not> permitted in any case!
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

package asit.moa.client.id;


/**
 * A SAML-Attribute to be appended to the final SAML-Assertion 
 * that will be passed to the online application. 
 */
public interface ExtendedSAMLAttribute {
  
  /**
   * The value of the SAML-Attribute. This must be either a
   * <code>org.w3c.Element</code> or a <code>java.lang.String</code> 
   * object. Each other type will be ignored. <br>
   * If, for example, the type of the actual SAML-Attribute is an
   * <code>&lt;xsd:boolean&gt;<code> the value must be either the String
   * <code>&quot;true&quot;</code> of <code>&quot;false&quot;</code>. 
   * Or the <code>&lt;xsd:integer&gt;<code> number <code>273</code> 
   * has to be the String &quot;273&quot;.
   *   
   * @return The value of the SAML-Attribute.
   */
  public Object getValue();
  
  /**
   * The name of the SAML-Attribute.
   * 
   * @return The name of the SAML-Attribute.
   */
  public String getName();
  
  /**
   * The namespace of the SAML-Attribute. If <code>null</code> the namespace 
   * <code>&quot;http://reference.e-government.gv.at/namespace/moa/20020822#&quot;</code>
   * will be used.
   * 
   * @return The namespace of the SAML-Attribute or <code>null</code>.
   */
  public String getNameSpace(); 

}
