package asit.moa.client.id;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import org.apache.catalina.Realm;
import org.apache.catalina.realm.GenericPrincipal;
import org.w3c.dom.Element;

/**
 * @author <a href="mailto:arne.tauber@iaik.tugraz.at">Arne Tauber</a>
 * @author modified by <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
 */
public class MOAIDPrincipal extends GenericPrincipal implements MOAIDPrincipalInterface {

	private String statusCode_;
	private String givenName_;
	private String familyName_;
	private String dateOfBirth_;

	private String nameQualifier;
	private String nameIdentifier;

	private String authority;
	private String pseudonym;
	// private String password; // hash  // TK: provided by super class

  private Hashtable extendedSamlAttributes = new Hashtable();  
	private Date loginTime;

	/**
	 * The underlying SAML-Assertion. With this assertion possible derived child classes are able to provide additional fields.

	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	private Element samlAssertion;

	private List subjectConfirmationAssertions = new ArrayList();
	
	/**
	 * The user's base id (in case of adequate configuration of MOA-ID).
	 * 
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	private String baseId;
	
	/**
	 *	The URL of the underlying type of citizen card (useful if more than one type of citizen card (e.g. chip/a1) is supported). 
	 * 
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	private String bkuURL;

  
  
  
  //-------------------------------------------------
	/**
	 * Returns the URL of the underlying type of citizen card.
	 * 
	 * @return The underlying type of citizen card.
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public String getBkuURL() {
		return this.bkuURL;
	}

	/**
	 * Sets the URL of the underlying type of citizen card.
	 * 
	 * @param bkuURL The URL of the underlying type of citizen card.
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public void setBkuURL(String bkuURL) {
		this.bkuURL = bkuURL;
	}

	/**
	 * Returns the user's base id (or {@code null} if MOA-ID is not adequately configured).
	 * <p>Note: Set the attribute {@code provideStammzahl="true"} of {@code AuthComponent} within
	 * the MOA-ID configuration file.</p>

	 * @return The user's base id or {@code null}
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public String getBaseId() {
		return this.baseId;
	}

	/**
	 *	Sets the user's base id (provided that MOA-ID is adequately configured.
	 * 
	 * @param baseId The user's baseid.
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public void setBaseId(String baseId) {
		this.baseId = baseId;
	}

	/**
	 * Returns the SAML-Assertion.
	 * 
	 * @return SAML-Assertion
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public Element getSamlAssertion() {
		return this.samlAssertion;
	}

	/**
	 *	Sets the SAML-Assertion.
	 * 
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public void setSamlAssertion(Element samlAssertion) {
		this.samlAssertion = samlAssertion;
	}

	public String getAuthority() {
		return this.authority;
	}

	public void setAuthority(String authority) {
		this.authority = authority;
	}

	public String getDateOfBirth() {
		return this.dateOfBirth_;
	}

	public void setDateOfBirth(String dateOfBirth) {
		this.dateOfBirth_ = dateOfBirth;
	}

	public String getFamilyName() {
		return this.familyName_;
	}

	public void setFamilyName(String familyName) {
		this.familyName_ = familyName;
	}

	public String getGivenName() {
		return this.givenName_;
	}

	public void setGivenName(String givenName) {
		this.givenName_ = givenName;
	}

	public Date getLoginTime() {
		return this.loginTime;
	}

	public void setLoginTime(Date loginTime) {
		this.loginTime = loginTime;
	}

	public String getNameIdentifier() {
		return this.nameIdentifier;
	}

	public void setNameIdentifier(String nameIdentifier) {
		this.nameIdentifier = nameIdentifier;
	}

	public String getNameQualifier() {
		return this.nameQualifier;
	}

	public void setNameQualifier(String nameQualifier) {
		this.nameQualifier = nameQualifier;
	}

	public String getPseudonym() {
		return this.pseudonym;
	}

	public void setPseudonym(String pseudonym) {
		this.pseudonym = pseudonym;
	}

	public String getStatusCode() {
		return this.statusCode_;
	}

	public void setStatusCode(String statusCode) {
		this.statusCode_ = statusCode;
	}

	public MOAIDPrincipal(Realm realm, String name, String password, List roles) {
		super(realm, name, password, roles);
	}

	public MOAIDPrincipal(Realm realm, String name, String password) {
		super(realm, name, password);
	}

	/**
	 * Creates an empty MOAIdPrincipal.
	 *  
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public MOAIDPrincipal() {
		super(null, null, null);
	}

	public void setPassword(String password) {
		super.password = password;
	}
	
	public void setRealm(Realm realm) {
		super.realm = realm;
	}
	
	public void setRoles(String[] roles) {
		super.roles = roles;
	}
	
	/**
	 * Gets a human readable String of this class.
	 * 
	 * @author <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("givenName=\"").append(this.givenName_).append("\"");
		buffer.append(", familyName=\"").append(this.familyName_).append("\"");
		buffer.append(", dateOfBirth=\"").append(this.dateOfBirth_).append("\"");
		buffer.append(", nameQualifier=\"").append(this.nameQualifier).append("\"");
		buffer.append(", nameIdentifier=\"").append(this.nameIdentifier).append("\"");
		buffer.append(", baseId=\"").append(this.baseId).append("\"");
		
		buffer.append(", roles (GenericPrincipal)=[");
		for (int i = 0; i < roles.length; i++) {
			if (i != 0) {
				buffer.append(", ");
			}
			buffer.append(super.roles[i]);
		}
		buffer.append("]");
		
		buffer.append(", authority=\"").append(this.authority).append("\"");
		buffer.append(", bkuURL=\"").append(this.bkuURL).append("\"");
		buffer.append(", loginTime=\"").append(this.loginTime != null ? (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(this.loginTime) : null).append("\"");
		buffer.append(", pseudonym=\"").append(this.pseudonym).append("\"");
		buffer.append(", statusCode=\"").append(this.statusCode_).append("\"");
		buffer.append(", name (GenericPrincipal)=\"").append(super.name).append("\"");
		return buffer.toString();
	}

  public Hashtable getExtendedSamlAttributes() {
    return extendedSamlAttributes;
  }
  
  public static String buildExtSamlId(String name, String nameSpace) {
    return name + nameSpace;
  }
  
  public Object getExtendedSamlAttribute(String name, String nameSpace) {
    return extendedSamlAttributes.get(buildExtSamlId(name, nameSpace));
  }
  
  /**
 * @deprecated Use {@link #addExtendedSamlAttribute(ExtendedSAMLAttribute)} instead
 */
  public void addExtendedSamlAttibute(ExtendedSAMLAttribute attr) {
   addExtendedSamlAttribute(attr);
  }

  public void addExtendedSamlAttribute(ExtendedSAMLAttribute attr) {
    extendedSamlAttributes.put(buildExtSamlId(attr.getName(), attr.getNameSpace()), attr.getValue());
  }
  
  public void setExtendedSamlAttributes(Hashtable extendedSamlAttributes) {
    this.extendedSamlAttributes = extendedSamlAttributes;
  }

  public List getSubjectConfirmationAssertions() {
    return subjectConfirmationAssertions;
  }

  public void setSubjectConfirmationAssertions(List subjectConfirmationAssertions) {
    this.subjectConfirmationAssertions = subjectConfirmationAssertions;
  }
}
