/*
 * Created on 28.01.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package asit.moa.client.id;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import asit.common.Utils;
import asit.framework.servlet.FrontEndServlet;
import asit.moa.moaid.MOAIDRealm;

/**
 * @author <a href="mailto:arne.tauber@iaik.tugraz.at">Arne Tauber</a>
 * @author modified by <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
 */
public class MOAIDPrincipalParser {

  static Logger logger_ = Logger.getLogger(MOAIDPrincipalParser.class.getName());
  
	/**
	 * Parses a MOA-ID response and creates a new MOAID-Principal.
	 * 
	 * @param data The MOA-ID response.
	 * @return The newly created MOAIDPrincipal.
	 * @see MOAIDPrincipal
	 * @author <a href="mailto:arne.tauber@iaik.tugraz.at">Arne Tauber</a>
	 * @author modified by <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
	 */
	public static MOAIDPrincipal parse(Element data) {
    
		MOAIDPrincipal principal = new MOAIDPrincipal(new MOAIDRealm(), "moaiduser", "moaidpassword", new ArrayList());
		String prefix = data.getPrefix();
		if (prefix == null) {
			prefix = "";
		}

		try {
      
		  ByteArrayOutputStream bos = new ByteArrayOutputStream();
      Utils.serializeElement(data, bos);
      logger_.debug(bos.toString());
      
			Node statusCode = XPathAPI.selectSingleNode(data, "self::" + prefix + ":Response/" + prefix + ":Status/" + prefix + ":StatusCode[@Value]");
			if (statusCode != null) {
				principal.setStatusCode(((Element) statusCode).getAttribute("Value"));
			}

			NodeList assertionNodeList = data.getChildNodes();
			Node assertionNode = null;
			for (int i = 0; i < assertionNodeList.getLength(); i++) {
				if (assertionNodeList.item(i) instanceof Element) {
					Element e = (Element) assertionNodeList.item(i);
					if ("Assertion".equals(e.getLocalName())) {
						assertionNode = e;
						// TK: set SAML-Assertion
						principal.setSamlAssertion(e);
					}
				}
			}

			
			// we have finished
			if (assertionNode == null) {
				return null;
			}

			//add saml:Assertion nodes of subject info
      NodeList subjectAssertionList =  XPathAPI.selectNodeList(assertionNode, "saml:AttributeStatement/saml:Subject/saml:SubjectConfirmation/saml:SubjectConfirmationData/saml:Assertion");
      if (subjectAssertionList != null) {
        for (int i=0; i<subjectAssertionList.getLength(); i++) {
          principal.getSubjectConfirmationAssertions().add(subjectAssertionList.item(i));
        }
      }
      
      
			
			prefix = assertionNode.getPrefix();
			if (prefix == null) {
				prefix = "";
			}
			Node givenName = XPathAPI.selectSingleNode(assertionNode, "self::" + prefix + ":Assertion/" + prefix + ":AttributeStatement/" + prefix + ":Attribute/" + prefix + ":AttributeValue/pr:Person/pr:Name/pr:GivenName/text()");
			if (givenName != null) {
				principal.setGivenName(givenName.getNodeValue());
			} else {
				return null;
			}

			// TK: parses the user's base id.
			Node baseid = XPathAPI.selectSingleNode(assertionNode, "self::"	+ prefix	+ ":Assertion/" + prefix + ":AttributeStatement/" + prefix + ":Attribute[@AttributeName='PersonData']/" + prefix + ":AttributeValue/pr:Person/pr:Identification[pr:Type/text()='urn:publicid:gv.at:baseid']/pr:Value/text()");
			if (baseid != null) {
				principal.setBaseId(baseid.getNodeValue());
			}

			Node familyName = XPathAPI.selectSingleNode(assertionNode, "self::" + prefix + ":Assertion/" + prefix + ":AttributeStatement/" + prefix + ":Attribute/" + prefix + ":AttributeValue/pr:Person/pr:Name/pr:FamilyName/text()");
			if (familyName != null) {
				principal.setFamilyName(familyName.getNodeValue());
			}

			Node dateOfBirth = XPathAPI.selectSingleNode(assertionNode, "self::" + prefix	+ ":Assertion/" + prefix + ":AttributeStatement/" + prefix + ":Attribute/" + prefix	+ ":AttributeValue/pr:Person/pr:DateOfBirth/text()");
			if (dateOfBirth != null) {
				principal.setDateOfBirth(dateOfBirth.getNodeValue());
			}

			Node nameQualifier = XPathAPI.selectSingleNode(assertionNode, "self::" + prefix + ":Assertion/" + prefix + ":AttributeStatement/" + prefix + ":Subject/" + prefix	+ ":NameIdentifier[@NameQualifier]");
			if (nameQualifier != null) {
				Element elem = (Element) nameQualifier;
				principal.setNameQualifier(elem.getAttribute("NameQualifier"));
			}

			Node nameIdentifier = XPathAPI.selectSingleNode(assertionNode, "self::" + prefix	+ ":Assertion/" + prefix + ":AttributeStatement/" + prefix + ":Subject/" + prefix + ":NameIdentifier[@NameQualifier]/text()");
			if (nameIdentifier != null) {
				principal.setNameIdentifier(nameIdentifier.getNodeValue());
			}

	      // TK: parses the URL of the underlying type of citizen card.
			Node bkuURL = XPathAPI.selectSingleNode(assertionNode, "self::" + prefix + ":Assertion/" + prefix + ":AttributeStatement/" + prefix + ":Attribute[@AttributeName=\"bkuURL\"]/" + prefix + ":AttributeValue/text()");
	      if (bkuURL != null) {
	        principal.setBkuURL(bkuURL.getNodeValue());
	      }
	    // save extended saml attributes
      NodeList samlAttributesList = XPathAPI.selectNodeList(data, "descendant-or-self::" + prefix + ":Attribute");
      for (int i=0; i<samlAttributesList.getLength(); i++) {
        Element samlAttributeElement = (Element)samlAttributesList.item(i);
        Node attrValueNode = XPathAPI.selectSingleNode(samlAttributeElement, prefix + ":AttributeValue").getChildNodes().item(0);
        Object value = null;
        if (attrValueNode != null) {
          if (attrValueNode.getNodeType() == Node.TEXT_NODE) {
            value = attrValueNode.getNodeValue();
          } else if (attrValueNode.getNodeType() == Node.ELEMENT_NODE) {
            value = attrValueNode;
          }
        }
        String name = samlAttributeElement.getAttribute("AttributeName");
        String nameSpace = samlAttributeElement.getAttribute("AttributeNamespace");
        ExtendedSAMLAttribute samlAttribute = new ExtendedSAMLAttributeImpl(value, name, nameSpace);
        principal.addExtendedSamlAttribute(samlAttribute);
      }
       
        
		} catch (Exception e) {
			e.printStackTrace();
		}
		return principal;
	}

}
