package asit.moa.client.sign;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerException;
import org.apache.log4j.Logger;
import org.apache.xpath.XPathAPI;
import org.apache.xpath.objects.XObject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import asit.common.Utils;
import asit.framework.servlet.FrontEndServlet;
import asit.moa.MoaConstants;
import asit.moa.client.AbstractClient;
import asit.moa.client.exception.ClientException;
import asit.moa.client.exception.RequestException;

/**
 * @author atauber
 */
public class CreateSignatureClientImpl extends AbstractClient implements CreateSignatureClient {

  protected SingleSignatureInfo[] infos_ = null;
  static Logger logger_ = Logger.getLogger(FrontEndServlet.class.getName());

  
  public CreateSignatureClientImpl() throws ClientException {
		super();
    qname_service_ = new QName("SignatureCreation");
	}

  public void setKeyIdentifier(String identifier) {
    if (identifier == null) {
      throw new NullPointerException("Argument \"identifier\" must not be null");
    }
    keyIdentifier_ = identifier;
  }

 
  /*
   * Gets the request document.
   */
  public Element getRequest() throws ClientException {
    //doc_ = super.createNewDocument();
    if (keyIdentifier_ == null) {
      throw new NullPointerException("Please set first the key identifier.");
    }
    if (infos_ == null) {
      throw new NullPointerException("Please set first at least one SingleSignatureInfo element.");
    }
    Element root = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CREATE_XML_SIGNATURE_REQUEST);
    root.setAttribute("xmlns" + MoaConstants.MOA_POSTFIX, MoaConstants.MOA_NS);
    root.setAttribute("xmlns" + MoaConstants.DSIG_POSTFIX, MoaConstants.DSIG_NS);
    
    // set key identifier
    Element keyIdentifier = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.KEY_IDENTIFIER);
    keyIdentifier.appendChild(doc_.createTextNode(keyIdentifier_));
    root.appendChild(keyIdentifier);
    
    // set single signature infos
    for (int i=0; i<infos_.length; i++) {
      root.appendChild(infos_[i].toElement(doc_));
    }
    
    return root;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.CreateSignatureClient#sendSignatureRequest(java.lang.String)
   */
  public Element sendSignatureRequest() throws ClientException, RequestException {
      Element resp = super.sendRequest();
      return handleRequest(resp);
  }
    
  
  /* (non-Javadoc)
   * @see asit.framework.moaclient.CreateSignatureClient#sendSignatureRequest(java.lang.String)
   */
  public Element sendSignatureRequest(String address) throws ClientException, RequestException {
      Element resp = super.sendRequest(address);
      return handleRequest(resp);
  }
     
  
  protected Element handleRequest(Element resp) throws ClientException, RequestException {
      try {
        
      Document response = null;
      try {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        //XMLSerializer serializer = new XMLSerializer(out, new OutputFormat("xml", "UTF-8", false));
        //serializer.serialize(resp);
        Utils.serializeElement(resp, out);
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        //factory.setNamespaceAware(true);
        DocumentBuilder builder;
        builder = factory.newDocumentBuilder();
        response = builder.parse(new ByteArrayInputStream(out.toByteArray()));
      } catch(Exception e) {
        e.printStackTrace();
        throw new ClientException(e);
      }
      
      NodeList signatures = XPathAPI.selectNodeList(response, "/CreateXMLSignatureResponse/SignatureEnvironment/child::node()[1]");
      //NodeList signatures = XPathAPI.selectNodeList(response, "/CreateXMLSignatureResponse");
      // ok, we have a signature
      logger_.debug("Anzahl der gefundenen Element: " + signatures.getLength());
      if (signatures.getLength() != 0) {
        Element ret = (Element)signatures.item(0);
        
        // WORKAROUND - has to be changed
        //ret.setAttribute("xmlns:dsig", "http://www.w3.org/2000/09/xmldsig#");
        return ret;
      } else {
        // get error
        XObject errorCode = XPathAPI.eval(response, "/CreateXMLSignatureResponse/ErrorResponse/ErrorCode/text()");
        if (errorCode != null) {
          int code = Integer.parseInt(errorCode.str());
          XObject info = XPathAPI.eval(response, "/CreateXMLSignatureResponse/ErrorResponse/Info/text()");
          throw new RequestException(code, info.str());
        } else {
          throw new ClientException("Unknown response format.");
        }
      }
      
    } catch (TransformerException e) {
      e.printStackTrace();
      throw new ClientException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.CreateSignatureClient#setSingleSignatureInfos(asit.framework.moaclient.SingleSignatureInfo[])
   */
  public void setSingleSignatureInfos(SingleSignatureInfo[] infos) {
    infos_ = infos;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.CreateSignatureClient#sendSignatureRequest(java.lang.String, org.w3c.dom.Element)
   */
  public Element sendSignatureRequest(String address, Element request) throws ClientException, RequestException {
    Element resp = super.sendRequest(address, request);
    return handleRequest(resp);
  }
    
}
