package asit.moa.client.sign;

import iaik.ixsil.algorithms.Transform;
import iaik.ixsil.exceptions.AlgorithmException;
import iaik.ixsil.exceptions.UtilsException;
import iaik.ixsil.util.Utils;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import asit.moa.MoaConstants;
import asit.moa.client.exception.ClientException;


/**
 * @author atauber
 */
public class SignReferenceImpl implements SignReference {

  protected Document xmlContent_ = null;
  protected NodeList xmlContentList_ = null;
  protected String base64Content_ = null;
  protected boolean isXML_ = false;
  protected int structure_ = SignReference.STRUCTURE_DETACHED;
  protected boolean isChildOfManifest_ = false;
  protected Transform[] transforms_ = null;
  protected Element transformsElement_ = null;
  protected String transformsProfileID_ = null;
  protected MetaInfo metaInfo_ = null;
  protected String reference_ = null;
  protected String locRefContent_ = null;
  
  public SignReferenceImpl() {
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setXMLContent(org.w3c.dom.Document)
   */
  public void setXMLContent(Document doc) {
    xmlContent_ = doc;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setBinaryContent(byte[])
   */
  public void setBinaryContent(byte[] binaryContent) {
    try {
      base64Content_ = new String(Utils.base64Encode(binaryContent));
    } catch (UtilsException e) {
      e.printStackTrace();
    }
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setIsXML(boolean)
   */
  public void setIsXML(boolean isXML) {
    isXML_ = isXML;
  }


  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setStructure(int)
   */
  public void setStructure(int structure) {
    structure_ = structure;
  }


  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setIsChildOfManifest(boolean)
   */
  public void setIsChildOfManifest(boolean isChildOfManifest) {
    isChildOfManifest_ = isChildOfManifest;
  }


  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setTransformations(iaik.ixsil.algorithms.Transform[])
   */
  public void setTransformations(Transform[] transforms) {
    transforms_ = transforms;
  }
  
  public void setTransformations(Element transforms) {
    transformsElement_ = transforms;
  }


  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setTransformsProfileID(java.lang.String)
   */
  public void setTransformsProfileID(String id) {
    transformsProfileID_ = id;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setMetaInfo(asit.framework.moaclient.MetaInfo)
   */
  public void setMetaInfo(MetaInfo metaInfo) {
    metaInfo_ = metaInfo;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#toElement(org.w3c.dom.Document)
   */
  public Element toElement(Document doc) throws ClientException {
    Element dataObjectInfo = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.DATA_OBJECT_INFO);
    
    if (structure_ == SignReference.STRUCTURE_DETACHED) {
      dataObjectInfo.setAttribute(MoaConstants.STRUCTURE, MoaConstants.DETACHED);
    } else {
      dataObjectInfo.setAttribute(MoaConstants.STRUCTURE, MoaConstants.ENVELOPING);
    }
    if (isChildOfManifest_) {
      dataObjectInfo.setAttribute(MoaConstants.CHILD_OF_MANIFEST, "true");
    } else {
      dataObjectInfo.setAttribute(MoaConstants.CHILD_OF_MANIFEST, "false");
    }

    
    Element dataObject = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.DATA_OBJECT);
    dataObjectInfo.appendChild(dataObject);
    
    if (reference_ != null) {
      dataObject.setAttribute("Reference", reference_);
    }
    
    // embedd the document to sign
    if (xmlContent_ != null || xmlContentList_ != null) {
      Element xmlContent = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.XML_CONTENT);
      
      // always preserve space
      xmlContent.setAttribute("xml:space", "preserve");
      
      if (xmlContent_ != null) {
      
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        try {
          builder = factory.newDocumentBuilder();
          //Document docToAdd = builder.parse(fileToSign);
          Node importNode = doc.importNode(xmlContent_.getDocumentElement(), true);
          xmlContent.appendChild(importNode);
          dataObject.appendChild(xmlContent);
        } catch (ParserConfigurationException e) {
          e.printStackTrace();
          throw new ClientException(e);
        }
      } else {
        for (int i=0; i<xmlContentList_.getLength(); i++) {
          xmlContent.appendChild(doc.importNode(xmlContentList_.item(i), true));
        }
      }
    } else if (locRefContent_ != null) {
      Element locRef = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.LOC_REF_CONTENT);
      dataObject.appendChild(locRef);
      locRef.appendChild(doc.createTextNode(locRefContent_));
    } else if (base64Content_ != null) {
      Element base64Content = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.BASE64CONTENT);
      dataObject.appendChild(base64Content);
      base64Content.appendChild(doc.createTextNode(base64Content_));
    }
    
    if ((transforms_ != null) || (metaInfo_ != null) || (transformsElement_ != null)) {
      Element createTransformsInfoProfile = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CREATE_TRANSFORMS_INFO_PROFILE);
      dataObjectInfo.appendChild(createTransformsInfoProfile);
      
      Element createTransformsInfo = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CREATE_TRANSFORMS_INFO);
      createTransformsInfoProfile.appendChild(createTransformsInfo);
      
      // add transformations
      if (transformsElement_ != null) {
        createTransformsInfo.appendChild(doc.importNode(transformsElement_, true));
      } else if (transforms_ != null) {
        Element transforms = doc.createElementNS(MoaConstants.DSIG_NS, MoaConstants.DSIG_PREFIX + MoaConstants.TRANSFORMS);
        createTransformsInfo.appendChild(transforms);
        
        for (int i=0; i<transforms_.length; i++) {
          Element transform;
          try {
            transform = transforms_[i].toElement(doc);
          } catch (AlgorithmException e) {
            e.printStackTrace();
            throw new ClientException(e);
          }
          Node tInsert = doc.importNode(transform, true);
          transforms.appendChild(tInsert);
        }
      }
      if (metaInfo_ != null) {
        // META-INFO
        Element finalDataMetaInfo = metaInfo_.toElement(doc);
        createTransformsInfo.appendChild(finalDataMetaInfo);
      }

    // else we add a transformation profile ID  
    } else {
      Element createTransformsInfoProfileId = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CREATE_TRANSFORMS_INFO_PROFILE_ID);
      createTransformsInfoProfileId.appendChild(doc.createTextNode(transformsProfileID_));
      dataObjectInfo.appendChild(createTransformsInfoProfileId);
    }

    return dataObjectInfo;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setReference(java.lang.String)
   */
  public void setReference(String reference) {
    reference_ = reference;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SignReference#setLocRefContent(java.lang.String)
   */
  public void setLocRefContent(String reference) {
    locRefContent_ = reference;
  }
  public void setXMLContent(NodeList xmlContentList_) {
    this.xmlContentList_ = xmlContentList_;
  }
}
