package asit.moa.client.sign;

import iaik.utils.Util;
import java.util.Vector;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import asit.moa.MoaConstants;
import asit.moa.client.exception.ClientException;


/**
 * @author atauber
 */
public class SingleSignatureInfoImpl implements SingleSignatureInfo {

  protected Vector references_ = null;
  protected boolean isSecurityLayerConform_ = false;
  
  protected Element docToEmbed_ = null;
  protected byte[] docToEmbedBase64_ = null;
  protected NodeList nodesToEmbed_ = null;
  protected String locRefContent_ = null;
  protected String environmentReference_ = null;
  
  protected String xPathExpression_ = null;
  protected int location_ = 0;
  
  protected Vector namespacePrefixes_ = null;
  protected Vector namespaceURI_ = null;
  
  static Logger logger_ = Logger.getLogger(SingleSignatureInfo.class.getName());
  
  public SingleSignatureInfoImpl() {
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SingleSignatureInfo#setReferences(asit.framework.moaclient.SignReference[])
   */
  public void setReferences(SignReference[] references) {
    references_ = new Vector();
    if (references != null) {
      for (int i=0; i<references.length; i++) {
        references_.addElement(references[i]);
      }
    }
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SingleSignatureInfo#setIsSecurityLayerConform(boolean)
   */
  public void setIsSecurityLayerConform(boolean isConform) {
    isSecurityLayerConform_ = isConform;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SingleSignatureInfo#setDocumentToEmbed(org.w3c.dom.Document, java.lang.String, int)
   */
  public void setDocumentToEmbed(Element doc, String xPath, int position) {
    docToEmbed_ = doc;
    xPathExpression_ = xPath;
    location_ = position;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SingleSignatureInfo#toElement(org.w3c.dom.Document)
   */
  public Element toElement(Document doc) throws ClientException {
    // Create elements
    Element singleSignatureInfo = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.SINGLE_SIGNATURE_INFO);
    if (isSecurityLayerConform_) {
      singleSignatureInfo.setAttribute(MoaConstants.SECURITY_LAYER_CONFORMITY, "true");
    } else {
      singleSignatureInfo.setAttribute(MoaConstants.SECURITY_LAYER_CONFORMITY, "false");
    }
    
    if (references_ != null) {
      logger_.debug("Adding references in SingleSignatureInfo: " + references_.size());
      for (int i=0; i<references_.size(); i++) {
        logger_.debug("Adding reference: " + i);
        singleSignatureInfo.appendChild(((SignReference)references_.elementAt(i)).toElement(doc));
      }
    }
    
    Element createSignatureInfo = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CREATE_SIGNATURE_INFO);
    singleSignatureInfo.appendChild(createSignatureInfo);
    Element createSignatureEnvironment = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CREATE_SIGNATURE_ENVIRONMENT);
    if (environmentReference_ != null) {
      createSignatureEnvironment.setAttribute(MoaConstants.REFERENCE, environmentReference_);
    }
    createSignatureInfo.appendChild(createSignatureEnvironment);
    
    if (docToEmbed_ != null) {
      Element xmlContent = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.XML_CONTENT);
      
      xmlContent.setAttribute("xml:space", "preserve");
      createSignatureEnvironment.appendChild(xmlContent);
      Node nodeToAdd = doc.importNode(docToEmbed_, true);
      xmlContent.appendChild(nodeToAdd);
    } else if (nodesToEmbed_ != null) { 
      Element xmlContent = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.XML_CONTENT);
      for (int i=0; i<nodesToEmbed_.getLength(); i++) {
        xmlContent.appendChild(doc.importNode(nodesToEmbed_.item(i), true));
      }
      createSignatureEnvironment.appendChild(xmlContent);
    } else if (docToEmbedBase64_ != null) {
      Element base64Content = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.BASE64_CONTENT);
      
      createSignatureEnvironment.appendChild(base64Content);
      base64Content.appendChild(doc.createTextNode(new String(Util.Base64Encode(docToEmbedBase64_))));
    } else if (locRefContent_ != null) {
      Element locRefContent = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.LOC_REF_CONTENT);
      
      createSignatureEnvironment.appendChild(locRefContent);
      locRefContent.appendChild(doc.createTextNode(locRefContent_));
    }
    Element createSignatureEnvironmentProfile = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CREATE_SIGNATURE_ENVIRONMENT_PROFILE);
    createSignatureInfo.appendChild(createSignatureEnvironmentProfile);
    
    Element createSignatureLocation = doc.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CREATE_SIGNATURE_LOCATION);
    createSignatureEnvironmentProfile.appendChild(createSignatureLocation);
    createSignatureLocation.setAttribute("Index", String.valueOf(location_));
    if (namespacePrefixes_ != null) {
      for (int i=0; i<namespacePrefixes_.size(); i++) {
        createSignatureLocation.setAttribute((String)namespacePrefixes_.elementAt(i), (String)namespaceURI_.elementAt(i));
      }
    }
    createSignatureLocation.appendChild(doc.createTextNode(xPathExpression_));
      
    return singleSignatureInfo;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SingleSignatureInfo#addReference(asit.framework.moaclient.SignReference)
   */
  public void addReference(SignReference reference) {
    if (reference != null) {
      if (references_ == null) {
        references_ = new Vector();
      }
      references_.addElement(reference);
    }
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.SingleSignatureInfo#addDocumentToEmbedNamespace(java.lang.String, java.lang.String)
   */
  public void addDocumentToEmbedNamespace(String prefix, String uri) {
    
    if (namespacePrefixes_ == null) {
      namespacePrefixes_ = new Vector();
    }
    if (namespaceURI_ == null) {
      namespaceURI_ = new Vector();
    }
    namespacePrefixes_.addElement(prefix);
    namespaceURI_.addElement(uri);
  }

  /* (non-Javadoc)
   * @see asit.moa.client.sign.SingleSignatureInfo#setEnvironmentReference(java.lang.String)
   */
  public void setEnvironmentReference(String reference) {
    environmentReference_ = reference;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.sign.SingleSignatureInfo#setDocumentToEmbed(org.w3c.dom.Element)
   */
  public void setDocumentToEmbed(Element element) {
    docToEmbed_ = element;
  }


  
  /* (non-Javadoc)
   * @see asit.moa.client.sign.SingleSignatureInfo#setDocumentToEmbedBase64(byte[])
   */
  public void setDocumentToEmbedBase64(byte[] document) {
    docToEmbedBase64_ = document;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.sign.SingleSignatureInfo#setDocumentToEmbedLocRefContent(java.lang.String)
   */
  public void setDocumentToEmbedLocRefContent(String location) {
    locRefContent_ = location;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.sign.SingleSignatureInfo#setSignatureLocation(java.lang.String, int)
   */
  public void setSignatureLocation(String xpath, int position) {
    xPathExpression_ = xpath;
    location_ = position;
    
  }

  /* (non-Javadoc)
   * @see asit.moa.client.sign.SingleSignatureInfo#setDocumentToEmbed(org.w3c.dom.NodeList)
   */
  public void setDocumentToEmbed(NodeList list) {
    nodesToEmbed_ = list;
    
  }
}
