package asit.moa.client.verify;

import iaik.asn1.structures.Name;
import iaik.x509.X509Certificate;
import java.math.BigInteger;
import java.util.Date;
import java.util.List;

import org.w3c.dom.Element;

/**
 * @author <a href="mailto:arne.tauber@iaik.tugraz.at">Arne Tauber</a>
 * @author modified by <a href="mailto:thomas.knall@iaik.tugraz.at">Thomas Knall</a>
 */
public interface VerifyResult {

	Name getSubjectName();

	/**
	 * Returns the (subject's) serial number (from {@code X509IssuerSerial}-node).
	 * 
	 * @return The subject's serial number.
	 */
	BigInteger getSubjectSerial();

	Name getIssuerName();

	/**
	 * Returns the serial number from the {@code X509IssuerSerial}-node. In fact
	 * that node contains the subject's serial number not the issuer's. So ignore
	 * the misleading method name.
	 * 
	 * @return The subject's serial number.
	 * @deprecated Use {@link #getSubjectSerial()} instead.
	 */
	BigInteger getIssuerSerial();

	X509Certificate getCertificate();

	boolean isQualifiedCertificate();

	boolean isPublicAuthority();

	/**
    * @deprecated Use {@link #getPublicAuthorityCode()} instead.
    */
   String getPublicAuthorithyCode();

   String getPublicAuthorityCode();

	int getSignatureCheckCode();

	/**
	 * Returns the failed signature references.
	 * 
	 * @return An integer array of failed signature references.
	 * @deprecated Use {@link #getFailedSignatureReferences()} instead.
	 */
	int[] getFailedReferences();

	int[] getFailedSignatureReferences();

	int getManifestCheckCode();
	
	int getXmldsigManifestCheckCode();

	int[] getFailedManifestReferences();

	int getCertificateCheckCode();

	/**
	 * Returns the MOA {@code VerifyXMLSignatureResponse}.
	 * 
	 * @return The MOA {@code VerifyXMLSignatureResponse}.
	 */
	Element getVerifyXMLSignatureResponse();

	/**
	 * Returns date (and time) the verification was based on.
	 * 
	 * @return The date of verification.
	 */
	Date getDateOfVerification();
	
   /**
    * Returns an array of {@link HashInputData} elements each containing the referenced content of
    * a {@code dsig:Reference} or {@code null} if there are no {@code HashInputData} elements.
    * 
    * @return An array of {@link HashInputData} elements or {@code null}.
    */
	HashInputData[] getHashInputData();

}
