package asit.moa.client.verify;

import iaik.asn1.ObjectID;
import iaik.ixsil.exceptions.UtilsException;
import iaik.ixsil.util.Utils;
import iaik.utils.Base64OutputStream;
import iaik.utils.RFC2253NameParser;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.apache.log4j.Logger;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import asit.common.LogFactory;
import asit.framework.servlet.notification.VerifyNotificationFactory;
import asit.moa.MoaConstants;
import asit.moa.client.AbstractClient;
import asit.moa.client.exception.ClientException;


/**
 * @author atauber
 */
public class VerifySignatureClientImpl extends AbstractClient
    implements
      VerifySignatureClient {

  
  protected Vector additionalNamespaces_ = null;
  protected Hashtable additionalReferences_ = null;
  protected Date verifyDate_ = null;
  protected Element xmlSignature_ = null;
  protected String base64Signature_ = null;
  protected String xpathLocation_ = null;
  protected String trustProfileID_ = null;
  
  protected int timeZoneHours_ = 0;
  protected int timeZoneMinutes_ = 0;
  
  protected boolean returnHashInputData_ = false;
  
  protected static Logger log_ = LogFactory.getLogger(VerifyNotificationFactory.class.getName());
  
  /**
   * @throws ClientException
   */
  public VerifySignatureClientImpl() throws ClientException {
    super();
    qname_service_ = new QName("SignatureVerification");
  }

  public void setDate(Date date) {
    verifyDate_ = date;
  }

  
  public void setContent(Element signature) {
    xmlSignature_ = signature;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.VerifiySignatureClient#setSignature(byte[])
   */
  
  public void setContent(String signatur) throws ClientException {
    base64Signature_ = signatur;
  }
  
  public void setContent(byte[] signature) throws ClientException {
    try {
      base64Signature_ = new String(Utils.base64Encode(signature));
    } catch (UtilsException e) {
      e.printStackTrace();
      throw new ClientException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.VerifiySignatureClient#setSignatureLocation(java.lang.String)
   */
  public void setSignatureLocation(String xpathLocation) {
    xpathLocation_ = xpathLocation;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.VerifiySignatureClient#setTrustProfileID(java.lang.String)
   */
  public void setTrustProfileID(String id) {
    trustProfileID_ = id;
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.Client#getRequestDocument()
   */
  public Element getRequest() throws ClientException {
    //doc_ = super.createNewDocument();
    
    Element root = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.VERIFY_XML_SIGNATURE_REQUEST);
    root.setAttribute("xmlns" + MoaConstants.MOA_POSTFIX, MoaConstants.MOA_NS);
    
    // add additional namespaces
    if (additionalNamespaces_ != null) {
      for (int i=0; i<additionalNamespaces_.size(); i++) {
        Namespace ns = (Namespace)additionalNamespaces_.elementAt(i);
        if (ns.prefix_ == null || "".equals(ns.prefix_)) {
           // attribute must satisfy (NCName':')?NCName so we must not set a ":" for default namespace
      	  root.setAttribute("xmlns", ns.uri_);
      	  log_.warn("MOA-SP does not like the default namespace for parts of signature location. Use some prefix instead.");
        } else {
           root.setAttribute("xmlns:" + ns.prefix_, ns.uri_);
        }
      }
    }
    
    // add datetime
    if (verifyDate_ != null) {
       Element dateTime = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.DATE_TIME);
       dateTime.appendChild(doc_.createTextNode(formatDate()));
       root.appendChild(dateTime);
    }
    
    // element SignatureInfo
    Element signatureInfo = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.VERIFY_SIGNATURE_INFO);
    root.appendChild(signatureInfo);
    
    Element verifySignatureEnvironment = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.VERIFY_SIGNATURE_ENVIRONMENT);
    signatureInfo.appendChild(verifySignatureEnvironment);
    // add content to verify
    if (xmlSignature_ != null) {
      Element xmlContent = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.XML_CONTENT);
      //xmlContent.setAttribute("xml:space", "preserve");
      Node importNode = doc_.importNode(xmlSignature_, true);
      xmlContent.appendChild(importNode);
      verifySignatureEnvironment.appendChild(xmlContent);
    } else if (base64Signature_ != null) {
      Element base64Content = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.BASE64_CONTENT);
      base64Content.appendChild(doc_.createTextNode(base64Signature_));
      verifySignatureEnvironment.appendChild(base64Content);
    } else {
      throw new NullPointerException("One of XMLContent or Base64Content must not be null. Define first a content to verify!!");
    }
    
    if (xpathLocation_ == null) {
      throw new NullPointerException("Please define first the location of the signature.");
    }
    Element verifySignatureLocation = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.VERIFY_SIGNATURE_LOCATION);
    verifySignatureLocation.appendChild(doc_.createTextNode(xpathLocation_));
    signatureInfo.appendChild(verifySignatureLocation);

    if(additionalReferences_ != null) {
        // insert additional external file references as base64 encoded data
        java.util.Iterator iter = additionalReferences_.keySet().iterator();
        while(iter.hasNext()) {
            
            String uri = (String)iter.next();            

            Element supplementProfile = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.SUPPLEMENT_PROFILE);            
            Node supplementData = null;
            
            boolean use_base64 = true;
            if(use_base64) {
                
                try {
                    supplementData = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.BASE64_CONTENT);
                    
                    ByteArrayOutputStream referenceb64 = new ByteArrayOutputStream();
                    InputStream reference = (InputStream)additionalReferences_.get(uri);
                    Base64OutputStream b64 = new Base64OutputStream(referenceb64);
                    byte[] reference_content = new byte[reference.available()];
                    reference.read(reference_content);
                    b64.write(reference_content);
                    b64.close();
                    reference.close();
                    
                    supplementData.appendChild(doc_.createTextNode(new String(referenceb64.toByteArray())));    
                }
                catch(java.io.IOException e) {
                    throw new ClientException(e);
                }
                
            } else {
                
                try {
                    supplementData = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.XML_CONTENT);
                    
                    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                    factory.setNamespaceAware(true);
                    DocumentBuilder builder = factory.newDocumentBuilder();
                    Element supplement = builder.parse((InputStream)additionalReferences_.get(uri)).getDocumentElement();
                    Node data = doc_.importNode(supplement, true);
                    
                    supplementData.appendChild(data);
                }
                catch(Exception e) {
                    throw new ClientException(e);
                }
                
            }   
            
            Element supplementContent = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.CONTENT);
            supplementContent.setAttribute(MoaConstants.REFERENCE, uri);
            supplementContent.appendChild(supplementData);
            supplementProfile.appendChild(supplementContent);
            root.appendChild(supplementProfile);
        }
    }
    
    if (returnHashInputData_) {
      Element hashInputData = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.HASH_INPUT_DATA);
      root.appendChild(hashInputData);
    }
    
    
    if(trustProfileID_ == null) {
        throw new NullPointerException("Please define first a trust profile id.");
    }
        

    Element trustProfileID = doc_.createElementNS(MoaConstants.MOA_NS, MoaConstants.MOA_PREFIX + MoaConstants.TRUST_PROFILE_ID);
    trustProfileID.appendChild(doc_.createTextNode(trustProfileID_));
    root.appendChild(trustProfileID);

    return root;
  }

  protected String formatDate() {
    if (verifyDate_ == null) {
      verifyDate_ = new Date();
    }

    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

    String timeZoneString = new SimpleDateFormat("Z").format(verifyDate_);
    
    String prefix = timeZoneString.substring(0, 3);
    String postfix = timeZoneString.substring(3);

    return format.format(verifyDate_).toString() + prefix + ":" + postfix;
  }
  
  
  public void addAdditionalReference(String uri, InputStream reference) throws ClientException
  {
     if(additionalReferences_ == null) {
         additionalReferences_ = new Hashtable();
     }
     if(additionalReferences_.containsKey(uri)) {
         throw new ClientException("Additional reference of URI \"" + uri + "\" exists.");
     }
     additionalReferences_.put(uri, reference);
  }

  public void addAdditionalNamespace(String prefix, String uri) {
    if (prefix == null) {
      throw new NullPointerException("Argument \"prefix\" must not be null.");
    }
    if (uri == null) {
      throw new NullPointerException("Argument \"uri\" must not be null.");
    }
    if (additionalNamespaces_ == null) {
      additionalNamespaces_ = new Vector();
    }
    additionalNamespaces_.addElement(new Namespace(prefix, uri));
  }

  /* (non-Javadoc)
   * @see asit.framework.moaclient.VerifySignatureClient#sendVerifyRequest(java.lang.String)
   */
  public VerifyResult sendVerifyRequest(String address) throws ClientException {
      
      try {
      	RFC2253NameParser.register("givenName", ObjectID.givenName);
			Element response = super.sendRequest(address);
			VerifyResultImpl resultImpl = new VerifyResultImpl(response);
			resultImpl.dateOfVerification_ = this.verifyDate_;
			  
			return resultImpl;
        

        /*SimpleVerifyResultFactory result = new SimpleVerifyResultFactory();
        NodeList signatures = XPathAPI.selectNodeList(response, "/VerifyXMLSignatureResponse/SignerInfo");
        // ok, we have a signer info
        if (signatures.getLength() != 0) {
          
          // statically register potential RFC2253 shortname's and appertaining ObjectID's
          RFC2253NameParser.register("givenName", ObjectID.givenName);
          
          // extract parameters
          XObject signerDN = XPathAPI.eval(response, "/VerifyXMLSignatureResponse/SignerInfo/X509Data/X509SubjectName/text()");
          XObject issuerDN = XPathAPI.eval(response, "/VerifyXMLSignatureResponse/SignerInfo/X509Data/X509IssuerSerial/X509IssuerName/text()");
          XObject certSerial = XPathAPI.eval(response, "/VerifyXMLSignatureResponse/SignerInfo/X509Data/X509IssuerSerial/X509SerialNumber/text()");
          XObject cert = XPathAPI.eval(response, "/VerifyXMLSignatureResponse/SignerInfo/X509Data/X509Certificate/text()");
          XObject signatureCheck = XPathAPI.eval(response, "/VerifyXMLSignatureResponse/SignatureCheck/Code/text()");
          XObject certificateCheck = XPathAPI.eval(response, "/VerifyXMLSignatureResponse/CertificateCheck/Code/text()");
          
          if (signatureCheck != null) {
            if ("0".equals(signatureCheck.str())) {
              result.setSignatureCheck(true);
              result.setSignatureCheckResult("Die berprfung der Hash-Werte und des Werts der Signatur konnte erfolgreich durchgefhrt werden.");
            } else {
              result.setSignatureCheckResult("Die berprfung der Hash-Werte und des Werts der Signatur konnte NICHT erfolgreich durchgefhrt werden.");
            }
          }

          if (certificateCheck != null) {
            if ("0".equals(certificateCheck.str())) {
              result.setCertificatecheck(true);
              result.setCertificateCheckResult("Eine formal korrekte Zertifikatskette vom Signatorzertifikat zu einem vertrauenswrdigen Wurzelzertifikat konnte konstruiert werden. Jedes Zertifikat dieser Kette ist zum in der Anfrage angegebenen Prfzeitpunkt gltig.");
            } else {
              result.setCertificateCheckResult("Eine formal korrekte Zertifikatskette vom Signatorzertifikat zu einem vertrauenswrdigen Wurzelzertifikat konnte NICHT konstruiert werden.");
            }
          }

          
          // certificate's serial number
          if (certSerial != null) {
            result.setCertSerial(certSerial.str());
          }
          
          // certificate quality
          logger_.debug("Parsed certificate xpath entry is null: " + (cert == null));
          if (cert != null) {
            try {
              logger_.debug("Parsing certificate.");
              
              X509Certificate certificate = new X509Certificate(Utils.base64Decode(cert.str().getBytes("UTF-8")));
              result.setCertificate(certificate);
              
              logger_.debug("Parsing certificate is null." + (certificate == null));
              
              logger_.debug("Parsing BasicConstraints");
              V3Extension ext = certificate.getExtension(BasicConstraints.oid);
              if (ext != null) {
                BasicConstraints bc = (BasicConstraints)ext;
                logger_.debug("BasicConstraints is CA: " + bc.ca());
                if (!bc.ca()) {
                  result.setCertQuality("Gewhnliches Zertifikat");
                } else {
                  result.setCertQuality("Intermediate oder Root CA");
                }
              } else {
                result.setCertQuality("Gewhnliches Zertifikat");
              }
            } catch(Exception e) {
              //throw new ClientException(e);
            }
          }
          
          // signer's DN
          if (signerDN != null) {
            RFC2253NameParser parser = new RFC2253NameParser(signerDN.str());
            try {
              Name name = parser.parse();
              result.setSignerName(name.getRDN(ObjectID.commonName));
              result.setSignerOrganizationalUnit(name.getRDN(ObjectID.organizationalUnit));
              result.setSignerOrganization(name.getRDN(ObjectID.organization));
              result.setSignerCountry(name.getRDN(ObjectID.country));
            } catch (RFC2253NameParserException e2) {
              e2.printStackTrace();
              throw new ClientException(e2);
            }
          }
          
          // issuer's DN
          if (issuerDN != null) {
            RFC2253NameParser parser = new RFC2253NameParser(issuerDN.str());
            try {
              Name name = parser.parse();
              result.setIssuerName(name.getRDN(ObjectID.commonName));
              result.setIssuerOrganizationalUnit(name.getRDN(ObjectID.organizationalUnit));
              result.setIssuerOrganization(name.getRDN(ObjectID.organization));
              result.setIssuerCountry(name.getRDN(ObjectID.country));
            } catch (RFC2253NameParserException e2) {
              e2.printStackTrace();
              throw new ClientException(e2);
            }
          }
          return result;
        } else {
          // get error
          throw new ClientException("Unknown response format.");
        }
       */ 
      } catch (Exception e) {
        e.printStackTrace();
        throw new ClientException(e);
      }
    }

  /*
   * @see asit.moa.client.verify.VerifySignatureClient#setReturnHashInputData(boolean)
   */
  public void setReturnHashInputData(boolean value) {
    returnHashInputData_ = value;
  }

  /* (non-Javadoc)
   * @see asit.moa.client.verify.VerifySignatureClient#sendVerifyRequest()
   */
  public VerifyResult sendVerifyRequest() throws ClientException {
    return sendVerifyRequest(defaultAddress_);
  }
}

// inline class for handling namespaces
class Namespace {
 
  String prefix_ = null;
  String uri_ = null;
  
  Namespace(String prefix, String uri) {
    prefix_ = prefix;
    uri_ = uri;
  }
  
  
}