package asit.moa.moaid;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URLEncoder;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.catalina.HttpRequest;
import org.apache.catalina.HttpResponse;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.Realm;
import org.apache.catalina.Session;
import org.apache.catalina.authenticator.AuthenticatorBase;
import org.apache.catalina.authenticator.Constants;
import org.apache.catalina.authenticator.SavedRequest;
import org.apache.catalina.deploy.LoginConfig;
import org.apache.catalina.realm.GenericPrincipal;



/**
 * MOA-ID Valve
 * 
 * @author <a href="mailto:arne.tauber@iaik.tugraz.at">Arne Tauber</a>
 */
public class MOAIDAuthenticator
    extends AuthenticatorBase {


    // ----------------------------------------------------- Instance Variables


    /**
     * Descriptive information about this implementation.
     */
    protected static final String info =
        "org.apache.catalina.authenticator.FormAuthenticator/1.0";
    
    private String bkuAuswahlURL_ = null;
    
    private String moaidURL_ = null;


    private String oaURL_;


    private String sslStore_;


    private String sslPwd_;

    private boolean passEnabled_;

    private String errorPage_;

    // muss leer sein
    private List autoRoles_ = new ArrayList();

    private String target_;

    private String bkuTemplate_;
    
    private final static String REQUEST_URL = "ORIGINAL_REQUEST_URL";


    private String templateURL_;


    private String sslStoreType_;
    
    // ------------------------------------------------------------- Properties


    /**
     * Return descriptive information about this Valve implementation.
     */
    public String getInfo() {

        return (MOAIDAuthenticator.info);

    }


    // --------------------------------------------------------- Public Methods


    /**
     * Authenticate the user making this request, based on the specified
     * login configuration.  Return <code>true</code> if any specified
     * constraint has been satisfied, or <code>false</code> if we have
     * created a response challenge already.
     *
     * @param request Request we are processing
     * @param response Response we are creating
     * @param config    Login configuration describing how authentication
     *              should be performed
     *
     * @exception IOException if an input/output error occurs
     */
    public boolean authenticate(HttpRequest request,
                                HttpResponse response,
                                LoginConfig config)
        throws IOException {

        
      //System.out.println("\n\n\nAuthenticatin...\n");
      //System.out.println("OK:: " +  MOAIDAuthenticator.class.getResourceAsStream("/test.txt.1"));
      
//      try {
//        Class.forName("Dumper");
//      } catch(Exception e) {
//        e.printStackTrace();
//      }
      
        log("Ok, now we are authenticating...");
      
        // References to objects we will need later
        HttpServletRequest hreq =
          (HttpServletRequest) request.getRequest();
        HttpServletResponse hres =
          (HttpServletResponse) response.getResponse();
        Session session = null;

        
        // Have we already authenticated someone?
        
        Principal principal = hreq.getUserPrincipal();
        /*log("Session: " + (session == null) + "/" + (getSession(request, true) == null));
        if (principal == null &&
            getSession(request, true) != null) {
          log("Hole aus Session.");
          principal = (Principal)getSession(request, true).getNote(Constants.FORM_PRINCIPAL_NOTE);
          log("!= null? " + (principal != null));
        }*/
        String ssoId = (String) request.getNote(Constants.REQ_SSOID_NOTE);
        if (principal != null) {
            
          log("Already authenticated '" +
                    principal.getName() + "'");
                //request.setUserPrincipal(principal);
            // Associate the session with any existing SSO session
            if (ssoId != null)
                associate(ssoId, getSession(request, true));
            return (true);
        }

        // Is there an SSO session against which we can try to reauthenticate?
        if (ssoId != null) {
          log("SSO Id " + ssoId + " set; attempting reauthentication");
            // Try to reauthenticate using data cached by SSO.  If this fails,
            // either the original SSO logon was of DIGEST or SSL (which
            // we can't reauthenticate ourselves because there is no
            // cached username and password), or the realm denied
            // the user's reauthentication for some reason.
            // In either case we have to prompt the user for a logon */
            if (reauthenticateFromSSO(ssoId, request))
                return true;
        }

        // Have we authenticated this user before but have caching disabled?
        if (!cache) {
            session = getSession(request, true);
            log("Checking for reauthenticate in session " + session);
            String username =
                (String) session.getNote(Constants.SESS_USERNAME_NOTE);
            String password =
                (String) session.getNote(Constants.SESS_PASSWORD_NOTE);
            if ((username != null) && (password != null)) {
              log("Reauthenticating username '" + username + "'");
                principal =
                    context.getRealm().authenticate(username, password);
                if (principal != null) {
                    session.setNote(Constants.FORM_PRINCIPAL_NOTE, principal);
                    register(request, response, principal,
                             Constants.FORM_METHOD,
                             username, password);
                    return (true);
                }
                log("Reauthentication failed, proceed normally");
            }
        }

        // Is this the re-submit of the original request URI after successful
        // authentication?  If so, forward the *original* request instead.
        if (matchRequest(request)) {
            session = getSession(request, true);
            log("Restore request from session '" + session.getId() + "'");
            principal = (Principal)
                session.getNote(Constants.FORM_PRINCIPAL_NOTE);
            register(request, response, principal, "MOAID",
                     (String) session.getNote(Constants.SESS_USERNAME_NOTE),
                     (String) session.getNote(Constants.SESS_PASSWORD_NOTE));
            if (restoreRequest(request, session)) {
              log("Proceed to restored request");
                return (true);
            } else {
              log("Restore of original request failed");
                hres.sendError(HttpServletResponse.SC_BAD_REQUEST);
                return (false);
            }
        }

        log("Ok, performing authentication...");
        
        // Acquire references to objects we will need to evaluate
        String contextPath = hreq.getContextPath();
        String requestURI = request.getDecodedRequestURI();
        response.setContext(request.getContext());

        // Is this a request for the login page itself?  Test here to avoid
        // displaying it twice (from the user's perspective) -- once because
        // of the "save and redirect" and once because of the "restore and
        // redirect" performed below.
        
        boolean isPasswordAuthentication = requestURI.endsWith("/j_security_check");
        
        String loginURI = null;
        
        
        
        // ok, check if password based authentication
        String passLoginURI = contextPath + config.getLoginPage();
        // name des servlets
        String servletName = bkuTemplate_ == null ? "StartAuthentication" : "SelectBKU";
        
        loginURI = moaidURL_ +
                  "moa-id-auth/" + servletName + "?Target=" +
                  URLEncoder.encode(target_, "UTF-8") +
                  "&OA=" + URLEncoder.encode(oaURL_, "UTF-8") + "moaid-login";
        
        if (bkuTemplate_ != null) {
          loginURI += "&BKUSelectionTemplate=" + URLEncoder.encode(bkuTemplate_, "UTF-8"); 
        }
        
        // fuege template url hinzu
        if (templateURL_ != null) {
          loginURI += "&Template=" + URLEncoder.encode(templateURL_, "UTF-8");
        }
        
        // dies trifft nur fuer passwort authentication zu
        if (requestURI.equals(passLoginURI)) {
                log("Requesting login page normally");
            return (true);      // Display the login page in the usual manner
        }

        // check if we have a bku auswahl page - if so, redirect to this page
        if (bkuAuswahlURL_ != null &&
            !"".equals(bkuAuswahlURL_)) {
          loginURI = bkuAuswahlURL_;
        }
        
        // Is this a request for the error page itself?  Test here to avoid
        // an endless loop (back to the login page) if the error page is
        // within the protected area of our security constraint
        String errorURI = errorPage_; //contextPath + config.getErrorPage();
        if (requestURI.equals(errorURI)) {
                log("Requesting error page normally");
            return (true);      // Display the error page in the usual manner
        }

        // Is this the action request from the login page?
        boolean loginAction =
            requestURI.startsWith(contextPath) &&
            requestURI.endsWith("/moaid-login");
        
        if (isPasswordAuthentication) {
          loginAction = true;
        }
        
        // No -- Save this request and redirect to the form login page
        if (!loginAction) {
            session = getSession(request, true);
            log("Save request in session '" + session.getId() + "'");
            saveRequest(request, session);
            log("Redirect to login page '" + loginURI + "'");
            //hres.sendRedirect(hres.encodeRedirectURL(loginURI));
            
            // manual redirect
            PrintWriter writer = hres.getWriter();
            hres.setContentType("text/html");
            String redirectString = "<html><head><title>redirect</title>" +
                "<meta meta http-equiv=\"refresh\" content=\"0 url=" + loginURI + "\">" +
                "<body/></html>";
                
            writer.println(redirectString);
            
            return (false);
        }
        
        // Yes -- Validate the specified credentials and redirect
        // to the error page if they are not correct
        Realm realm = new MOAIDRealm();//context.getRealm();
        String username = null;
        String password = null;
        
        log("Checking authentication: " + isPasswordAuthentication);
        
        if (isPasswordAuthentication) {
          username = hreq.getParameter("j_username");
          password = hreq.getParameter("j_password");
        } else {
        
          log("Realm instance of MOA-ID: " + (realm instanceof MOAIDRealm));
          
          // setze moa-id daten
          if (realm instanceof MOAIDRealm) {
            log("Realm is MOA-ID");
            log("SSLStore: " + sslStore_);
            ((MOAIDRealm)realm).setMOAIDUrl(moaidURL_ + "moa-id-auth/services/GetAuthenticationData"); // url von moa-id
            ((MOAIDRealm)realm).setMOAIDSSLStore(sslStore_); // ssl trust store
            ((MOAIDRealm)realm).setMOAIDSSLPassword(sslPwd_); // ssl password
            ((MOAIDRealm)realm).setMOAIDSSLStoreType(sslStoreType_); // ssl store type
            ((MOAIDRealm)realm).setAutoRoles(autoRoles_); // auto roles
            log("Ok, parameters are set.");
          } else {
            log("We do not have a MOA-ID Realm");
          }
          
          username = hreq.getParameter("SAMLArtifact");
          password = hreq.getParameter("");
          //if (debug >= 1)
          log("Authenticating username '" + username + "'");
        }
        
        log("No do REALM authentication...");
        
        principal = realm.authenticate(username, password);
        
        log("OK done REALM authentication...");
        
        if (principal == null) {
          log("Redirect to error page '" + errorURI + "'");
            hres.sendRedirect(hres.encodeRedirectURL(errorURI));
            return (false);
        }

        log("Authentication of '" + username + "' was successful");

        GenericPrincipal data = (GenericPrincipal)principal;
        
        
        if (data != null &&
            data.getRoles() != null) {
          log("Number of roles found: " + data.getRoles().length);
        }
        
        if (session == null)
            session = getSession(request, false);
        
        if (session == null) {
            
              log("User took so long to log on the session expired");
            hres.sendError(HttpServletResponse.SC_REQUEST_TIMEOUT,
                           "The time allowed for the login process has been exceeded. If you wish to continue you must either click back twice and re-click the link you requested or close and re-open your browser");
            return (false);
        }
        
        
        // Save the authenticated Principal in our session
        session.setNote(Constants.FORM_PRINCIPAL_NOTE, principal);

        // If we are not caching, save the username and password as well
        if (!cache) {
            session.setNote(Constants.SESS_USERNAME_NOTE, username);
            session.setNote(Constants.SESS_PASSWORD_NOTE, password);
        }

        // Redirect the user to the original request URI (which will cause
        // the original request to be restored)
        requestURI = savedRequestURL(session);
        
        // cut off beginning /
        
        String original = originalRequestURL(session);
        
        log("Redirecting to original '" + requestURI + "'");
        
        
        if (requestURI == null)
            hres.sendError(HttpServletResponse.SC_BAD_REQUEST,
                           "Invalid direct reference to form login page");
        else
            hres.sendRedirect(hres.encodeRedirectURL(original));
        return (false);

    }


    // ------------------------------------------------------ Protected Methods


    public void setBkuAuswahlURL(String url) {
      bkuAuswahlURL_ = url;
    }
    
    public String getBkuAuswahlURL() {
      return bkuAuswahlURL_;
    }
    
    public void setMoaidurl(String url) {
      moaidURL_ = url;
    }
    
    public String getMoaidurl() {
      return moaidURL_;
    }
    
    public void setOaurl(String url) {
      oaURL_ = url;
    }
    
    public String getOaurl() {
      return oaURL_;
    }
    
    public void setBkutemplate(String template) {
      bkuTemplate_ = template;
    }
    
    public String getBkutemplate() {
      return bkuTemplate_;
    }
    
    public void setTemplateurl(String url) {
      templateURL_ = url;
    }
    
    public String getTemplateurl() {
      return templateURL_;
    }
    
    public void setAutoRoles(String roles) {
      String[] roleList = roles.split(",");
      if (roleList != null) {
        autoRoles_ = new ArrayList();
        for (int i=0; i<roleList.length; i++) {
          autoRoles_.add(roleList[i]);
        }
      }
    }
    
    public List getAutoRoleList() {
      return autoRoles_;
    }
    
    // Diese Funktion wird nicht benoetigt
    public String getAutoRoles() {
      return null;
    }
    
    public void setTarget(String url) {
      target_ = url;
    }
    
    public String getTarget() {
      return target_;
    }
    
    public void setErrorpage(String url) {
      errorPage_ = url;
    }
    
    public String getErrorpage() {
      return errorPage_;
    }
    
    public void setSslstore(String url) {
      sslStore_ = url;
    }
    
    public String getSslstore() {
      return sslStore_;
    }
    
    public void setSslpwd(String url) {
      sslPwd_ = url;
    }
    
    public String getSslpwd() {
      return sslPwd_;
    }
    
    /**
     * Does this request match the saved one (so that it must be the redirect
     * we signalled after successful authentication?
     *
     * @param request The request to be verified
     */
    protected boolean matchRequest(HttpRequest request) {

      // Has a session been created?
      Session session = getSession(request, false);
      if (session == null)
          return (false);

      // Is there a saved request?
      SavedRequest sreq = (SavedRequest)
          session.getNote(Constants.FORM_REQUEST_NOTE);
      if (sreq == null)
          return (false);

      // Is there a saved principal?
      if (session.getNote(Constants.FORM_PRINCIPAL_NOTE) == null)
          return (false);

      // Does the request URI match?
      HttpServletRequest hreq = (HttpServletRequest) request.getRequest();
      String requestURI = hreq.getRequestURI();
      if (requestURI == null)
          return (false);
      return (requestURI.equals(sreq.getRequestURI()));

    }


    /**
     * Restore the original request from information stored in our session.
     * If the original request is no longer present (because the session
     * timed out), return <code>false</code>; otherwise, return
     * <code>true</code>.
     *
     * @param request The request to be restored
     * @param session The session containing the saved information
     */
    protected boolean restoreRequest(HttpRequest request, Session session) {

        // Retrieve and remove the SavedRequest object from our session
        SavedRequest saved = (SavedRequest)
            session.getNote(Constants.FORM_REQUEST_NOTE);
        session.removeNote(Constants.FORM_REQUEST_NOTE);
        session.removeNote(Constants.FORM_PRINCIPAL_NOTE);
        if (saved == null)
            return (false);

        // Modify our current request to reflect the original one
        request.clearCookies();
        Iterator cookies = saved.getCookies();
        while (cookies.hasNext()) {
            request.addCookie((Cookie) cookies.next());
        }
        request.clearHeaders();
        Iterator names = saved.getHeaderNames();
        while (names.hasNext()) {
            String name = (String) names.next();
            Iterator values = saved.getHeaderValues(name);
            while (values.hasNext()) {
                request.addHeader(name, (String) values.next());
            }
        }
        request.clearLocales();
        Iterator locales = saved.getLocales();
        while (locales.hasNext()) {
            request.addLocale((Locale) locales.next());
        }
        request.clearParameters();
        if ("POST".equalsIgnoreCase(saved.getMethod())) {
            Iterator paramNames = saved.getParameterNames();
            while (paramNames.hasNext()) {
                String paramName = (String) paramNames.next();
                String paramValues[] =
                    saved.getParameterValues(paramName);
                request.addParameter(paramName, paramValues);
            }
        }
        request.setMethod(saved.getMethod());
        request.setQueryString(saved.getQueryString());
        request.setRequestURI(saved.getRequestURI());
        return (true);

    }


    /**
     * Save the original request information into our session.
     *
     * @param request The request to be saved
     * @param session The session to contain the saved information
     */
    private void saveRequest(HttpRequest request, Session session) {

        // Create and populate a SavedRequest object for this request
        HttpServletRequest hreq = (HttpServletRequest) request.getRequest();
        SavedRequest saved = new SavedRequest();
        Cookie cookies[] = hreq.getCookies();
        if (cookies != null) {
            for (int i = 0; i < cookies.length; i++)
                saved.addCookie(cookies[i]);
        }
        Enumeration names = hreq.getHeaderNames();
        while (names.hasMoreElements()) {
            String name = (String) names.nextElement();
            Enumeration values = hreq.getHeaders(name);
            while (values.hasMoreElements()) {
                String value = (String) values.nextElement();
                saved.addHeader(name, value);
            }
        }
        Enumeration locales = hreq.getLocales();
        while (locales.hasMoreElements()) {
            Locale locale = (Locale) locales.nextElement();
            saved.addLocale(locale);
        }
        Map parameters = hreq.getParameterMap();
        Iterator paramNames = parameters.keySet().iterator();
        while (paramNames.hasNext()) {
            String paramName = (String) paramNames.next();
            String paramValues[] = (String[]) parameters.get(paramName);
            saved.addParameter(paramName, paramValues);
        }
        saved.setMethod(hreq.getMethod());
        saved.setQueryString(hreq.getQueryString());
        saved.setRequestURI(hreq.getRequestURI());

        // Stash the SavedRequest in our session for later use
        session.setNote(Constants.FORM_REQUEST_NOTE, saved);
        
        session.setNote(REQUEST_URL, hreq.getRequestURL());

    }


    /**
     * Return the request URI (with the corresponding query string, if any)
     * from the saved request so that we can redirect to it.
     *
     * @param session Our current session
     */
    private String savedRequestURL(Session session) {

        SavedRequest saved =
            (SavedRequest) session.getNote(Constants.FORM_REQUEST_NOTE);
        if (saved == null)
            return (null);
        StringBuffer sb = new StringBuffer(saved.getRequestURI());
        
        if (saved.getQueryString() != null) {
            sb.append('?');
            sb.append(saved.getQueryString());
        }
        return (sb.toString());

    }

    private String originalRequestURL(Session session) {
      Object note = session.getNote(REQUEST_URL);
      
      if (note == null) {
        return null;
      }
      return note.toString();
    }
    

  public void start() throws LifecycleException {
    super.start();
    setCache(true);
    log("Initialisiere.");
  }
  
    public String getSslStoreType_() {
      return sslStoreType_;
    }
    public void setSslStoreType_(String sslStoreType_) {
      this.sslStoreType_ = sslStoreType_;
    }
}
