package asit.moa.moaid;

import java.beans.PropertyChangeListener;
import java.security.Principal;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;
import org.apache.catalina.Container;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.Realm;
import org.apache.catalina.ServerFactory;
import org.apache.catalina.core.StandardServer;
import org.apache.catalina.realm.GenericPrincipal;
import org.apache.catalina.realm.RealmBase;
import asit.moa.client.ClientFactory;
import asit.moa.client.id.MOAIDClient;
import asit.moa.client.id.MOAIDPrincipal;

public class MOAIDRealm extends RealmBase implements Realm {

  private Container container;
  private String moaidurl_;
  private String sslstore_;
  private String sslpwd_;

  private List autoRoles_ = new ArrayList();
  
  protected String resourceName = "RoleVerificator";
  
  protected RoleVerificator roleVerificator_ = null;
  private String sslstoretype_;

  //protected static Logger log_ = LogFactory.getLogger(MOAIDRealm.class.getName());
  
  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#getContainer()
   */
  public Container getContainer() {
    return this.container;
  }

  
  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#setContainer(org.apache.catalina.Container)
   */
  public void setContainer(Container container) {
  
    /*if (!(container instanceof Context)) {
      throw new IllegalArgumentException("Kein Context");
    }
    super.setContainer(container);
    this.context = (Context) container;*/
    this.container = container;
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#getInfo()
   */
  public String getInfo() {
    return "MOAID";
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#addPropertyChangeListener(java.beans.PropertyChangeListener)
   */
  public void addPropertyChangeListener(PropertyChangeListener arg0) {
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#authenticate(java.lang.String, java.lang.String)
   */
  public Principal authenticate(String artifact, String notUsedParameter) {
    
    log("MOAIDRealm Authentifizierung");
    
    log("RoleVerificator: " + roleVerificator_);
    
    if (roleVerificator_ == null) {
      try {
        start();
      } catch(Exception err) {
        return null;
      }
      if (roleVerificator_ == null) {
        return null;
      }
    }
    log("ok going to fetch data.");
    try {
      // connect to moa-id and retrieve information
      MOAIDClient client = ClientFactory.getMOAIDClient();
      client.setSAMLArtifact(artifact);
      
      // ssl moa-id verbindung
      if (sslstore_ != null &&
          !"".equals(sslstore_)) {
        log("Setze SSL Optionen: " + sslstore_ + "/" + sslpwd_);
        client.setSSLTrustStore(sslstore_, sslpwd_, sslstoretype_);
      }
      
      MOAIDPrincipal moaidPrincipal = new MOAIDPrincipal(this, null, null, new ArrayList());
      
      log("Fetche Authentifizierungsdaten von MOA-ID");
      
      MOAIDPrincipal data = client.getAuthenticationData(moaidurl_);
      
      log("Prinicipal null ? " + (data == null));
      
      if (data == null) {
        return null;
      }
      
      log("Anmeldedaten:");
      log("NameIdentifier: " + data.getNameIdentifier());
      log("NameQualifier: " + data.getNameQualifier());
      
      log("Hole und update roles");
      
      roleVerificator_.updateRoles(data);
      
      // update auto-roles
      addAutoRoles(data);
      
      log("Anzahl der gefundenen Roles: " + data.getRoles().length);
      log("Roles: " + Arrays.asList(data.getRoles()).toString());
      
      if (data.getRoles().length == 0) {
        return null;
      }
      
      data.setRealm(this);
      
      return data;
    } catch(Exception e) {
      e.printStackTrace();
      return null;
    }
    
    
  }

  private void addAutoRoles(MOAIDPrincipal principal) {
    String[] currentRoles = principal.getRoles();
    
    Vector all = new Vector();
    
    // Auto Roles
    all.addAll(autoRoles_);
    
    if (currentRoles != null) {
      
      for (int i=0; i<currentRoles.length; i++) {
        all.add(currentRoles[i]);
      }
    }
    String[] newRoles = new String[all.size()];
    all.copyInto(newRoles);
    principal.setRoles(newRoles);
  }
  
  public void setMOAIDUrl(String url) {
    moaidurl_ = url;
  }
  
  public void setMOAIDSSLStore(String url) {
    sslstore_ = url;
  }
  
  public void setMOAIDSSLStoreType(String url) {
    sslstoretype_ = url;
  }

  public void setMOAIDSSLPassword(String url) {
    sslpwd_ = url;
  }

  
  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#authenticate(java.lang.String, byte[])
   */
  public Principal authenticate(String arg0, byte[] arg1) {
    return null;
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#authenticate(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
   */
  public Principal authenticate(String arg0, String arg1, String arg2, String arg3, String arg4,
      String arg5, String arg6, String arg7) {
    return null;
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#authenticate(java.security.cert.X509Certificate[])
   */
  public Principal authenticate(X509Certificate[] arg0) {
    return null;
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#hasRole(java.security.Principal, java.lang.String)
   */
  public boolean hasRole(Principal principal, String role) {
    if (principal instanceof MOAIDPrincipal) {
      GenericPrincipal p = (GenericPrincipal)principal;
      return p.hasRole(role);
    } else {
      return false;
    }
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.Realm#removePropertyChangeListener(java.beans.PropertyChangeListener)
   */
  public void removePropertyChangeListener(PropertyChangeListener arg0) {
  }
  
  /**
   * Return the global JNDI name of the <code>UserDatabase</code> resource
   * we will be using.
   */
  public String getResourceName() {

      return resourceName;

  }


  /**
   * Set the global JNDI name of the <code>UserDatabase</code> resource
   * we will be using.
   *
   * @param resourceName The new global JNDI name
   */
  public void setResourceName(String resourceName) {

      this.resourceName = resourceName;

  }
  
  public synchronized void start() throws LifecycleException {
    log("Starte MOAID Realm");
    try {
        StandardServer server = (StandardServer) ServerFactory.getServer();
        javax.naming.Context context = server.getGlobalNamingContext();
        log("Fetche RoleVerificator: " + resourceName);
        roleVerificator_ = (RoleVerificator) context.lookup(resourceName);
        log("Role Verificator null?: " + (roleVerificator_ == null));
    } catch (Throwable e) {
        e.printStackTrace();
        //log(sm.getString("userDatabaseRealm.lookup", resourceName), e);
        //roleVerificator_ = null;
    }
    if (roleVerificator_ == null) {
        throw new LifecycleException
            ("Kein RoleVerificator");
    }

    // Perform normal superclass initialization
    //super.start();

  }

  public synchronized void stop() throws LifecycleException {

    // Perform normal superclass finalization
    super.stop();

    // Release reference to our verificator
    //roleVerificator_ = null;

}
  
  /* (non-Javadoc)
   * @see org.apache.catalina.realm.RealmBase#getName()
   */
  protected String getName() {
    return null;
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.realm.RealmBase#getPassword(java.lang.String)
   */
  protected String getPassword(String arg0) {
    return null;
  }

  /* (non-Javadoc)
   * @see org.apache.catalina.realm.RealmBase#getPrincipal(java.lang.String)
   */
  protected Principal getPrincipal(String arg0) {
    return null;
  }
  
  /**
   * @return Returns the autoRoles_.
   */
  public List getAutoRoles() {
    return autoRoles_;
  }
  /**
   * @param autoRoles_ The autoRoles_ to set.
   */
  public void setAutoRoles(List autoRoles) {
    this.autoRoles_ = autoRoles;
  }
}
