package asit.moa.moaid.manager;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import asit.moa.moaid.Citizen;


public class JDBCRoleManager implements RoleManager {

  protected Connection connection_ = null;
  
  public JDBCRoleManager(String jdbcURL) throws RoleManagerException {
    try {
      connection_ = DriverManager.getConnection(jdbcURL);
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  public List getRolesByCitizen(int citizenId) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT role_id FROM citizen_roles WHERE citizen_id=?");
      st.setInt(1, citizenId);
      ResultSet rs = st.executeQuery();
      Vector ret = new Vector();
      if (rs.next()) {
        ret.add(getRoleById(rs.getInt("role_id")));
      }
      return ret;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  public String getRoleById(int id) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT role_name FROM roles WHERE role_id=?");
      st.setInt(1, id);
      ResultSet rs = st.executeQuery();
      if (rs.next()) {
        return rs.getString("role_name");
      }
      return null;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  
  public int getRoleId(String roleName) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT role_id FROM roles WHERE role_name=?");
      st.setString(1, roleName);
      ResultSet rs = st.executeQuery();
      if (!rs.next()) {
        return -1;
      }
      return rs.getInt("role_id");
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  public List getRoleCitizensCommon(String roleName, boolean isProfile) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT citizen_id FROM " + getCommonTableName(isProfile) + " WHERE " + getCommonRowName(isProfile) + "=?");
      st.setInt(1, getCommonId(roleName, isProfile));
      ResultSet rs = st.executeQuery();
      Vector ret = new Vector();
      while (rs.next()) {
        ret.add(new Integer(rs.getInt("citizen_id")));
      }
      return ret;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  public List getCitizensByCommonName(String roleName, boolean isProfile) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT citizen_id FROM " + getCommonTableName(isProfile) + " WHERE " + getCommonRowName(isProfile) +"=?");
      st.setInt(1, getCommonId(roleName, isProfile));
      ResultSet rs = st.executeQuery();
      Vector ret = new Vector();
      while (rs.next()) {
        ret.add(getCitizen(rs.getInt("citizen_id")));
      }
      return ret;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  
  
  public List getCommonCitizensExclude(String roleName, boolean isProfile) throws RoleManagerException {
    List allCitizens = getCitizens();
    List roles = getCitizensByCommonName(roleName, isProfile);
    Vector ret = new Vector();
    for (Iterator it = allCitizens.iterator(); it.hasNext();) {
      Object next = it.next();
      if (!roles.contains(next)) {
        ret.add(next);
      }
    }
    return ret;
  }
  
  public int getProfileId(String roleName) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT profile_id FROM profiles WHERE profile_name=?");
      st.setString(1, roleName);
      ResultSet rs = st.executeQuery();
      if (!rs.next()) {
        return -1;
      }
      return rs.getInt("profile_id");
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  public int getCitizenId(Citizen citizen) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT citizen_id FROM citizen WHERE given_name=? AND family_name=? and date_of_birth=?");
      st.setString(1, citizen.getGivenName());
      st.setString(2, citizen.getFamilyName());
      st.setDate(3, new java.sql.Date(citizen.getDateOfBirth().getTime()));
      ResultSet rs = st.executeQuery();
      if (!rs.next()) {
        return -1;
      }
      return rs.getInt("citizen_id");
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  public int getCitizenId(String bpk) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT citizen_id FROM bpk WHERE bpk=?");
      st.setString(1, bpk);
      ResultSet rs = st.executeQuery();
      if (!rs.next()) {
        return -1;
      }
      return rs.getInt("citizen_id");
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  private String getCommonTableName(boolean isProfile) {
    return isProfile ? "citizen_profiles" : "citizen_roles";
  }
  
  private String getCommonRowName(boolean isProfile) {
    return isProfile ? "profile_id" : "role_id";
  }
  
  private int getCommonId(String str, boolean isProfile) throws RoleManagerException {
    return isProfile ? getProfileId(str) : getRoleId(str); 
  }
  
  /*
   * @see asit.moa.moaid.manager.RoleManager#addCitizenRole(org.apache.catalina.Role, java.lang.String, java.lang.String, java.util.Date)
   */
  public void addCommon(String role, Citizen citizen, boolean isProfile)
      throws RoleManagerException {
    
    int roleId = getCommonId(role, isProfile); 
    if (roleId == -1) {
      throw new RoleManagerException("Eintrag existiert nicht.");
    }
    int citizenId = getCitizenId(citizen); 
    if (citizenId == -1) {
      throw new RoleManagerException("Citizen existiert nicht.");
    }

    try {
      PreparedStatement st = connection_.prepareStatement("INSERT INTO " + getCommonTableName(isProfile) + " (citizen_id," + getCommonRowName(isProfile) +") VALUES (?,?)");
      st.setInt(1, citizenId);
      st.setInt(2, roleId);
      st.execute();
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
    
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenRole(org.apache.catalina.Role, java.lang.String, java.lang.String, java.util.Date)
   */
  public void removeCommon(String role, Citizen citizen, boolean isProfile)
      throws RoleManagerException {
    
    int roleId = getCommonId(role, isProfile); 
    if (roleId == -1) {
      throw new RoleManagerException("Eintrag existiert nicht.");
    }
    int citizenId = getCitizenId(citizen); 
    if (citizenId == -1) {
      throw new RoleManagerException("Citizen existiert nicht.");
    }

    try {
      // lsche aus citizen - tabelle
      PreparedStatement st = connection_.prepareStatement("DELETE FROM " +  getCommonTableName(isProfile) + " WHERE citizen_id = ? AND " + getCommonRowName(isProfile) + "= ?");
      st.setInt(1, citizenId);
      st.setInt(2, roleId);
      st.execute();
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenRoles(org.apache.catalina.Role)
   */
  public void removeCommon(String role, boolean isProfile) throws RoleManagerException {
    
    int roleId =  getCommonId(role, isProfile); 
    if (roleId == -1) {
      throw new RoleManagerException("Eintrag existiert nicht.");
    }

    try {
      // lsche aus citizen - tabelle
      PreparedStatement st = connection_.prepareStatement("DELETE FROM " + getCommonTableName(isProfile) + " WHERE " + getCommonRowName(isProfile) + " = ?");
      st.setInt(1, roleId);
      st.execute();
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenRoles(java.lang.String, java.lang.String, java.util.Date)
   */
  public void removeCommon(Citizen citizen, boolean isProfile) throws RoleManagerException {
    
    int citizenId = getCitizenId(citizen); 
    if (citizenId == -1) {
      throw new RoleManagerException("Citizen existiert nicht.");
    }

    try {
      // lsche aus citizen - tabelle
      PreparedStatement st = connection_.prepareStatement("DELETE FROM " + getCommonTableName(isProfile) + " WHERE citizen_id = ?");
      st.setInt(1, citizenId);
      st.execute();
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#addCitizenProfile(org.apache.catalina.Role, java.lang.String, java.lang.String, java.util.Date)
   */
  public void addCitizenProfile(String role, Citizen citizen)
      throws RoleManagerException {
    addCommon(role, citizen, true);
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenProfile(org.apache.catalina.Role, java.lang.String, java.lang.String, java.util.Date)
   */
  public void removeCitizenProfile(String role, Citizen citizen)
      throws RoleManagerException {
    removeCommon(role, citizen, true);
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenProfiles(org.apache.catalina.Role)
   */
  public void removeCitizenProfiles(String role) throws RoleManagerException {
    removeCommon(role, true);
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenProfiles(java.lang.String, java.lang.String, java.util.Date)
   */
  public void removeCitizenProfiles(Citizen citizen) throws RoleManagerException {
    removeCommon(citizen, true);
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#addBPK(java.lang.String, java.lang.String, java.lang.String, java.util.Date)
   */
  public void addBPK(String base64BPK, Citizen citizen)
      throws RoleManagerException {
    

    int citizenId = getCitizenId(citizen); 
    if (citizenId == -1) {
      throw new RoleManagerException("Citizen existiert nicht.");
    }

    int bpkId = getCitizenId(base64BPK); 
    if (bpkId != -1) {
      throw new RoleManagerException("BPK existiert bereits.");
    }

    
    try {
      PreparedStatement st = connection_.prepareStatement("INSERT INTO bpk (citizen_id,bpk) VALUES (?,?)");
      st.setInt(1, citizenId);
      st.setString(2, base64BPK);
      st.execute();
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeBPK(java.lang.String)
   */
  public void removeBPK(String base64BPK) throws RoleManagerException {
    
    int citizenId = getCitizenId(base64BPK); 
    if (citizenId == -1) {
      throw new RoleManagerException("BPK existiert nicht.");
    }

    try {
      // lsche aus citizen - tabelle
      PreparedStatement st = connection_.prepareStatement("DELETE FROM bpk WHERE bpk = ?");
      st.setString(1, base64BPK);
      st.execute();
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#addCitizen(java.lang.String, java.lang.String, java.util.Date)
   */
  public void addCitizen(Citizen citizen)
      throws RoleManagerException {

    int citizenId = getCitizenId(citizen); 
    if (citizenId != -1) {
      throw new RoleManagerException("Citizen existiert bereits.");
    }

    try {
      PreparedStatement st = connection_.prepareStatement("INSERT INTO citizen (given_name,family_name,date_of_birth) VALUES (?,?,?)");
      st.setString(1, citizen.getGivenName());
      st.setString(2, citizen.getFamilyName());
      st.setDate(3, new java.sql.Date(citizen.getDateOfBirth().getTime()));
      st.execute();
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizen(java.lang.String, java.lang.String, java.util.Date)
   */
  public void removeCitizen(Citizen citizen)
      throws RoleManagerException {
   
    int citizenId = getCitizenId(citizen); 
    if (citizenId == -1) {
      throw new RoleManagerException("Citizen existiert nicht.");
    }

    try {
      // lsche aus citizen - tabelle
      PreparedStatement st = connection_.prepareStatement("DELETE FROM citizen WHERE citizen_id = ?");
      st.setInt(1, citizenId);
      st.execute();

      // lsche aus bpk
      st = connection_.prepareStatement("DELETE FROM bpk WHERE citizen_id = ?");
      st.setInt(1, citizenId);
      st.execute();

      // lsche aus citizen roles
      st = connection_.prepareStatement("DELETE FROM citizen_roles WHERE citizen_id = ?");
      st.setInt(1, citizenId);
      st.execute();
      
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
    
  }
  
  public void removeCitizen(int citizenId)
  throws RoleManagerException {

    try {
      // lsche aus citizen - tabelle
      PreparedStatement st = connection_.prepareStatement("DELETE FROM citizen WHERE citizen_id = ?");
      st.setInt(1, citizenId);
      st.execute();
    
      // lsche aus bpk
      st = connection_.prepareStatement("DELETE FROM bpk WHERE citizen_id = ?");
      st.setInt(1, citizenId);
      st.execute();
    
      // lsche aus citizen roles
      st = connection_.prepareStatement("DELETE FROM citizen_roles WHERE citizen_id = ?");
      st.setInt(1, citizenId);
      st.execute();
      
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#getCitizens()
   */
  public List getCitizens() throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT * FROM citizen ORDER BY family_name, given_name, date_of_birth");
      ResultSet rs = st.executeQuery();
      
      Vector ret = new Vector();
      while (rs.next()) {
        ret.add(new Citizen(rs.getString("given_name"), rs.getString("family_name"), rs.getDate("date_of_birth")));
      }
      return ret;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#addRole(org.apache.catalina.Role)
   */
  public void addSingle(String role, boolean isProfile) throws RoleManagerException {
    
    int roleId = getCommonId(role, isProfile); 
    if (roleId != -1) {
      throw new RoleManagerException("Eintrag existiert bereits.");
    }

    try {
      PreparedStatement st = connection_.prepareStatement("INSERT INTO " + getSingleTable(isProfile) + " (" + getSingleRow(isProfile) + ") VALUES (?)");
      st.setString(1, role);
      st.execute();
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeRole(org.apache.catalina.Role)
   */
  public void removeSingle(String role, boolean isProfile) throws RoleManagerException {
    
    int roleId = getCommonId(role, isProfile); 
    if (roleId == -1) {
      throw new RoleManagerException("Eintrag existiert nicht.");
    }

    try {
      
      // loesche alle gemeinsamen
      removeCommon(role, isProfile);
      
      // lsche roles
      PreparedStatement st = connection_.prepareStatement("DELETE FROM " + getSingleTable(isProfile) + " WHERE " + getSingleRow(isProfile) + " = ?");
      st.setString(1, role);
      st.execute();
      
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  public String getSingleTable(boolean isProfile) {
    return isProfile ? "profiles" : "roles";
  }
  
  public String getSingleRow(boolean isProfile) {
    return isProfile ? "profile_name" : "role_name";
  }
  
  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#getRoles()
   */
  public List getSingle(boolean isProfile) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT * FROM " + getSingleTable(isProfile) + " ORDER BY " + getSingleRow(isProfile));
      ResultSet rs = st.executeQuery();
      
      Vector ret = new Vector();
      while (rs.next()) {
        ret.add(rs.getString(getSingleRow(isProfile)));
      }
      return ret;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  public List getBPKs() throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT * FROM bpk");
      ResultSet rs = st.executeQuery();
      
      Vector ret = new Vector();
      while (rs.next()) {
        ret.add(rs.getString("bpk"));
      }
      return ret;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }
  
  public Citizen getCitizen(int id) throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT * FROM citizen WHERE citizen_id=?");
      st.setInt(1, id);
      ResultSet rs = st.executeQuery();
      
      Vector ret = new Vector();
      if (rs.next()) {
        return new Citizen(rs.getString("given_name"), rs.getString("family_name"), rs.getDate("date_of_birth")); 
      }
      return null;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  public List getBPKCitizens() throws RoleManagerException {
    try {
      PreparedStatement st = connection_.prepareStatement("SELECT * FROM bpk");
      ResultSet rs = st.executeQuery();
      
      Vector ret = new Vector();
      while (rs.next()) {
        ret.add(new Integer(rs.getInt("citizen_id")));
      }
      return ret;
    } catch(Exception e) {
      throw new RoleManagerException(e);
    }
  }

  
  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#addAuthorityProfile(java.lang.String)
   */
  public void addAuthorityProfile(String profileName) throws RoleManagerException {
    addSingle(profileName, true);
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeAuthorityProfile(java.lang.String)
   */
  public void removeAuthorityProfile(String profileName) throws RoleManagerException {
    removeSingle(profileName, true);
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#getAuthorityProfiles()
   */
  public List getAuthorityProfiles() throws RoleManagerException {
    return getSingle(true);
  }

  

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#addCitizenRole(java.lang.String, asit.moa.moaid.Citizen)
   */
  public void addCitizenRole(String role, Citizen citizen) throws RoleManagerException {
    addCommon(role, citizen, false);
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenRole(java.lang.String, asit.moa.moaid.Citizen)
   */
  public void removeCitizenRole(String role, Citizen citizen) throws RoleManagerException {
    removeCommon(role, citizen, false);
    
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenRoles(java.lang.String)
   */
  public void removeCitizenRoles(String role) throws RoleManagerException {
    removeCommon(role, false);
    
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeCitizenRoles(asit.moa.moaid.Citizen)
   */
  public void removeCitizenRoles(Citizen citizen) throws RoleManagerException {
    removeCommon(citizen, false);
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#addRole(java.lang.String)
   */
  public void addRole(String role) throws RoleManagerException {
    addSingle(role, false);
    
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#removeRole(java.lang.String)
   */
  public void removeRole(String role) throws RoleManagerException {
    removeSingle(role, false);
    
  }

  /* (non-Javadoc)
   * @see asit.moa.moaid.manager.RoleManager#getRoles()
   */
  public List getRoles() throws RoleManagerException {
    return getSingle(false);
  }
  
  public static void main(String[] args) throws Exception {
    /*Class.forName("org.postgresql.Driver");
    JDBCRoleManager m = new JDBCRoleManager("jdbc:postgresql://althaia.iaik.at/moaid?user=pkimodule&password=pkimodule");
    System.out.println(m.getRoleId("webbescheide"));
    Date date = new Date();
    date.setYear(79);
    date.setMonth(7);
    date.setDate(21);
    //m.addRole("test");
    List c = m.getCitizens();
    List r = m.getCitizensByRoleName("admin");
    List s = m.getRoleCitizensExclude("admin");
    System.out.println(c);
    System.out.println(r);
    System.out.println(s);*/
  }
}
