package asit.moa.moazs;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.sf.hibernate.HibernateException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;
import asit.common.Utils;
import asit.moa.client.ClientFactory;
import asit.moa.client.deliver.DeliveryTypes;
import asit.moa.client.deliver.moazs.DeliveryResponse;
import asit.moa.client.deliver.moazs.ErrorDeliveryResponse;
import asit.moa.client.deliver.moazs.MOAZSClient;
import asit.moa.client.deliver.moazs.PartialSuccessDeliveryResponse;
import asit.moa.client.deliver.moazs.ResponseTypes;
import asit.moa.client.deliver.moazs.SuccessDeliveryResponse;
import asit.moa.moazs.notifier.Config;
import asit.moa.moazs.notifier.ProtocolEntry;
import asit.moa.moazs.notifier.UtilsConfig;
import asit.moa.moazs.notifier.UtilsProtocolEntry;
import asit.moa.moazs.notifier.dao._RootDAO;


public class MOAZSWrapper extends HttpServlet {
  
  static Log logger_ = LogFactory.getLog(MOAZSWrapper.class.getName());
  
  protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException,
      IOException {
    doPost(request, response);
  }
  
  protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException,
      IOException {
    
    // Buffer request
    logger_.debug("Reading MOAZS-Request.");
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    int ch;
    if (request.getContentLength() != -1) {
      for (int i=0; i<request.getContentLength() && ((ch = request.getInputStream().read()) != -1); i++) {
        out.write(ch);
      }
    } else {
      while ((ch = request.getInputStream().read()) != -1) {
        out.write(ch);
      }
    }
    logger_.debug("Fetching URL");
    String url = null;
    // Get MOA-ZS Connection URL
    try {
      _RootDAO.initialize();
      Config config = UtilsConfig.get("moazsurl");
      if (config != null) {
        url = config.getValue();
      }
    } catch(HibernateException e) {
      e.printStackTrace();
    }
    logger_.debug("URL is: " + url);    
    try {
      // now connect with moazs client and send raw data
      if (url == null) {
        // liefere fehlermeldung 504 (Wrong configuration)
        ErrorDeliveryResponse errorResposne = new ErrorDeliveryResponse("", "", "504", "MOAZSWrapper Internal Configuration Error: no MOA-ZS URL specified.", null);
        Element resp = Utils.packIntoSoap(errorResposne.getElement());
        response.setContentType("text/xml");
        Utils.serializeElement(resp, response.getOutputStream());
        return;
      }
      
      MOAZSClient client = (MOAZSClient)ClientFactory.getDeliveryClient(DeliveryTypes.MOAZS);
      DeliveryResponse deliveryResponse = client.sendDeliveryRequest(url, new ByteArrayInputStream(out.toByteArray()));
      
      if (deliveryResponse == null) {
        response.sendError(500, "Internal Server Error - MOAZS Client Null Response");
        return;
      }
      
      // now decide based on response
      
      // Error - Wird direkt an Client weitergeleitet
      if (ResponseTypes.ERROR.equals(deliveryResponse.getType())) {
        Element resp = Utils.packIntoSoap(deliveryResponse.getElement());
        response.setContentType("text/xml");
        Utils.serializeElement(resp, response.getOutputStream());
        return;
      }
      
      // Success - oder PartialSuccess: Warten auf Datenbankantwort
      
      int seconds = Integer.MAX_VALUE;
      try {
        Config cfg = UtilsConfig.get("asyncto");
        seconds = Integer.parseInt(cfg.getValue());
      } catch(Exception e) {
        e.printStackTrace();
      }
      // ein 0 Timeout Wert lsst einen maximalen Timeout wert zu
      if (seconds == 0) {
        seconds = Integer.MAX_VALUE;
      }
      boolean hasAnswer = false;
      for (int i=0; i<seconds && !hasAnswer; i++) {
        logger_.debug("Try to fetch entry with mzsid " + deliveryResponse.getMZSDeliveryID());
        ProtocolEntry e = null;
        try { e= UtilsProtocolEntry.getProtocolEntry(deliveryResponse.getMZSDeliveryID()); } catch(Exception err) { err.printStackTrace(); }
                
        if (e != null) {
          
          // update fetched status
          try {
            e.setFetched(1);
            UtilsProtocolEntry.addProtocolEntry(e);
          } catch(Exception err) {
            err.printStackTrace();
          }
          
          hasAnswer = true;
          
          DeliveryResponse r =  null;
          if (ResponseTypes.SUCCESS.equals(e.getStatus())) {
            r = new SuccessDeliveryResponse(e.getAppId(), e.getMoazsId()); 
          } else {
            r = new ErrorDeliveryResponse(e.getAppId(), e.getMoazsId(), e.getErrorCode(), e.getErrorMessage(), e.getErrorFileName());  
          }
          
          Element toSend = Utils.packIntoSoap(r.getElement());
          response.setContentType("text/xml");
          Utils.serializeElement(toSend, response.getOutputStream());
          return;
          
        } else {
          
          // try to fetch each second
          Thread.sleep(1000);
        }
      }

      // send a pending
      DeliveryResponse r = new PartialSuccessDeliveryResponse(deliveryResponse.getAppDeliveryID(), deliveryResponse.getMZSDeliveryID());
      Element toSend = Utils.packIntoSoap(r.getElement());
      response.setContentType("text/xml");
      Utils.serializeElement(toSend, response.getOutputStream());
    } catch(Exception e) {
      response.sendError(500, "Internal Server Error - " + e.getMessage());
    }
  }
}
