package asit.verifier.paper;

import iaik.ixsil.algorithms.CanonicalizationAlgorithmImplCanonicalXML;
import iaik.ixsil.algorithms.DigestAlgorithmImplSHA1;
import iaik.ixsil.algorithms.SignatureAlgorithmImplRSA;
import iaik.ixsil.algorithms.Transform;
import iaik.ixsil.algorithms.TransformImplCanonicalXMLWithComments;
import iaik.ixsil.algorithms.TransformImplEnvelopedSignature;
import iaik.ixsil.core.Position;
import iaik.ixsil.core.Signer;
import iaik.ixsil.core.SignerReference;
import iaik.ixsil.core.SignerSignature;
import iaik.ixsil.core.SignerSignedInfo;
import iaik.ixsil.init.IXSILConstants;
import iaik.ixsil.init.IXSILInit;
import iaik.ixsil.keyinfo.KeyManagerImpl;
import iaik.ixsil.keyinfo.x509.KeyProviderImplX509Data;
import iaik.ixsil.keyinfo.x509.X509Data;
import iaik.ixsil.util.URI;
import iaik.security.rsa.RSAPrivateKey;
import iaik.x509.X509Certificate;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.StringReader;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;
import org.apache.fop.apps.Driver;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import asit.common.Utils;
import asit.moa.MoaConstants;
import asit.moa.client.util.MoaUtils;
import asit.moa.client.verify.VerifyResult;
import asit.moa.client.verify.VerifySignatureClient;


public class PaperNotification {
  
  private String senderName;
  
  private String recipientName;
  private String recipientAdressLine;
  private String recipientPostalCode;
  private String municipality;
  
  private String deliveryId;
  private String deliveryQuality;
  //private String deliveryNumber;
  private String deliveryServer;
  
  private Date firstNotification;
  
  public String getBase64SignatureValue() {
    return base64SignatureValue;
  }
  public void setBase64SignatureValue(String base64SignatureValue) {
    this.base64SignatureValue = base64SignatureValue;
  }
  public String getSigningTime() {
    return signingTime;
  }
  public void setSigningTime(String signingTime) {
    this.signingTime = signingTime;
  }
  //private Date secondNotification;
  
  private String signingTime;
  private String base64SignatureValue;
  
  private VerifySignatureClient client;
  private RSAPrivateKey key;
  private X509Certificate cert;

  private Date dueDate;
  
  
  public X509Certificate getCert() {
    return cert;
  }
  public void setCert(X509Certificate cert) {
    this.cert = cert;
  }
  public RSAPrivateKey getKey() {
    return key;
  }
  public void setKey(RSAPrivateKey key) {
    this.key = key;
  }
  public VerifySignatureClient getClient() {
    return client;
  }
  public void setClient(VerifySignatureClient client) {
    this.client = client;
  }
  
  
  
  public VerifyResult verifySignature(String url) throws PaperNotificationException {
    
    try {
      Element signedNotification = getSignedElement();
      client.setContent(signedNotification);
      client.addAdditionalNamespace("dsig", MoaConstants.DSIG_NS);
      client.setSignatureLocation("//dsig:Signature");
      return client.sendVerifyRequest(url);
      
    } catch(Exception e) {
      throw new PaperNotificationException(e);
    }
  }
  
  public InputStream getPDFNotification(InputStream foxsl) throws PaperNotificationException {

//    System.out.println("IS: " + foxsl);
    try {
      ByteArrayOutputStream out = new ByteArrayOutputStream();
      ByteArrayOutputStream os = new ByteArrayOutputStream();
      long start = System.currentTimeMillis();
      Utils.serializeElement(getSignedElement(), os);
      Driver driver=new Driver();
      driver.setOutputStream(out);
      driver.setRenderer(Driver.RENDER_PDF);
      Transformer transformer=TransformerFactory.newInstance().newTransformer(new StreamSource(foxsl));
      //transformer.setParameter("page-count","#");
      transformer.transform(new StreamSource(new StringReader(os.toString("ISO-8859-1"))),
        new SAXResult(driver.getContentHandler()));
      long end = System.currentTimeMillis();
//      System.out.println(end-start);
      return new ByteArrayInputStream(out.toByteArray());
    } catch(Exception e) {
      throw new PaperNotificationException(e);
    }
  }
  
  public Element getSignedElement() throws PaperNotificationException {

    try {
      Element toSign = getElement();
      
      //Utils.serializeDocument(toSign.getOwnerDocument(), System.out);
      
      Position sigPosition = new Position("/n:PaperNotification", "n http://reference.e-government.org/namespace/zuse/recipientnotification/10#", 0);
      Signer signer = new Signer(toSign.getOwnerDocument(), new URI(IXSILConstants.DUMMY_ABSURI_), sigPosition); //notification,
  
      SignerSignature signature = signer.getSignature();
      SignerSignedInfo signedInfo = signature.getSignerSignedInfo();
      CanonicalizationAlgorithmImplCanonicalXML c14nAlg = new CanonicalizationAlgorithmImplCanonicalXML();
      signedInfo.setCanonicalizationAlgorithm(c14nAlg);
      SignatureAlgorithmImplRSA signatureAlg = new SignatureAlgorithmImplRSA();

      Transform envelopedSignatureTransform = new TransformImplEnvelopedSignature();
      TransformImplCanonicalXMLWithComments c14nTransform = new TransformImplCanonicalXMLWithComments();
      Transform[] transforms = new Transform[] { envelopedSignatureTransform, c14nTransform };
      
      signatureAlg.setSignerKey(key);
      signedInfo.setSignatureAlgorithm(signatureAlg);
      // Create reference
      SignerReference reference = signedInfo.createReference();
      reference.setId("reference-data-0");
      reference.setURI(new URI(""));
      DigestAlgorithmImplSHA1 digestAlg = new DigestAlgorithmImplSHA1();
      reference.setDigestAlgorithm(digestAlg);
      // set transformations
      if (transforms != null) {
        for (int i = 0; i < transforms.length; i++) {
          reference.insertTransformAt(transforms[i], i);
        }
      }
      signedInfo.addReference(reference);
      
      X509Data x509Data = new X509Data();
      x509Data.insertHintAt(cert, 0);
      KeyManagerImpl keyManager = new KeyManagerImpl(signer.toDocument());
      KeyProviderImplX509Data x509DataKeyProvider = new KeyProviderImplX509Data(signer.toDocument());
      x509DataKeyProvider.insertX509DataAt(x509Data, 0);
      keyManager.addKeyProvider(x509DataKeyProvider);
      signer.getSignature().setKeyManager(keyManager);
      
      Element etsi = MoaUtils.createXAeDS(signingTime, "id-01", "ref-01", cert, signer.toDocument());
      iaik.ixsil.core.Object object = signature.createObject(etsi);
      object.setId("etsi-id");
      signer.getSignature().addObject(object);
      SignerReference etsiRef = signedInfo.createReference();
      etsiRef.setType(new URI("http://uri.etsi.org/01903/v1.1.1#SignedProperties"));
      etsiRef
          .setURI(new URI("#xmlns(etsi=http://uri.etsi.org/01903/v1.1.1%23)%20xpointer(id('"
                          + "etsi-id"
                          + "')/child::etsi:QualifyingProperties/child::etsi:SignedProperties)"));
      etsiRef.setDigestAlgorithm(digestAlg);
      signedInfo.addReference(etsiRef);
      // End etsi-QualifyingProperties
      signer.getSignature().sign();
      Document signedDoc = signer.toDocument();
      Node oldSigValue = XPathAPI.selectSingleNode(signedDoc, "descendant-or-self::dsig:SignatureValue/text()");
      oldSigValue.setNodeValue(base64SignatureValue);
      return signedDoc.getDocumentElement();
    } catch(Exception e) {
      throw new PaperNotificationException(e);
    }
  }

  // erzeuge dokument
  public Element getElement() throws PaperNotificationException {
    try {
      Document doc = Utils.createEmptyDocument(false);
      Element root = doc.createElement("n:PaperNotification");
      root.setAttribute("xmlns:n", "http://reference.e-government.org/namespace/zuse/recipientnotification/10#");
      root.setAttribute("xmlns:dsig", "http://www.w3.org/2000/09/xmldsig#");
      
      Element sender = doc.createElement("n:Sender");
      root.appendChild(sender);
      sender.appendChild(doc.createElement("n:Name")).appendChild(doc.createTextNode(senderName));
      
      Element recipient = doc.createElement("n:Recipient");
      
      recipient.appendChild(doc.createElement("n:Name")).appendChild(doc.createTextNode(recipientName));
      recipient.appendChild(doc.createElement("n:AddressLine")).appendChild(doc.createTextNode(recipientAdressLine));
      recipient.appendChild(doc.createElement("n:PostalCode")).appendChild(doc.createTextNode(recipientPostalCode));
      recipient.appendChild(doc.createElement("n:Municipality")).appendChild(doc.createTextNode(municipality));
      root.appendChild(recipient);
      
      root.appendChild(doc.createElement("n:DeliveryID")).appendChild(doc.createTextNode(deliveryId));
      root.appendChild(doc.createElement("n:DeliveryQuality")).appendChild(doc.createTextNode(deliveryQuality));
      //root.appendChild(doc.createElement("n:DeliveryNumber")).appendChild(doc.createTextNode(deliveryNumber));
      root.appendChild(doc.createElement("n:DeliveryServer")).appendChild(doc.createTextNode(deliveryServer));
      
      SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
      
      Element dates = doc.createElement("n:Dates");
      dates.appendChild(doc.createElement("n:FirstNotification")).appendChild(doc.createTextNode(format.format(firstNotification)));
      //dates.appendChild(doc.createElement("n:SecondNotification")).appendChild(doc.createTextNode(format.format(secondNotification)));
      dates.appendChild(doc.createElement("n:DueDate")).appendChild(doc.createTextNode(format.format(dueDate)));
      root.appendChild(dates);
      doc.appendChild(root);
      
      // make namespace aware
      ByteArrayOutputStream out = new ByteArrayOutputStream();
//      Utils.serializeElement(root, System.out);
      Utils.serializeElement(root, out);
      return Utils.readDocFromIsNS(new StringReader(out.toString("ISO-8859-1"))).getDocumentElement();
    } catch(Exception e) {
      throw new PaperNotificationException(e);
    }
    
  }
  
  public String getDeliveryId() {
    return deliveryId;
  }
  public void setDeliveryId(String deliveryId) {
    this.deliveryId = deliveryId;
  }
//  public String getDeliveryNumber() {
//    return deliveryNumber;
//  }
//  public void setDeliveryNumber(String deliveryNumber) {
//    this.deliveryNumber = deliveryNumber;
//  }
  public String getDeliveryQuality() {
    return deliveryQuality;
  }
  public void setDeliveryQuality(String deliveryQuality) {
    this.deliveryQuality = deliveryQuality;
  }
  public String getDeliveryServer() {
    return deliveryServer;
  }
  public void setDeliveryServer(String deliveryServer) {
    this.deliveryServer = deliveryServer;
  }
  public Date getDueDate() {
    return dueDate;
  }
  public void setDueDate(Date dueDate) {
    this.dueDate = dueDate;
  }
  public Date getFirstNotification() {
    return firstNotification;
  }
  public void setFirstNotification(Date firstNotification) {
    this.firstNotification = firstNotification;
  }
  public String getMunicipality() {
    return municipality;
  }
  public void setMunicipality(String municipality) {
    this.municipality = municipality;
  }
  public String getRecipientAdressLine() {
    return recipientAdressLine;
  }
  public void setRecipientAdressLine(String recipientAdressLine) {
    this.recipientAdressLine = recipientAdressLine;
  }
  public String getRecipientName() {
    return recipientName;
  }
  public void setRecipientName(String recipientName) {
    this.recipientName = recipientName;
  }
  public String getRecipientPostalCode() {
    return recipientPostalCode;
  }
  public void setRecipientPostalCode(String recipientPostalCode) {
    this.recipientPostalCode = recipientPostalCode;
  }
//  public Date getSecondNotification() {
//    return secondNotification;
//  }
//  public void setSecondNotification(Date secondNotification) {
//    this.secondNotification = secondNotification;
//  }
  public String getSenderName() {
    return senderName;
  }
  public void setSenderName(String senderName) {
    this.senderName = senderName;
  }
  
  public static void main(String[] args) throws Exception {
    IXSILInit.init(new URI("file:c:/development/ixsil/init/properties/init.properties"));
    PaperNotification n = new PaperNotification();
    n.setCert(new X509Certificate(new FileInputStream("c:/at_crypt.cer")));
    KeyPairGenerator gen = KeyPairGenerator.getInstance("RSA", "IAIK");
    gen.initialize(1024);
    PrivateKey pk = gen.generateKeyPair().getPrivate();
    RSAPrivateKey key = (RSAPrivateKey)pk;
    
    System.setProperty("file.encoding", "UTF-8");
    
    n.setKey(key);
    n.setSenderName("Arne Tuber");
    n.setFirstNotification(new Date());
    //n.setSecondNotification(new Date());
    n.setDeliveryId("12345");
    //n.setDeliveryNumber("76");
    n.setDeliveryQuality("RSa");
    n.setDeliveryServer("http://www.zustellung.gv.at");
    n.setDueDate(new Date());
    n.setMunicipality("Graz");
    n.setRecipientAdressLine("Inffeldgasse 16/A");
    n.setRecipientName("Arne Tauber");
    n.setRecipientPostalCode("8010");
    n.setSigningTime("21-08-1979");
    
    //Utils.serializeElement(n.getSignedElement(), System.out);
    InputStream is = n.getPDFNotification(new FileInputStream("c:/temp/postkarte.xsl"));
    int ch;
    FileOutputStream out = new FileOutputStream("c:/temp/postkarte.pdf");
    while((ch = is.read()) != -1) {
      out.write(ch);
    }
    out.close();
    
  }
  
}
